if ( !window.console ) window.console = {};
if ( !window.console.log ) window.console.log = function(){};

window.CONFIG = {
    device: 'desktop'
};

new QWebChannel(qt.webChannelTransport,
    function(channel){
        window.plugin = channel.objects.plugin;
    }
);

var pageKey = "";
var leftFrame;
var rightFrame;
var leftPageIsEmpty;
var rightPageIsEmpty;
var annotatedVersion = false;
var annotated = false;

function replaceCss(inputFrame) {
    var cssSources = inputFrame.contents().find('[href^="css/"][href$=".css"]');
    cssSources.each(function() {
        var oldHref = $(this).attr('href');
        var newHref = '../../OPS/'+oldHref;
        $('<link>').attr({
            href: newHref,
            rel: 'stylesheet'
        }).appendTo($(this).parent());
    });

    $('<link>').attr({
        href: "qrc:/customKitabooAssets/css/icons_replacement.css",
        rel: 'stylesheet'
    }).appendTo(inputFrame.contents().find("head"));

    $('<link>').attr({
        href: "qrc:/customKitabooAssets/css/custom_css.css",
        rel: 'stylesheet'
    }).appendTo(inputFrame.contents().find("head"));
}

function setupMarkupFunctions(inputFrame) {

    var frameContentWindow = inputFrame[0].contentWindow;
    var frameId = inputFrame.attr('id');

    // Deciding which method to call based on popup type
     var manageLoadPopup = function(markupId, type, a, b) {
        if (type === 'audio')
            plugin.openAudio(a, frameId);
        else if (type === 'video')
            plugin.openVideo(a, frameId);
        else if (type === 'autologging')
            plugin.openTokenedHttpLink(a, frameId);
        else if (type === 'secureurl')
            plugin.openHttpLink(a, frameId);
        else if (type === 'image')
            plugin.openImage(a, frameId, b);
        else if (type === 'documents'){
            var isMap = a.endsWith(".sme");
            if (!isMap)
                plugin.openExternalDocument(a, frameId);
            else
                plugin.openMap(a,frameId);
        } else if (type == "mindmap"){
            plugin.openMap(a,frameId);
        } else if (type === "youtube_video"){
            plugin.openYoutubeVideo(a,frameId)
        }
        else
            plugin.openUnsupportedContent(type);
    };

    var manageJumpToPage = function (pageName,markupId) {
        plugin.jumpToPage(pageName);
    }

    var manageHtmlActivity = function(path, markupId, type)
    {
        plugin.openHtml(path, frameId,type);
    }

    frameContentWindow.bt = {
        loadPopup: manageLoadPopup,
        jumpToPage: manageJumpToPage,
        htmlActivity: manageHtmlActivity
    }
}

function fitSingleToWidth() {
    var viewportWidth = window.innerWidth - 16; // 16 sarebbe circa la larghezza della scrollbar della PageView.
    var viewportHeight = window.innerHeight;

    var kitabooPageSize = rightFrame.contents().find('meta[name="viewport"]').attr('content');
    var kitabooPageWidth = kitabooPageSize.split(',')[0].split('width=')[1];

    var widthRatio = viewportWidth / kitabooPageWidth;

    rightFrame.width(viewportWidth);
    rightFrame.height(viewportHeight);

    rightFrame.contents().find('body').css({'-webkit-transform': 'scale('+widthRatio+')', '-webkit-transform-origin': '0 0'});
}

function fitSingleToHeight() {

}

function fitSingleToView() {
    var scale;
    var viewportWidth = window.innerWidth;
    var viewportHeight = window.innerHeight;

    var kitabooPageSize = rightFrame.contents().find('meta[name="viewport"]').attr('content');
    var kitabooPageWidth = kitabooPageSize.split(',')[0].split('width=')[1];
    var kitabooPageHeight = kitabooPageSize.split(',')[1].split('height=')[1];

    var widthRatio = viewportWidth / kitabooPageWidth;
    var heightRatio = viewportHeight / kitabooPageHeight;

    scale = Math.min(widthRatio, heightRatio);

    rightFrame.width(kitabooPageWidth * scale);
    rightFrame.height(kitabooPageHeight * scale);

    rightFrame.contents().find('body').css({'-webkit-transform': 'scale('+scale+')', '-webkit-transform-origin': '0 0', 'margin': '0 auto'});
}


function fitDoubleToWidth() {

}

function fitDoubleToHeight() {

}

function fitDoubleToView() {
    var scale;
    var viewportWidth = window.innerWidth;
    var viewportHeight = window.innerHeight;
    var kitabooPageSize;

    if (leftPageIsEmpty) {
        kitabooPageSize = rightFrame.contents().find('meta[name="viewport"]').attr('content');
    }
    else {
        kitabooPageSize = leftFrame.contents().find('meta[name="viewport"]').attr('content');
    }

    var kitabooPageWidth = kitabooPageSize.split(',')[0].split('width=')[1];
    var kitabooPageHeight = kitabooPageSize.split(',')[1].split('height=')[1];

    var widthRatio = (viewportWidth / 2) / kitabooPageWidth;
    var heightRatio = viewportHeight / kitabooPageHeight;

    scale = Math.min(widthRatio, heightRatio);

    leftFrame.width(kitabooPageWidth * scale);
    leftFrame.height(kitabooPageHeight * scale);
    rightFrame.width(kitabooPageWidth * scale);
    rightFrame.height(kitabooPageHeight * scale);

    leftFrame.contents().find('body').css({'-webkit-transform': 'scale('+scale+')', '-webkit-transform-origin': '0 0'});
    rightFrame.contents().find('body').css({'-webkit-transform': 'scale('+scale+')', '-webkit-transform-origin': '0 0'});
    rightFrame.attr('scrolling', 'no');
    leftFrame.attr('scrolling', 'no');
}

function showAnnotatedVersion(inputFrame)
{
	if(!annotatedVersion)return;
	var sections = inputFrame.contents().find('section');
	if(annotated){
	   $(sections[1]).show();
	   $(sections[0]).hide();
	}else {
	   $(sections[0]).show();
	   $(sections[1]).hide();
	}
}


function setAnnotated(isAnnotated)
{
    annotatedVersion = true;
    annotated = isAnnotated;
    showAnnotatedVersion(rightFrame);
    showAnnotatedVersion(leftFrame);
}

function loadWebPage(page, mode, _pageKey)
{
    pageKey = _pageKey;
    leftFrame = $("#left");
    rightFrame = $("#right");

    rightFrame.attr('src', page);
    var recalcFrameSize = function() {
        console.log('Recalc right frame size')

        if (mode === 'fit_width') {
            fitSingleToWidth();
        }
        else if (mode === 'fit_height') {
            fitSingleToHeight();
        }
        else if (mode === 'fit_page') {
            fitSingleToView();
        }
        else {
            console.log('Unrecognised page view mode.');
        }
    }

    var frameLoaded = function() {
        leftFrame.hide();
        replaceCss(rightFrame);
        setupMarkupFunctions(rightFrame);
        showAnnotatedVersion(rightFrame);
        setTimeout(function(){plugin.domChanged(pageKey,"single");}, 500);
    }

    leftFrame.off("load")
    rightFrame.off("load");
    $(window).off("resize");

    rightFrame.load(function(){
        frameLoaded();
        recalcFrameSize();
    });
    $(window).resize(recalcFrameSize)
}

function loadWebPages(pageLeft, pageRight, mode, _pageKey)
{
    console.log("loadwebpages", _pageKey);
    pageKey = _pageKey;
    leftFrame = $("#left");
    rightFrame = $("#right");

    leftFrame.attr('src', pageLeft);
    rightFrame.attr('src', pageRight);

    leftPageIsEmpty = (pageLeft === "");
    rightPageIsEmpty = (pageRight === "");

    var loaded = 0;

    // cose da fare ogni volta che si ridimensiona la pagina (ricalcolo dimensioni)
    var recalcFramesSize = function () {
        console.log('Recalc frames size')

        if (mode === 'fit_width') {
            fitDoubleToWidth();
        }
        else if (mode === 'fit_height') {
            fitDoubleToHeight();
        }
        else if (mode === 'fit_page') {
            fitDoubleToView();
        }
        else {
            console.log('Unrecognised page view mode.');
        }
    }

    // cose da fare al caricamento della pagina, una volta soltanto.
    var framesLoaded = function(){
        leftFrame.show();
        rightFrame.show();
        replaceCss(leftFrame);
        replaceCss(rightFrame);
        setupMarkupFunctions(leftFrame);
        setupMarkupFunctions(rightFrame);
        showAnnotatedVersion(rightFrame);
        showAnnotatedVersion(leftFrame);

        setTimeout(function(){plugin.domChanged(pageKey,"double");}, 500);
    }

    leftFrame.off("load")
    rightFrame.off("load");
    $(window).off("resize");

    if (leftPageIsEmpty) {
        $('#right').load(function() {
             framesLoaded();
             recalcFramesSize()
        });
    }
    else if (rightPageIsEmpty) {
        $('#left').load(function() {
             framesLoaded();
             recalcFramesSize()
        });
    }
    else {
        $('#left, #right').load(function() {
            if (++loaded === 2) {
                framesLoaded();
                recalcFramesSize()
            }
        });
    }
    $(window).resize(recalcFramesSize);
}
