function hexToRgb(hex) {
    if (hex.indexOf("#")!==0) return hex;

    var shorthandRegex = /^#?([a-f\d])([a-f\d])([a-f\d])$/i;
    hex = hex.replace(shorthandRegex, function(m, r, g, b) {
        return r + r + g + g + b + b;
    });
    var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
    return result ? "rgb("+parseInt(result[1],16)+", "+parseInt(result[2], 16)+", "+parseInt(result[3],16)+")": "rgb(189, 195, 199)";
}

function rgb2rgba(rgb, alpha) {
    var a = (alpha === true) ? 1 : 0.15;
    /* translate the color hex in rgb if needed */
    rgb = hexToRgb(rgb);
    rgb = rgb.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);

    return "rgba("+rgb[1]+", "+rgb[2]+", "+rgb[3]+", "+a+")";
}

function isWhite(rgb){
    rgb = hexToRgb(rgb);
    rgb = rgb.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
    if (rgb[1] == "255" && rgb[2] == "255" && rgb[3] == "255")
        return true;
    return false;
}

function rgba2rgb(rgba){
    if (rgba.indexOf('rgb(')==0){
        rgba = rgba.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
        return "rgb("+rgba[1]+", "+rgba[2]+", "+rgba[3]+")";
    }
    if (rgba.indexOf('rgba(')==0){
        rgba = rgba.match(/^rgba\((\d+),\s*(\d+),\s*(\d+),\s*(.*)\s*\)$/);
        return "rgb("+rgba[1]+", "+rgba[2]+", "+rgba[3]+")";
    }
    return rgba;
}

/*! GetDevicePixelRatio | Author: Tyson Matanich, 2012 | License: MIT */
(function (window) {
    window.getDevicePixelRatio = function () {
        var ratio = 1;
        // To account for zoom, change to use deviceXDPI instead of systemXDPI
        if (window.screen.systemXDPI !== undefined && window.screen.logicalXDPI !== undefined && window.screen.systemXDPI > window.screen.logicalXDPI) {
            // Only allow for values > 1
            ratio = window.screen.systemXDPI / window.screen.logicalXDPI;
        }
        else if (window.devicePixelRatio !== undefined) {
            ratio = window.devicePixelRatio;
        }
        return ratio;
    };
})(this);



if (!String.prototype.endsWith) {
    Object.defineProperty(String.prototype, 'endsWith', {
        value: function(searchString, position) {
            var subjectString = this.toString();
            if (position === undefined || position > subjectString.length) {
                position = subjectString.length;
            }
            position -= searchString.length;
            var lastIndex = subjectString.indexOf(searchString, position);
            return lastIndex !== -1 && lastIndex === position;
        }
    });
}

// serve per retrocompatibilità con le vecchie mappe fatte senza editor evoluto
function getLabelPixelSize(val){
    switch(val){
        case 1: return 14;
        case 2: return 16;
        case 3: return 20;
        case 4: return 24;
        case 5: return 28;
        break;
    }
    return 20; // il default comunque
};

define(["mapController","node","edge"],function(MapController,Node,Edge) {
    /**
     * Creates main Map App Object.
     *
     * @param {jQuery} $_canvas The canvas where draw
     * @constructor
     */
    var App = function ($_canvas) {
        var me = this;
        this.device = App.NATIVE_APP_DEVICE.STANDALONE;
        this.deviceVersion = "";
        this.canView = true;
        this.canManipulate = true;
        this.mapControllers = {};
        this.nodes = {};
        this.currentMapId = App.DEFAULT_MAP_ID;
        this.$canvas = $_canvas;
        this.mapController = null;
        this.node = null;
        this.extendMargin = {
            X: 210,
            Y: 300
        };
        this.scaling = false;
        this.interactionModeBeforePinch = null;
        this.zoomBeforePinch = 1;
        this.pinchDistance = 1;
        this.cke_Editor = null;
        window.addEventListener("resize", function() {
            //$('#titleText').html('resize');
            // al resize si deve calcolare il migliore zoom possibile, in modo da far vedere tutta la mappa, a meno di android
            if (me.device == App.NATIVE_APP_DEVICE.ANDROID &&
                ((me.node!= null &&  (me.node.nodeData.textProperties.editing || me.node.comment.editing))
                    ||
                (me.mapController !=null && me.mapController.isEditingEdge )))
                return;
            //if (this.node !=null && this.node.nodeData.textProperties!=null && ) return;
            me.findBestZoom();
			me.fixCanvasSize();
            me.placeCommentInAllNodes(me.currentMapId, me.mapController.getZoom().scaleX, me.mapController.getZoom().scaleY);
            me.getMapController().redraw();
        }, false);
        $("#mapCanvas").unbind("click").bind("click", eventOnMapCanvas);
        $("#mapCanvas").unbind("touchstart").bind("touchstart", eventOnMapCanvas);
        //$("#mapCanvas").unbind("touchmove").bind("touchmove", eventOnTouchMove);
        //$("#mapCanvas").unbind("touchend").bind("touchend", eventOnTouchEnd);
        this.nodesCount = 0;
        this.progressbar = $("#progressbar");
        this.progressLabel = $(".progress-label");
        me.fixCanvasSize();
        this.editing = false;
        // settings the defaults for nodes and edges: TODO find a way to set them as the same of node and edgedata
        this.lastNodeData ={
            penColor: Node.BRUSHCOLORS.GRAY,
            penColorId: 1,
            nodeType: Node.NODETYPE.ROUNDRECT,
            textProperties:{
                size: Node.LABELSIZE.DEFAULT,
                bold: false,
                italic: false
            },
            nodeTempRoot: ""
        };
        this.lastEdgeData = {
            penColor:  Edge.BRUSHCOLORS.GRAY,
            penColorId: 1,
            penWidth: Edge.PENWIDTH.WIDTH4,
            edgeType: Edge.EDGETYPE.NONE,
            textProperties:{
                bold: false,
                italic: false
            }
        };

        console.log('Device ' + this.device + ' ' + this.deviceVersion);
        this.messageOnDisableView();
        this.messageOnDisableEditing();
    };

    // CONSTANTS
    App.DEFAULT_MAP_ID = "defaultMapId";

    App.NATIVE_APP_DEVICE = {
        IOS : 'ios',
        ANDROID: 'android',
        DESKTOP: 'desktop',
        WEB: 'web',
        STANDALONE: 'standalone'
    };

    // PUBLIC FUNCTIONS
    // I/O

    function getTouchEvent(e){
        if(me && me.device == App.NATIVE_APP_DEVICE.ANDROID){
            if(e.originalEvent.touches.length==2){
                var dist = Math.sqrt(
    						    (e.originalEvent.touches[0].pageX-e.originalEvent.touches[1].pageX) * (e.originalEvent.touches[0].pageX-e.originalEvent.touches[1].pageX) +
    						    (e.originalEvent.touches[0].pageY-e.originalEvent.touches[1].pageY) * (e.originalEvent.touches[0].pageY-e.originalEvent.touches[1].pageY));
    			if(me.pinchDistance<=1)
                    me.pinchDistance = dist;
                e.originalEvent.scale = dist / me.pinchDistance;
            }
        }
        return e.originalEvent;
    }

	 function eventOnMapCanvas(evt) {
            mc = document.app.getMapController();
            if ($('.nodeCommentOverlay:visible').length>0){
                return;
            }

            if ($('.linkEditorOverlay:visible').length>0){
                //TODO what to do if linkeditor is visible
                if($('#cke_EditorEdge:visible').length>0)
                    document.app.hideEditor();
            }

            if (mc){
                var point = mc.stage.globalToLocal(mc.stage.mouseX, mc.stage.mouseY);

                /*var tE = getTouchEvent(evt);
                if(tE && (tE.touches && tE.touches.length == 2)) {
                    this.scaling = true;
                    document.app.pinchStart(tE, point);
                }*/

                obj = mc.stage.getObjectsUnderPoint(point.x, point.y); //scopre se ci sono nodi
                if (obj.length==0 && !mc.isSelectingEdge){ // non c'è nulla sotto quel punto in cui è stato cliccato
                    document.app.hideEditor();
                    mc.setEditableLabels();

                }
            }
        }

        function eventOnTouchMove(e){
            if(this.scaling) {
                document.app.pinchMove(getTouchEvent(e));
            }
        }

        function eventOnTouchEnd(e){
            if(this.scaling) {
                document.app.pinchEnd(getTouchEvent(e));
                this.scaling = false;
            }
        }

        App.prototype.pinchStart = function(e, point){
            mc = document.app.getMapController();
            this.interactionModeBeforePinch = mc.getInteractionMode();
            this.zoomBeforePinch = mc.getZoom().scaleX;
            this.modeHand();
        }

        App.prototype.pinchMove = function(e){
            //console.log('pinchmove' + e.scale);
            mc = document.app.getMapController();
            console.log('scale'+e.scale);
            console.log('zoom'+this.zoomBeforePinch);
            var zoom = (e.scale) * (this.zoomBeforePinch*100);
            console.log('newzoom'+zoom);
            if(zoom>500) zoom = 500;
            /*mc.setOffSet({
                x:-this.pinchCenter.x * (zoom - this.pinchCenter.zoom)/100,
                y:-this.pinchCenter.y * (zoom - this.pinchCenter.zoom)/100
            });*/
            mc.setZoom(zoom);

        }

        App.prototype.pinchEnd = function(e){
            if(this.interactionModeBeforePinch==MapController.MODE.DIRECT){
                this.modeDirect();
            }
            this.pinchDistance = 1;
        }

    /**
     * Set the device tha can be used to deal with maps.
     * desktop, ios, android, web -> see App.NATIVE_APP_DEVICE
     * @param device
     */
    App.prototype.setDevice = function(device){

        var stringDevice = device.toLowerCase().split(" ");
        if (stringDevice[0])
            this.device = stringDevice[0];
        if (stringDevice[1])
        {    
            if (this.device == App.NATIVE_APP_DEVICE.ANDROID)
            {
                this.deviceVersion = parseInt(stringDevice[1], 10);
            }
            else
            {
                // non si sa che formato di version hanno le altre app, per cui per il momento viene impostato as it is.
                // gestire un formato di version significa che una certa feature è dipendente dalla version
                // e pertanto sarebbe comunque una nuova distribuzione della web app.
                this.deviceVersion = stringDevice[1]; 
            }
        }
        // sulla base del device abilito e disabilito delle funzionalita
        if (this.device == App.NATIVE_APP_DEVICE.ANDROID)
        {
            if (this.deviceVersion <=16)
            {
                 this.canView = false;
                 this.canManipulate = false;
            }
            else if (this.deviceVersion >=17 && this.deviceVersion <=18 )
            {
                this.canView = true;
                this.canManipulate = false;
            }
        }

        //this.messageOnDisableView();
        //this.messageOnDisableEditing();
    };

    /**
     *
     */
    App.prototype.messageOnDisableView = function()
    {

        if (!this.canView){
            $('#disable-view-maps .unsupported-device').text(this.device);
            $('#disable-view-maps').show();
            $('#mapCanvas').hide();
        }

    };

    /**
     *
     */
    App.prototype.messageOnDisableEditing = function()
    {
        if (!this.canManipulate){
            $('#disable-editing-maps .unsupported-device').text(this.device);
            $('#disable-editing-maps').show();
            $('#disable-editing-maps .close-disable-message').unbind('click').bind('click', function(){
                $('#disable-editing-maps').hide();
            });
        }
    };

    App.prototype.useMapData = function () {
        this.mapController.redraw();
        this.node = null;

        this.setMapTitle(this.currentMapId);

        if (!this.canManipulate) {
            this.modeHand();
        }
    }

    /**
     * Shows a map.
     *
     * @param {string} mapId map id, default to current if null
     */
    App.prototype.showMap = function (mapId) {
        //per applicazione su web serve questo controllo per evitare duplicazione nodi
        //if(!mapId)
            //return;

        this.messageOnDisableView();
        this.messageOnDisableEditing();

        if (!this.canView) return;


        this.fixCanvasSize();
        var that = this;

        this.getMapController(mapId, function(mapController){
            that.mapController = mapController;
            that.mapController.findMap();

            that.mapController.redraw();
            that.node = null;

            that.setMapTitle(mapId);

            if (!that.canManipulate) {
                that.modeHand();
            }
            document.nativeService.notifyMapLoaded();
        });

    };


    /*
     * open an empty map
     */
    App.prototype.openEmptyMap = function(mapId){
        this.fixCanvasSize();

        var that = this;
        this.getMapController(mapId, function(mapController){
            that.mapController = mapController;
            that.mapController.redraw();
            that.node = null;
            that.setMapTitle(mapId);
            document.nativeService.notifyMapLoaded();
        });
    }


    App.prototype.getMapTitle = function(mapId)
    {
        if (!mapId)
            mapId = this.currentMapId;
        return this.mapControllers[mapId].map.map.mapProps.mapName;
    }

    App.prototype.setMapTitle = function (mapId){
        if (!mapId)
            mapId = this.currentMapId;
        var mapTitle = this.mapControllers[mapId].map.map.mapProps.mapName;
        $('#titleText').text(mapTitle);
        $('#inputText .titleNodeLabel').val(mapTitle);
        this.handleTitleMap();
    };

    App.prototype.setMapAuthor = function(mapId, author){
        if (mapId==undefined)
            mapId= this.currentMapId;
        mc  = this.getMapController(mapId);
        mc.map.map.mapProps.author = author;
    };

    App.prototype.setMapDate = function(mapId, date){
        if (mapId==undefined)
            mapId= this.currentMapId;
        mc  = this.getMapController(mapId);
        mc.map.map.mapProps.date = date;
    };

    App.prototype.updateMapTitle = function(mapId, title){
        if (mapId==undefined)
            mapId= this.currentMapId;
        mc  = this.getMapController(mapId);

        if (title == null) title = mc.map.map.mapProps.mapName;
        mc.map.map.mapProps._preMapName = mc.map.map.mapProps.mapName;
        mc.map.map.mapProps.mapName = title;
		document.nativeService.saveMapSession();
    };


    App.prototype.hideEditor = function() {
        $(".nodeEditor").hide();
        $(".nodeCommentOverlay").hide();
        $(".linkEditor").hide();
        $(".nodeTextEditorOverlay").hide();
        if(me.mapController.isEditingEdge){
            me.mapController.isEditingEdge = false;
                        var from_edge = new Edge(me.actualEdge.toJson());
                        //var s = $.trim(evt.editor.getData());
                        //me.actualEdge.titleText = s;
                        edgeView.updateWidget(me.mapController.map.nodes[me.actualEdge.source], me.mapController.map.nodes[me.actualEdge.target]);
                        me.mapController.updateMap(me.actualEdge.source, false);
                        me.mapController.redraw();
                        me.mapController.setUpdateModelVariant(from_edge, me.actualEdge);
                        var linkEditor = $('.linkEditor');
                        var editLinkLabelInput = linkEditor.find('.linkLabelInput');
                        if (!editLinkLabelInput.hasClass('selected')) {
                            editLinkLabelInput.hide();
                        }
        }
        if(this.cke_Editor&&this.cke_Editor.status=="ready"){
            me.mapController.updateMap(this.node.id, 100);
        }
        if(this.cke_Editor&&this.cke_Editor.status!="destroyed"){
            this.cke_Editor.destroy(true);
        }

    };


    App.prototype.setVisibleEditor = function(value){
        if (value==true) {
            $(".nodeEditor").css('visibility', 'visible');
        }
        else {
            $(".nodeEditor").css('visibility', 'hidden');
        }
    };

    App.prototype.hideOverlayContents = function() {
        if(this.node) $("."+this.node.id).hide();
        $(".nodeReadCommentOverlay").hide();
    };


    App.prototype.showEditor = function() {

        var data = this.node.nodeData,
            nodeEditor = $(".nodeEditor");

        nodeEditor.hide();

        this.hideLinkEditor();

        nodeEditor.find(".pinNodeBtn").toggleClass("selected", data.mode === Node.MODE.ROOT);
        nodeEditor.find(".editLabelBtn").toggleClass("selected", data.textProperties.editing);
        var me = this;
        var node = this.node;
        if (data.textProperties.editing) {

            //nodeEditor.find(".nodeLabelInput").show();
            nodeEditor.find("#nodeTextEditorOverlay").show();
            if(!CKEDITOR.instances || !CKEDITOR.instances['Editor']){
                this.cke_Editor = CKEDITOR.replace('Editor',{
                    on: {
                        instanceReady: function(){
                            $('#cke_Editor').css('left', (node.geometry.width/window.devicePixelRatio)/2 - $('#cke_Editor').outerWidth()/2);
                            $('#nodeTextEditorOverlay').css('left', (node.geometry.width/window.devicePixelRatio)/2 - $('#nodeTextEditorOverlay').outerWidth()/2);
                        }
                    },
                    fontSize_defaultLabel: getLabelPixelSize(node.nodeData.textProperties.size)+'px'

                });
                CKEDITOR.addCss('.cke_editable {font-size: '+getLabelPixelSize(node.nodeData.textProperties.size)+'px;}');
                this.cke_Editor.on( "change", function( evt ) {
                    var s = $.trim(evt.editor.getData().replace(/\n/g,''));
                    //if (s) {
                    if(node.nodeData.textProperties.bold){
                        node.nodeData.textProperties.bold=false;
                    }
                    if(node.nodeData.textProperties.italic){
                        node.nodeData.textProperties.italic=false;
                    }
                    var from_node = $.extend({}, node);
                    node.titleText = s;
                    /*BTW-431 me.mapController.updateMap(node.id, 100);*/
                    me.mapController.setUpdateModelVariant(from_node, node);
                    //me.onNodeStateChange(node);
                });
            }
            nodeEditor.find(".nodeLabelInput").show()
                .focus()
                .unbind("keypress").bind("keypress", function(e) {
                    /*if (e.which === 13) {
                     editLabelBtn.removeClass("selected");
                     editLabelInput.unbind("keypress")
                     .hide();
                     node.nodeData.textProperties.editing = false;
                     }*/
                });
        } else {
            nodeEditor.find(".nodeLabelInput").hide();
        }

        nodeEditor.find(".boldBtn").toggleClass("selected", data.textProperties.bold);
        nodeEditor.find(".italicBtn").toggleClass("selected", data.textProperties.italic);
        nodeEditor.find(".colorBtn").removeClass("selected")
            .eq(data.penColorId - 1).addClass("selected");
        if (this.node.comment.editing) {
            this.setupCommentOverlay(this.node);
        } else {
            $(".nodeCommentOverlay").hide();
            nodeEditor.show();
        }

        nodeEditor.find(".deleteBtn").unbind("click").bind("click", function() {
            //console.log("CANCELLA");
            nodeEditor.find(".editLabelBtn").removeClass("selected");
            if(node && node.nodeData && node.nodeData.textProperties){
                node.nodeData.textProperties.editing = false;
            }
            me.editing = false;
            //me.deleteNode(node);
            node.titleText = '';
            if(node){
                me.mapController.updateMap(node.id);
            }

            if(me.cke_Editor&&me.cke_Editor.status!="destroyed"){
                me.cke_Editor.destroy(true);
            }
            $(".nodeTextEditorOverlay").hide();
            //me.hideEditor();
            //me.showEditor();

        });
        nodeEditor.find(".saveBtn").unbind("click").bind("click", function() {
            //console.log("SALVA");
            nodeEditor.find(".editLabelBtn").removeClass("selected");
            if(node && node.nodeData && node.nodeData.textProperties && node.nodeData.textProperties.editing){
                node.nodeData.textProperties.editing = false;
            }
            me.editing=false;
            if(node){
                me.mapController.updateMap(node.id);
            }
            if(me.cke_Editor&&me.cke_Editor.status!="destroyed"){
                me.cke_Editor.destroy(true);
            }
            $(".nodeTextEditorOverlay").hide();
            //me.hideEditor();
            //me.showEditor();
        });

    };

    App.prototype.onNodeStateChange = function(node) {

        console.log("ENTERING onStateChange();");
        if (node==undefined) return;
        if(!node && this.node!==null) node = this.node;

        this.node = node;

        this.editing = false;
        //BTB-851 per evitare che il nodo
        this.setVisibleEditor(true);

        var dpr = window.devicePixelRatio || 1;

        var px = node.geometry.x - (node.geometry.width / 2),
            py = node.geometry.y - (node.geometry.height / 2),

            me = this,

            minw = me.mapController.getNodeView(node.id).text.width+25,
            minh = me.mapController.getNodeView(node.id).text.height+15,

            originW = node.geometry.width,
            originH = node.geometry.height,

            zoomX = me.mapController.getZoom().scaleX,
            zoomY = me.mapController.getZoom().scaleY,

            offX = me.mapController.getOffSet().offSetX/dpr,
            offY = me.mapController.getOffSet().offSetY/dpr;

        /* disabilito l'editing da tutti i nodi che non siano il nodo corrente*/
        for (var nodeId in me.mapController.mapView.nodeViews)
        {
            if (nodeId!=node.id){
                //console.log(nodeId);
                me.mapController.mapView.nodeViews[nodeId].node.nodeData.textProperties.editing = false;
            }
        }

        $(".nodeEditor")
            .css("margin-top", (node.geometry.y/dpr*zoomX + (node.geometry.height/dpr / 2)*zoomY) + offY)
            .css("margin-left", node.geometry.x/dpr*zoomY - (node.geometry.width/dpr / 2) + offX - 2);

        // Editor window
        $(".nodeEditorOverlay").css("left", (node.geometry.width/dpr / 2) - (($(".nodeEditorOverlay").outerWidth() / 2)))
            .css("top", 45 );


        var margin_left =  node.geometry.x*zoomY/dpr + offX/dpr - $(".nodeCommentOverlay").outerWidth()/2;
        if (margin_left < 0) margin_left = 0;
        // Node comment
        $(".nodeCommentOverlay")
            .css("margin-top", (node.geometry.y*zoomX/dpr + (node.geometry.height/dpr / 2)*zoomY) + offY/dpr)
            .css("margin-left", margin_left);

        // Node shape
        if (node.nodeData.nodeType == Node.NODETYPE.ELLIPSE) {
            $(".shapeBtn").addClass("rectangle");
        } else{
            $(".shapeBtn").removeClass("rectangle");
        }

        // Input label

        var editLabelInput = $(".nodeLabelInput");
        editLabelInput.removeClass().addClass("nodeLabelInput");

        editLabelInput.val(node.titleText)
            .css("width", (node.geometry.width/dpr*zoomX)- 20);

        if (node.nodeData.textProperties.bold) {
            editLabelInput.addClass("labelBold");
        }
        if (node.nodeData.textProperties.italic) {
            editLabelInput.addClass("labelItalic");
        }
        editLabelInput.addClass("labelSize" + node.nodeData.textProperties.size);
        // class for enabling colors on the label
        var labelClassColor = Node.LABELCOLOR.CHILD;
        if (node.nodeData.mode == Node.MODE.ROOT) labelClassColor = Node.LABELCOLOR.ROOT;
        if (isWhite(node.nodeData.penColor))  labelClassColor = Node.LABELCOLOR.CHILD;
        editLabelInput.addClass(labelClassColor);

        setTimeout(function(){
            // hack to force rendering and calculate outerHeight()
            editLabelInput.css("top", -(node.geometry.height/dpr*zoomX + editLabelInput.outerHeight()/dpr) / 2 + 2)
                .css("left", -((node.geometry.width/dpr*zoomX - node.geometry.width/dpr)/2)+10);
        }, 20);



        this.placeCommentInNode(node, zoomX, zoomY, offX*dpr, offY*dpr);


        this.activateCommentsBtn();
        // Top left Resizer
        $(".resizeHandle-topLeft")
            .css({ left: 0, top: 0,
                marginLeft: -((node.geometry.width/dpr*zoomX - node.geometry.width/dpr)/2)-5,
                marginTop: -(node.geometry.height/dpr*zoomY)-5*zoomY
            })
            .attr("startX", -((node.geometry.width/dpr*zoomX - node.geometry.width/dpr)/2)-5)
            .attr("startY", -(node.geometry.height/dpr*zoomY)-5*zoomY);

        // Bottom left resizer
        $(".resizeHandle-bottomLeft")
            .css({ left: 0, top: 0,
                marginLeft: -((node.geometry.width/dpr*zoomX - node.geometry.width/dpr)/2)-5,
                marginTop:-5
            })
            .attr("startX", -((node.geometry.width/dpr*zoomX - node.geometry.width/dpr)/2)-5)
            .attr("startY", -5);

        // Top Right resizer
        $(".resizeHandle-topRight")
            .css({ left: 0, top: 0,
                marginLeft: ((node.geometry.width/dpr*zoomX + node.geometry.width/dpr)/2)-5,
                marginTop: -(node.geometry.height/dpr*zoomY)-5*zoomY
            })
            .attr("startX", ((node.geometry.width/dpr*zoomX + node.geometry.width/dpr)/2)-5)
            .attr("startY", -(node.geometry.height/dpr*zoomY)-5*zoomY);

        // Bottom right resizer
        $(".resizeHandle-bottomRight")
            .css({ left: 0, top: 0,
                marginLeft: ((node.geometry.width/dpr*zoomX + node.geometry.width/dpr)/2)-5,
                marginTop: -5
            })
            .attr("startX", ((node.geometry.width/dpr*zoomX + node.geometry.width/dpr)/2)-5)
            .attr("startY", -5);

        // Link Handles

        var linkHandleSize = parseInt( $(".linkHandle-top").css("width"), 10 );

        // Top
        $(".linkHandle-top").css("margin-top", -node.geometry.height/dpr * zoomY - linkHandleSize / 2 - 3)
            .css("margin-left", node.geometry.width/dpr / 2 - linkHandleSize / 2)
            .attr("startX", node.geometry.width/dpr / 2 - linkHandleSize / 2)
            .attr("startY", -node.geometry.height/dpr * zoomY - linkHandleSize / 2 - 3);

        // Bottom
        $(".linkHandle-bottom").css("margin-top", -zoomY - linkHandleSize / 2 + 3)
            .css("margin-left", node.geometry.width/dpr / 2 - linkHandleSize / 2)
            .attr("startY", -zoomY - linkHandleSize / 2 + 3)
            .attr("startX", node.geometry.width/dpr / 2 - linkHandleSize / 2);

        // Left
        $(".linkHandle-left").css("margin-top", -node.geometry.height/dpr / 2 * zoomY - linkHandleSize / 2)
            .css("margin-left", -(node.geometry.width/dpr * zoomX - node.geometry.width/dpr) / 2 - linkHandleSize / 2)
            .attr("startX", -node.geometry.height/dpr / 2 * zoomY - linkHandleSize / 2)
            .attr("startY", -(node.geometry.width/dpr * zoomX - node.geometry.width/dpr) / 2 - linkHandleSize / 2);

        // Right
        $(".linkHandle-right").css("margin-top", -node.geometry.height/dpr / 2 * zoomY - linkHandleSize / 2)
            .css("margin-left", (node.geometry.width/dpr * zoomX + node.geometry.width/dpr) / 2 - linkHandleSize / 2 + 3)
            .attr("startX", -node.geometry.height/dpr / 2 * zoomY - linkHandleSize / 2 + 3)
            .attr("startY", (node.geometry.width/dpr * zoomX + node.geometry.width/dpr) / 2 - linkHandleSize / 2);

        // Backgrounds

        var handleBckColor = node.nodeData.penColor;
        if (isWhite(node.nodeData.penColor)){
            handleBckColor = Node.BRUSHCOLORS.GRAY
        }
        //console.log(handleBckColor);
        $(".linkHandle").css("background-color", handleBckColor)
            .show();

        $(".nodeReadCommentOverlay").hide();

        /* draw the delete button +/
         imageDidaOverlay
         */
        if (node.image!= null && node.image.name!=""){
            $(".deleteImageBtn")
                .css({ left: 0, top: 0,
                    marginLeft: ((node.geometry.width/dpr*zoomX + node.geometry.width/dpr)/2)-36,
                    marginTop: -(node.geometry.height/dpr*zoomY) + 4*zoomY,
                    display: 'block'
                })
                .attr("startX", ((node.geometry.width/dpr*zoomX + node.geometry.width/dpr)/2)-36)
                .attr("startY", -(node.geometry.height/dpr*zoomY) + 4*zoomY);
            //me.mapController.updateMap(node.id);
        }else{
            $(".deleteImageBtn").css('display', 'none');
        }
        me.showEditor();
        this.handleButtons(node);

    };





    /*                      NODE EDITING
     ------------------------------------------------------------------------- */

    App.prototype.handleButtons = function(node) {

        var me = this,
            nodeEditor = $(".nodeEditor");

        // copy of a node, so I have from json
        var from_node = new Node(node.toJson());

        // COLOR

        nodeEditor.find(".colorBtn").unbind("click").bind("click", function() {
            var color = $(this).css("background-color"),
                nodeView = me.mapController.getNodeView(node.id),
                alpha = node.mode == Node.MODE.CHILD ? 0.3 : 1,
                colorClass = $(this).attr("class").split(" ")[1];

            var nodeMode = nodeView.node.nodeData.mode ? Node.MODE.ROOT : Node.MODE.CHILD;
            // if the background color of a node is white, then
            nodeView.node.setBrushColor(color, nodeView.node.nodeData.mode)
            nodeView.node.setPenColor(color);

            me.lastNodeData.penColor = color;

            nodeView.node.nodeData.penColorId = +colorClass.substring(colorClass.indexOf("-")+1, colorClass.length);

            me.lastNodeData.penColorId = nodeView.node.nodeData.penColorId;
            // if it is white then the label has to be black
            if (isWhite(color)){
                var titleTextElement = $('#node_content_' + nodeView.node.id);
                var labelClassColor = Node.LABELCOLOR.CHILD;
                titleTextElement.addClass(labelClassColor);
            }

            nodeEditor.find(".colorBtn").removeClass("selected");
            $(this).addClass("selected");


            nodeEditor.find(".linkHandle").css("background-color", node.nodeData.penColor);
            if (isWhite(node.nodeData.penColor)){
                nodeEditor.find(".linkHandle").css("background-color", Node.BRUSHCOLORS.GRAY);

            }
            me.mapController.updateMap(node.id);
            me.mapController.setUpdateModelVariant(from_node, node);
            me.onNodeStateChange(node);

        });

        // SHAPE

        var shapeButton = nodeEditor.find(".shapeBtn");
        shapeButton.unbind("click").bind("click", function() {
            var nodeView = me.mapController.getNodeView(node.id),
                isRoundRect =(nodeView.node.nodeData.nodeType == Node.NODETYPE.ROUNDRECT);
            shapeButton.toggleClass("rectangle");
            nodeView.node.nodeData.nodeType = isRoundRect ? Node.NODETYPE.ELLIPSE : Node.NODETYPE.ROUNDRECT;
            me.lastNodeData.nodeType = nodeView.node.nodeData.nodeType;
            me.mapController.updateMap(node.id);
            me.mapController.setUpdateModelVariant(from_node, node);
            me.onNodeStateChange(node);

        });

        // LABEL EDITING

        var editLabelBtn = nodeEditor.find(".editLabelBtn"),
            editLabelInput = nodeEditor.find(".nodeLabelInput");

        editLabelBtn.unbind("click").bind("click", function() {
            var editing = !editLabelBtn.hasClass("selected");
            editLabelBtn.toggleClass("selected");
            var nodeTextValue=node.titleText;
            nodeTextValue = nodeTextValue.replace('font-weight:600','font-weight:700').replace('font-weight: 600','font-weight: 700');
            if(node.nodeData.textProperties.bold){
                if(nodeTextValue.indexOf("<p")==0){
                    /*var pre=nodeTextValue.substring(0,nodeTextValue.indexOf(">")+1);
                    nodeTextValue=nodeTextValue.substring(nodeTextValue.indexOf(">")+1);
                    nodeTextValue=nodeTextValue.replace("</p>","");
                    nodeTextValue="<strong>"+nodeTextValue;
                    nodeTextValue=pre+nodeTextValue;
                    nodeTextValue=nodeTextValue+"eqweqwqwe</strong>";*/
                    nodeTextValue="<strong>"+nodeTextValue+"</strong>";
                }else{
                    nodeTextValue="<strong>"+nodeTextValue+"</strong>";
                }

            }
            if(node.nodeData.textProperties.bold){
                if(nodeTextValue.indexOf("<p")==0){
                    /*var pre=nodeTextValue.substring(0,nodeTextValue.indexOf(">")+1);
                    nodeTextValue=nodeTextValue.substring(nodeTextValue.indexOf(">")+1);
                    nodeTextValue=nodeTextValue.replace("</p>","");
                    nodeTextValue="<strong>"+nodeTextValue;
                    nodeTextValue=pre+nodeTextValue;
                    nodeTextValue=nodeTextValue+"eqweqwqwe</strong>";*/
                    nodeTextValue="<strong>"+nodeTextValue+"</strong>";
                }else{
                    nodeTextValue="<strong>"+nodeTextValue+"</strong>";
                }

            }
            if(node.nodeData.textProperties.italic){
                nodeTextValue="<em>"+nodeTextValue+"</em>";
            }
            editLabelInput.val(nodeTextValue);
            node.titleText=nodeTextValue;
            node.nodeData.textProperties.editing = editing;
            if (editing) {
                if(!CKEDITOR.instances || !CKEDITOR.instances['Editor']){
                    me.cke_Editor = CKEDITOR.replace('Editor', {
                        on: {
                            instanceReady: function(){
                                $('#cke_Editor').css('left', (node.geometry.width/window.devicePixelRatio)/2 - $('#cke_Editor').outerWidth()/2);
                                $('#nodeTextEditorOverlay').css('left', (node.geometry.width/window.devicePixelRatio)/2 - $('#nodeTextEditorOverlay').outerWidth()/2);

                            }
                        },
                        fontSize_defaultLabel: getLabelPixelSize(node.nodeData.textProperties.size)+'px'
                    });
                    CKEDITOR.addCss('.cke_editable {font-size: '+getLabelPixelSize(node.nodeData.textProperties.size)+'px;}');
                    me.cke_Editor.on( "change", function( evt ) {
                        var s = $.trim(evt.editor.getData().replace(/\n/g,''));
                        //if (s) {
                        if(node.nodeData.textProperties.bold){
                            node.nodeData.textProperties.bold=false;
                        }
                        if(node.nodeData.textProperties.italic){
                            node.nodeData.textProperties.italic=false;
                        }
                        var from_node = $.extend({}, node);
                        node.titleText = s;
                        //BTW-431 me.mapController.updateMap(node.id, 100);
                        me.mapController.setUpdateModelVariant(from_node, node);
                        //me.onNodeStateChange(node);
                    });
                }
                editLabelInput.show()
                    .focus()
                    .unbind("keypress").bind("keypress", function(e) {
                        /*if (e.which === 13) {
                         editLabelBtn.removeClass("selected");
                         editLabelInput.unbind("keypress")
                         .hide();
                         node.nodeData.textProperties.editing = false;
                         }*/
                    });
            } else {
                editLabelInput.unbind("keypress")
                    .hide();


            }

            me.mapController.updateMap(node.id, 100);
            me.onNodeStateChange(node);
        });

        editLabelInput.unbind("change").bind("change", function() {
            var s = $.trim(editLabelInput.val());
            //if (s) {
            node.titleText = s;
            me.mapController.updateMap(node.id, 100);
            me.mapController.setUpdateModelVariant(from_node, node);
            me.onNodeStateChange(node);

            //}
        });

        // EDIT COMMENT
        var commentBtn = nodeEditor.find(".commentBtn");

        commentBtn.unbind("click").bind("click", function() {
            me.hideEditor();
            me.setupCommentOverlay(node);
        });

        commentBtn.toggleClass("editComment", node.deepText.length > 0 );


        // RESIZE

        var zoom =  this.mapController.getZoom(),
            zoomX = zoom.scaleX,
            zoomY = zoom.scaleY;

        /* qui si calcola la maggiore with tra testo e img

         */
        var nodeView = me.mapController.getNodeView(node.id),
            minw = 58;
        minh = 38,
            originW = node.geometry.width,
            originH = node.geometry.height;

        nodeEditor.find(".resizeHandle").draggable();

        nodeEditor.find(".resizeHandle").unbind("dragstart").bind("dragstart", function(event,ui) {
            $('#comment_contents .'+node.id).css('visibility', 'hidden');
        });

        nodeEditor.find(".resizeHandle").unbind("dragstop").bind("dragstop", function(event,ui) {
            me.onNodeStateChange(node);
            me.mapController.updateMap(node.id);
            me.mapController.setUpdateModelVariant(from_node, node);
            $('#comment_contents .'+node.id).css('visibility', 'visible');
            me.fixCanvasSize();
        });

        nodeEditor.find('.resizeHandle').unbind('drag').bind('drag', function(event, ui){
            me.hideEditor();
            me.hideOverlayContents(node);

            var ww, hh;

            if ($(this).hasClass('resizeHandle-topRight')){
                ww = originW + ui.position.left * 2,
                    hh = originH + ui.position.top * -2;
                if (ww<minw || hh<minh) {
                    console.log(!"isMin");
                    return;
                }
                var newX = nodeEditor.find(".resizeHandle-bottomLeft").attr("startX") - (ui.position.left * zoomX),
                    newY = nodeEditor.find(".resizeHandle-bottomLeft").attr("startY") - (ui.position.top * zoomY);
                nodeEditor.find(".resizeHandle-bottomLeft").css("margin-left", newX)
                    .css("margin-top", newY);

                newX = nodeEditor.find(".resizeHandle-topLeft").attr("startX") - ui.position.left * zoomX;
                newY = (nodeEditor.find(".resizeHandle-topLeft").attr("startY")) - ui.position.top * -1 * zoomY;
                nodeEditor.find(".resizeHandle-topLeft").css("margin-left", newX)
                    .css("margin-top", newY);

                newX = zoomX * ui.position.left + nodeEditor.find(".resizeHandle-bottomRight").attr("startX") * 1;
                newY = nodeEditor.find(".resizeHandle-bottomRight").attr("startY") - ui.position.top * zoomY;

                nodeEditor.find(".resizeHandle-bottomRight").css("margin-left", newX)
                    .css("margin-top", newY);
            }

            if ($(this).hasClass('resizeHandle-bottomLeft')){
                ww = originW + ui.position.left * -2,
                    hh = originH + ui.position.top * 2;
                if (ww<minw || hh<minh) {
                    console.log(!"isMin");
                    return;
                }

                var newX = nodeEditor.find(".resizeHandle-topRight").attr("startX")-ui.position.left * zoomX,
                    newY = nodeEditor.find(".resizeHandle-topRight").attr("startY")-ui.position.top * zoomY;
                nodeEditor.find(".resizeHandle-topRight").css("margin-left", newX)
                    .css("margin-top", newY);

                newX = nodeEditor.find(".resizeHandle-topLeft").attr("startX") - ui.position.left*-1 * zoomX;
                newY = (nodeEditor.find(".resizeHandle-topLeft").attr("startY")) - ui.position.top * 1 * zoomY;
                nodeEditor.find(".resizeHandle-topLeft").css("margin-left", newX)
                    .css("margin-top", newY);

                newX = (ui.position.left*-1*zoomX) + nodeEditor.find(".resizeHandle-bottomRight").attr("startX")*1;
                newY = nodeEditor.find(".resizeHandle-bottomRight").attr("startY")- ui.position.top * -1 *zoomY;

                nodeEditor.find(".resizeHandle-bottomRight").css("margin-left", newX)
                    .css("margin-top", newY);
            }

            if ($(this).hasClass('resizeHandle-topLeft')) {
                ww = originW + ui.position.left * -2,
                    hh = originH + ui.position.top * -2;
                if (ww<minw || hh<minh){
                    console.log(!"isMin");
                    return;
                }

                var newX = nodeEditor.find(".resizeHandle-topRight").attr("startX")-ui.position.left*zoomX,
                    newY = nodeEditor.find(".resizeHandle-topRight").attr("startY")-ui.position.top*-1*zoomY;
                nodeEditor.find(".resizeHandle-topRight").css("margin-left", newX)
                    .css("margin-top", newY);

                newX = nodeEditor.find(".resizeHandle-bottomLeft").attr("startX") - ui.position.left*-1 * zoomX;
                newY = nodeEditor.find(".resizeHandle-bottomLeft").attr("startY") - ui.position.top *1 * zoomY;
                nodeEditor.find(".resizeHandle-bottomLeft").css("margin-left",newX)
                    .css("margin-top",newY);

                newX = (ui.position.left*-1*zoomX) + nodeEditor.find(".resizeHandle-bottomRight").attr("startX")*1;
                newY = nodeEditor.find(".resizeHandle-bottomRight").attr("startY")- ui.position.top * 1 * zoomY;
                nodeEditor.find(".resizeHandle-bottomRight").css("margin-left",newX)
                    .css("margin-top",newY);
            }

            if ($(this).hasClass('resizeHandle-bottomRight')) {
                ww = originW + ui.position.left * 2,
                    hh = originH + ui.position.top * 2;
                if(ww<minw || hh<minh){
                    console.log(!"isMin");
                    return;
                }

                var newX = nodeEditor.find(".resizeHandle-topRight").attr("startX")-ui.position.left*-1*zoomX,
                    newY = nodeEditor.find(".resizeHandle-topRight").attr("startY")-ui.position.top*1*zoomY;
                nodeEditor.find(".resizeHandle-topRight").css("margin-left", newX)
                    .css("margin-top", newY);

                newX = nodeEditor.find(".resizeHandle-bottomLeft").attr("startX") - ui.position.left*1 * zoomX;
                newY = nodeEditor.find(".resizeHandle-bottomLeft").attr("startY") - ui.position.top *-1 * zoomY;
                nodeEditor.find(".resizeHandle-bottomLeft").css("margin-left",newX)
                    .css("margin-top",newY);

                newX = (ui.position.left*-1*zoomX) + nodeEditor.find(".resizeHandle-topLeft").attr("startX")*1;
                newY = nodeEditor.find(".resizeHandle-topLeft").attr("startY")- ui.position.top * 1 * zoomY;
                nodeEditor.find(".resizeHandle-topLeft").css("margin-left",newX)
                    .css("margin-top",newY);

            }

            node.setSize(parseInt(ww, 10), parseInt(hh, 10));
            node.setTextMaxSize(10, 10);
            me.mapController.stage.removeChild(nodeView.widget);
            nodeView = me.mapController.mapView.addNodeView(node, me.mapController.stage);
            nodeView.drawWidget();
            me.mapController.setNodeHandlers(nodeView);
            me.mapController.updateMap(node.id);

            //me.onNodeStateChange(node);

        });

        // LABEL FORMAT

        nodeEditor.find(".boldBtn").unbind("click").bind("click", function() {
            $(this).toggleClass("selected");
            node.nodeData.textProperties.bold = $(this).hasClass("selected");
            //me.lastNodeData.textProperties.bold = node.nodeData.textProperties.bold;
            //
            me.mapController.updateMap(node.id, 100);
            me.mapController.setUpdateModelVariant(from_node, node);
            me.onNodeStateChange(node);
        });

        nodeEditor.find(".italicBtn").unbind("click").bind("click", function() {
            $(this).toggleClass("selected");
            node.nodeData.textProperties.italic = $(this).hasClass("selected");
            //me.lastNodeData.textProperties.italic = node.nodeData.textProperties.italic;
            //
            me.mapController.updateMap(node.id, 100);
            me.mapController.setUpdateModelVariant(from_node, node);
            me.onNodeStateChange(node);
        });

        nodeEditor.find(".sizeDecreaseBtn").unbind("click").bind("click", function() {
            var size = node.nodeData.textProperties.size;
            if (size > 1) {
                node.nodeData.textProperties.size = size - 1;
            }
            //me.lastNodeData.textProperties.size = node.nodeData.textProperties.size;
            me.mapController.getNodeView(node.id).updateWidget();

            me.mapController.updateMap(node.id, 100);
            me.mapController.setUpdateModelVariant(from_node, node);
            me.onNodeStateChange(node);
        });

        nodeEditor.find(".sizeIncreaseBtn").unbind("click").bind("click", function() {
            var size = node.nodeData.textProperties.size;
            if (size < Node.LABELSIZE.MAX) {
                node.nodeData.textProperties.size = size + 1;
            }
            //me.lastNodeData.textProperties.size = node.nodeData.textProperties.size;
            me.mapController.getNodeView(node.id).updateWidget();

            me.mapController.updateMap(node.id, 100);
            me.mapController.setUpdateModelVariant(from_node, node);
            me.onNodeStateChange(node);
        });

        // PIN NODE

        nodeEditor.find(".pinNodeBtn").unbind("click").bind("click",function(){
            $(this).toggleClass("selected");
            var isRoot = $(this).hasClass("selected");
            var nodeMode = isRoot ? Node.MODE.ROOT : Node.MODE.CHILD;

            me.mapController.setRoot(node,isRoot);
            node.setBrushColor(node.nodeData.penColor, nodeMode);
            //node.nodeData.tempMode = nodeMode;

            var titleTextElement = $('#node_content_' + node.id);
            var labelClassColor = isRoot ? Node.LABELCOLOR.ROOT : Node.LABELCOLOR.CHILD;
            titleTextElement.addClass(labelClassColor);


            me.lastNodeData.nodeTempRoot = node.id;
            me.mapController.updateMap(node.id);
            me.mapController.setUpdateModelVariant(from_node, node);
        });

        // LINKS

        nodeEditor.find(".linkHandle").draggable();

        nodeEditor.find(".linkHandle").unbind("dragstart").bind("dragstart",function(){
            $(this).attr("sourceId", node.id);
            me.setVisibleEditor(false);
            me.mapController.startCreatingConnection(node);
        });

        nodeEditor.find(".linkHandle").unbind("drag").bind("drag",function(event){
            //console.log(event);
            //me.mapController.drawConnection(0, 0, event.pageX, event.pageY, node);

        });

        nodeEditor.find(".linkHandle").unbind("dragstop").bind("dragstop",function(event){
            hasCreatedConnection = me.mapController.createEdgeConnection(node, event.pageX, event.pageY);

            var _this = $(this);
            _this.attr("style", "");
            if (!hasCreatedConnection){
                me.setVisibleEditor(true);  // BTD-851 in questo caso mostro l'editor
                me.onNodeStateChange(node);
                console.log('the connection has not created');
            }
            //me.onNodeStateChange(node); coomentato per BTD-851
            setTimeout(function(){
                _this.attr("sourceId", "");
            }, 1000);
        });


        // EDITING IMAGE
        var imageOverlay = $(".imageDidaOverlay");

        imageOverlay.find('.deleteImageBtn').unbind('click').bind('click', function(event){
            //  it has to remove the image and then become a node without image
            node.image.name = "";
            me.mapController.updateMap(node.id);
            imageOverlay.find('.deleteImageBtn').hide();
            me.mapController.setUpdateModelVariant(from_node, node);
        });



        // Delete and Save

        nodeEditor.find(".nodeDelete").unbind("click").bind("click", function() {
            me.editing = false;
            me.deleteNode(node);
            me.hideEditor();
        });

        nodeEditor.find(".nodeSave").unbind("click").bind("click", function(){

            node.nodeData.textProperties.editing = false;
            me.editing=false;
            /*var isRoot = $('.pinNodeBtn').hasClass("selected");
             var nodeMode = isRoot ? Node.MODE.ROOT : Node.MODE.CHILD;
             me.mapController.setRoot(node,isRoot);
             node.setBrushColor(node.nodeData.penColor, nodeMode);*/

            me.mapController.updateMap(node.id);
            me.hideEditor();

        });

    };

    App.prototype.updateEditor = function() {
        if(this.node == null) return;
        var zoomX = this.mapController.getZoom().scaleX,
            zoomY = this.mapController.getZoom().scaleY,
            offX = this.mapController.getOffSet().offSetX,
            offY = this.mapController.getOffSet().offSetY,
            node = this.node;
        $(".nodeEditor").css("margin-top", (node.geometry.y*zoomX + (node.geometry.height / 2)*zoomY) + offY/window.devicePixelRatio)
            .css("margin-left", node.geometry.x*zoomY - (node.geometry.width / 2) + offX/window.devicePixelRatio - 2);
    };





    /*                      COMMENT OVERLAY
     ------------------------------------------------------------------------- */

    App.prototype.showCommentOverlay = function(evt) {

        var offX = this.mapController.getOffSet().offSetX,
            offY = this.mapController.getOffSet().offSetY,
            zoomX = this.mapController.getZoom().scaleX,
            zoomY = this.mapController.getZoom().scaleY;

        var overlay = $(".nodeReadCommentOverlay"),
            closeButton = overlay.find(".nodeCloseCommentOverlay"),
            content = $(".comment-content");

        var me = this,
            node = $(evt.target).attr("class").split(" ")[1],
            w = parseFloat(overlay.css("width"))*zoomX;

        node = this.mapController.getNodeView(node).node;

        function closeCommentOverlay() {
            $(".nodeReadCommentOverlay").hide();
        }

        content.html(node.deepText.replace(/\n/g, "<br/>"));
        closeButton.unbind("click").bind("click",function(){
            closeCommentOverlay();
        });

        var margin_left = node.geometry.x*zoomY/window.devicePixelRatio + offX/window.devicePixelRatio - overlay.outerWidth()/2;
        if (margin_left < 0 ) margin_left = 0;


        overlay
            .css("margin-top", (node.geometry.y*zoomX/window.devicePixelRatio + (node.geometry.height / 2)*zoomY/window.devicePixelRatio) + offY/window.devicePixelRatio)
            .css("margin-left", margin_left );

        overlay.show();

    };

    App.prototype.hideCommentOverlay = function(){
        $(".nodeCommentOverlay").hide();
    }

    App.prototype.setupCommentOverlay = function(node) {

        var me = this;

        function closeCommentOverlay(remove) {
            node.comment.editing = false;

            if (remove) {
                node.deepText = "";
                $('.commentReadBtn.' + node.id).remove();

            }

            $(".nodeCommentOverlay").hide();

            if(this.cke_Editor&&this.cke_Editor.status!="destroyed"){
                this.cke_Editor.destroy(true);
            }
            setTimeout(function(){
                me.onNodeStateChange(node);
            },100);
        }



        var overlay = $(".nodeCommentOverlay"),
            textarea = overlay.find("textarea");

        if(!CKEDITOR.instances || !CKEDITOR.instances['EditorComment']){

            me.cke_Editor = CKEDITOR.replace('EditorComment', {fontSize_defaultLabel: '14px'});
            me.cke_Editor.on( "change", function( evt ) {
                console.log('devo salvare');
                textarea.val(evt.editor.getData().replace(/\n/g,''));
            });
        }
        CKEDITOR.addCss('.cke_editable {font-size: 14px;}');


        overlay.show();
        node.comment.editing = true;
        textarea.focus()
            .val(node.deepText)
            .height(120);

        overlay.find(".deleteBtn").unbind("click").bind("click", function() {
            closeCommentOverlay(true);
        });
        overlay.find(".saveBtn").unbind("click").bind("click", function() {
            var from_node  = new Node(node.toJson());
            node.deepText = $.trim(textarea.val());
            textarea.val("");
            if(node.deepText.length!=0){
                // if does not exists the node we have to create it
                if ($("."+node.id).length<=0){
                    var nodeComment = $(".commentReadBtn-template").clone();
                    nodeComment.removeClass("commentReadBtn-template")
                    nodeComment.addClass("commentReadBtn");
                    nodeComment.addClass(node.id);
                    $("#comment_contents").append(nodeComment);
                }
                $("."+node.id).attr("active","true");

            }else{
                $("."+node.id).attr("active","false");
            }
            me.mapController.setUpdateModelVariant( from_node,  node)
            closeCommentOverlay();
        });
    };



    /*                      LINK EDITOR
     ------------------------------------------------------------------------- */


    App.prototype.hideLinkEditor = function(){
        // disable editing of the label
        var me = this.getMapController();
        me.isSelectingEdge = false;
        $(".linkEditor .linkLabelInput").hide();
        $(".linkEditor").hide();
    }

    App.prototype.showLinkEditor = function(edge) {

        var me = this.mapController,
            edgeView = me.getEdgeView(edge.id),
            point = (edgeView.shapeText.localToGlobal(edgeView.text.x + edgeView.text.width/2, edgeView.text.y + edgeView.text.height));

        zoomX = me.getZoom().scaleX,
            zoomY = me.getZoom().scaleY,

            offX = me.getOffSet().offSetX,
            offY = me.getOffSet().offSetY;


        // editor has to appear near the label of the edge
        var data = edge.edgeData,
            linkEditor = $('.linkEditor');


        linkEditor.css('margin-left', point.x/window.devicePixelRatio   - $(".linkEditorOverlay").outerWidth() / 2)
            .css('margin-top', point.y/window.devicePixelRatio + 28 );

        // arrow selection
        linkEditor.find('.editorBtn').removeClass('selected');
        switch(data.type){
            case Edge.EDGETYPE.NONE:
                linkEditor.find('.lineStyleNoBtn').addClass('selected');
                break;
            case Edge.EDGETYPE.FORWARD:
                linkEditor.find('.lineStyleEndBtn').addClass('selected');
                break;
            case Edge.EDGETYPE.BACKWARD:
                linkEditor.find('.lineStyleBeginBtn').addClass('selected');
                break;
            case Edge.EDGETYPE.BOTH:
                linkEditor.find('.lineStyleBothBtn').addClass('selected');
                break;
        };

        //determine the selected color for the edge
        linkEditor.find('.colorBtn').removeClass('selected').eq(data.penColorId - 1).addClass('selected');


        /*var editLinkLabelInput = linkEditor.find('.linkLabelInput');
         editLinkLabelInput.unbind("change").bind("change", function(event) {
         var s = $.trim(editLinkLabelInput.val());
         edge.titleText = s;
         edgeView.updateWidget();
         me.updateMap(edge.source, false);
         me.redraw();
         if (!editLinkLabelInput.hasClass('selected')) {
         editLinkLabelInput.hide();
         }
         });*/

        linkEditor.find(".boldBtn").toggleClass("selected", data.textProperties.bold);
        if (data.textProperties.bold){
            $('#edge_content_' + edge.id).addClass('labelBold');
        }

        linkEditor.find(".italicBtn").toggleClass("selected", data.textProperties.italic);
        if (data.textProperties.italic){
            $('#edge_content_' + edge.id).addClass('labelItalic');
        }


        this.handleLinkEditorButtons(edge);
        $(".nodeEditor").hide();
        $(".linkEditor").show();

    };


    App.prototype.handleLinkEditorButtons = function(edge){

        var me = this,
            linkEditor = $('.linkEditor');
        edgeView = me.mapController.getEdgeView(edge.id);
        source = me.mapController.getNodeView(edge.source).node;
        target = me.mapController.getNodeView(edge.target).node;

        me.mapController.isSelectingEdge = true;

        // enable the editing of the label
        var editLinkLabelInput = linkEditor.find('.linkLabelInput');
        editLinkLabelInput.find(".editLabelBtn").removeClass('selected');
        linkEditor.find(".editLabelBtn").unbind("click").bind("click", function(){
            me.mapController.isEditingEdge = true;
            var from_edge = new Edge(edge.toJson());
            $(this).toggleClass('selected');
            var editingEnabled = $(this).hasClass('selected');
            if (editingEnabled){
                // open the label
                linkEditor.find("#linkTextEditorOverlay").show();
                if(!CKEDITOR.instances || !CKEDITOR.instances['EditorEdge']){
                    //me.cke_Editor = CKEDITOR.replace('EditorEdge');
                    me.cke_Editor = CKEDITOR.replace('EditorEdge', { fontSize_sizes : '8/8px;9/9px;10/10px;11/11px;12/12px;14/14px;16/16px;18/18px;20/20px;22/22px;24/24px;26/26px;28/28px;', fontSize_defaultLabel: '14px'});
                    me.cke_Editor.on( "change", function( evt ) {
                        //editLinkLabelInput.val(evt.editor.getData());
                        var s = $.trim(evt.editor.getData().replace(/\n/g,''));
                        edge.titleText = s;
                        me.actualEdge=edge;
                    });
                }
                CKEDITOR.addCss('.cke_editable {font-size: 14px;}');
                var margin_left = parseInt((linkEditor.outerWidth() - editLinkLabelInput.outerWidth())/2, 10);
                editLinkLabelInput.css({'top': '-48px', 'margin-left': margin_left+"px" });
                editLinkLabelInput.val(edge.titleText);
                editLinkLabelInput.css('display', 'inline-block')
                    .focus()
                    .unbind("keypress").bind("keypress", function(e) {
                        if (e.which === 13) {
                            editLinkLabelInput.blur();
                            editLinkLabelInput.unbind("keypress").hide();
                            me.mapController.setUpdateModelVariant(from_edge, edge);
                            me.mapController.isEditingEdge = false;

                        }
                    })

            }else{
                me.mapController.isEditingEdge = false;
                editLinkLabelInput.hide();
            }
        });


        editLinkLabelInput.unbind("change").bind("change", function(event) {
            me.mapController.isEditingEdge = false;
            var from_edge = new Edge(edge.toJson());
            var s = $.trim(editLinkLabelInput.val());
            edge.titleText = s;
            edgeView.updateWidget(me.mapController.map.nodes[edge.source], me.mapController.map.nodes[edge.target]);
            me.mapController.updateMap(edge.source, false);
            me.mapController.redraw();
            me.mapController.setUpdateModelVariant(from_edge, edge);
            if (!editLinkLabelInput.hasClass('selected')) {
                editLinkLabelInput.hide();
            }

        });



        // COLOR
        linkEditor.find(".colorBtn").unbind('click').bind("click", function(){
            var from_edge = new Edge(edge.toJson());
            var color =  $(this).css("background-color"),
                colorClass = $(this).attr("class").split(" ")[1];
            //color selection
            var colorId = +colorClass.substring(colorClass.indexOf("-")+1, colorClass.length);
            linkEditor.find(".colorBtn").removeClass("selected");
            $(this).addClass("selected");
            me.setEdgeLineColor(color, colorId, edge.id, me.mapController.currentMapId);
            me.lastEdgeData.penColor = color;
            me.lastEdgeData.penColorId = colorId;
            me.mapController.setUpdateModelVariant(from_edge, edge);

        });

        // ARROW
        linkEditor.find(".editorLineStyle").unbind("click").bind("click", function(){
            var from_edge = new Edge(edge.toJson());
            edgeTypeClass = $(this).attr("class").split(" ")[2];
            selectedEdgeDirection = Edge.EDGETYPE.FORWARD;
            switch (edgeTypeClass){
                case 'lineStyleNoBtn':
                    me.setEdgeDirection(Edge.EDGETYPE.NONE, edge.id, me.mapController.currentMapId);
                    selectedEdgeDirection = Edge.EDGETYPE.NONE;
                    break;
                case 'lineStyleEndBtn':
                    me.setEdgeDirection(Edge.EDGETYPE.FORWARD,  edge.id, me.mapController.currentMapId);
                    selectedEdgeDirection = Edge.EDGETYPE.FORWARD;
                    break;
                case 'lineStyleBeginBtn':
                    me.setEdgeDirection(Edge.EDGETYPE.BACKWARD,  edge.id, me.mapController.currentMapId);
                    selectedEdgeDirection = Edge.EDGETYPE.BACKWARD;
                    break;
                case 'lineStyleBothBtn':
                    me.setEdgeDirection(Edge.EDGETYPE.BOTH,  edge.id, me.mapController.currentMapId);
                    selectedEdgeDirection = Edge.EDGETYPE.BOTH;
                    break;
            }
            me.lastEdgeData.edgeType = selectedEdgeDirection;
            //console.log(me.lastedgeType);
            linkEditor.find(".editorLineStyle").removeClass('selected');
            $(this).addClass("selected");
            me.mapController.setUpdateModelVariant(from_edge, edge);
        });


        linkEditor.find(".boldBtn").unbind('click').bind('click', function(){
            var from_edge = new Edge(edge.toJson());
            $(this).toggleClass("selected");
            var isBold = $(this).hasClass('selected');
            edge.edgeData.textProperties.bold = isBold;
            editLinkLabelInput.toggleClass('labelBold');
            edgeView.updateWidget(source, target);
            //me.lastEdgeData.textProperties.bold = isBold;
            me.mapController.updateMap(edge.source, false);
            me.mapController.setUpdateModelVariant(from_edge, edge);
        });


        linkEditor.find(".italicBtn").unbind('click').bind('click', function(){
            var from_edge = new Edge(edge.toJson());
            $(this).toggleClass("selected");
            var isItalic = $(this).hasClass('selected');
            edge.edgeData.textProperties.italic = isItalic;
            editLinkLabelInput.toggleClass('labelItalic');
            edgeView.updateWidget(source, target);

            //me.lastEdgeData.textProperties.italic = isItalic;
            me.mapController.updateMap(edge.source, false);
            me.mapController.setUpdateModelVariant(from_edge, edge);

        });

        linkEditor.find(".deleteBtn").unbind("click").bind("click", function() {
            //console.log("CANCELLA");
            //me.deleteEdge(edge.id, me.mapController.currentMapId);
            edge.titleText = '';
            if(me.cke_Editor&&me.cke_Editor.status!="destroyed"){
                me.cke_Editor.destroy(true);
            }
            $(".nodeTextEditorOverlay").hide();
            edgeView.updateWidget(me.mapController.map.nodes[edge.source], me.mapController.map.nodes[edge.target]);
            me.mapController.updateMap(edge.source, false);
            me.mapController.redraw();
            //me.hideEditor();

        });
        linkEditor.find(".saveBtn").unbind("click").bind("click", function() {
            //console.log("SALVA");
            editLinkLabelInput.removeClass('.labelItalic').removeClass('labelBold');
            editLinkLabelInput.hide();
            if(me.cke_Editor&&me.cke_Editor.status!="destroyed"){
                me.cke_Editor.destroy(true);
            }
            $(".nodeTextEditorOverlay").hide();
            me.hideEditor();
        });


        //SAVE SETTINGS OF EDGE
        linkEditor.find(".linkSave").unbind("click").bind("click", function(){
            editLinkLabelInput.removeClass('.labelItalic').removeClass('labelBold');
            editLinkLabelInput.hide();
            me.hideEditor();
        });

        //DELETE EDGE
        linkEditor.find(".linkDelete").unbind("click").bind("click", function(){
            //me.mapController.removeEdgeView(edge);
            me.deleteEdge(edge.id, me.mapController.currentMapId);
            me.hideEditor();
            //me.showEditor(); BTD-851 evitare che compaia l'editor nel nodo
        });

    }


    /*
     ------------------------------------------------------------------------- */

    /**
     * attiva i commenti
     */
    App.prototype.activateCommentsBtn = function()
    {
        var item =$("body").find(".commentReadBtn[active=true]").show();
        //console.log("item",item)
    }

    /**
     * nasconde i commenti
     */
    App.prototype.hideCommentsBtn = function()
    {
        var item =$("body").find(".commentReadBtn[active=true]").hide();
    }

    /**
     * Posiziona i commenti sul nodo
     * @param node
     * @param zoomX
     * @param zoomY
     * @param offX
     * @param offY
     */
    App.prototype.placeCommentInNode = function(node, zoomX, zoomY, offX, offY){
        // comment button
        if (!node) return;
        //console.log(zoomX);
        if(node.deepText.length>0) {
            $("."+node.id).css("display","block");
            $('.commentReadBtn.'+node.id).attr('active', 'true');
        }else{
            $("."+node.id).css("display","none");
        }

        var mleft = parseInt(node.geometry.x*zoomX/window.devicePixelRatio + offX/window.devicePixelRatio, 10);
        var mtop = parseInt(node.geometry.y*zoomY/window.devicePixelRatio + offY/window.devicePixelRatio, 10);
        
        if ($('.titleBar').is(':hidden')){
			mtop = mtop - 60;
		}

        $('.commentReadBtn.'+node.id)
            .css("left", mleft + ((node.geometry.width * zoomX/window.devicePixelRatio)/2 - $('.commentReadBtn').width()/2) + 'px')
            .css("top", mtop - (node.geometry.height * zoomX/window.devicePixelRatio)/2 + 45 + 'px');

        // SHOW COMMENT
        me = this;
        $(".commentReadBtn."+node.id).unbind("click").bind("click", function() {
            me.hideEditor();
            me.showCommentOverlay(event);
        });
    }

    App.prototype.placeCommentInAllNodes = function(mapId, zoomX, zoomY){
        var mapController = this.getMapController(mapId);
        if (mapController)
        {
            var offX = mapController.getOffSet().offSetX,
                offY = mapController.getOffSet().offSetY;
            for (var nodeId in mapController.mapView.nodeViews)
            {
                var  node = mapController.mapView.nodeViews[nodeId].node;
                this.placeCommentInNode(node, zoomX, zoomY, offX, offY);
            }
        }
    }

    /**
     * Close map and delete map controller
     *
     * @param {string} mapId map id, default to current if null
     */
    App.prototype.closeMap = function (mapId) {
        this.mapController = this.getMapController(mapId);
        if (this.mapController) {
            this.hideEditor();
            this.hideLinkEditor();
            var old_zoom = this.mapController.getZoom();
            this.mapController.clear();
            this.setZoom(old_zoom.scaleX*100);
            this.mapController.redraw();
            delete this.mapControllers[mapId];
        }
    };

    /**
     * Converts a map to Json.
     * If mapId not specified, use current map.
     *
     * @param {string} [mapId] The map id.
     * @returns {json} The map as Json
     */
    App.prototype.toJson = function (mapId) {
        return JSON.stringify(this.getMapController(mapId).toJson(mapId));
    };

    /**
     * Loads an imported and converted map
     * @param {string} [mapId] The map id
     */
    App.prototype.loadImportedMap = function (mapId) {
        this.showMap(mapId);
    };

    // MODES

    /**
     * Switches to direct mode
     * If mapId not specified, use current map.
     *
     * @param {string} [mapId]
     */
    App.prototype.modeDirect = function (mapId) {
        if (!this.canManipulate){
            return;
        }
        this.mapController = this.getMapController(mapId);
        this.mapController.switchInteractionModeToDirect();
        this.changeCanvasInteractionMode("interaction");
    };

    /**
     * Switches to hand mode
     * If mapId not specified, use current map.
     *
     * @param {string} [mapId]
     */
    App.prototype.modeHand = function (mapId) {
        this.mapController = this.getMapController(mapId);
        if (!this.canManipulate){
            return;
        }
        this.mapController.switchInteractionModeToHand();
        this.changeCanvasInteractionMode("hand");
    };

    // DEFAULT NODES VALUES

    App.DEFAULT_SHAPE_TYPE = "roundrect";

    // NODES

    /**
     * return the node id in point
     *
     * Find if the clicked point is over a node
     * @param mapId
     * @param x
     * @param y
     * @returns {string} empty string if no node selected, nodeId if any
     */
    App.prototype.getNodeFromPoint = function(mapId, x, y){
        var nodeIdSent = false;
        if (mapId==null)
            mapId = this.currentMapId;
        this.mapController = this.getMapController(mapId);
        var zoom = this.mapController.getZoom();
        //var point = this.mapController.stage.globalToLocal(x/zoom.scaleX, (y - $('.titleBar:visible').outerHeight())/zoom.scaleY);
        var relativex = x + window.scrollX;
        var relativey = y + window.scrollY - $('.titleBar:visible').outerHeight();
        var point = {x:relativex/zoom.scaleX, y:relativey/zoom.scaleY};
        obj = this.mapController.stage.getObjectUnderPoint(point.x, point.y);
        for (var nodeId in this.mapController.mapView.nodeViews) {
            if (this.mapController.mapView.nodeViews[nodeId].widget==obj){
                if (!this.isDesktopDevice()) {
                    return nodeId;
                }
                else {
                    nodeIdSent = true;
                    this.sendNodeId(nodeId, true);
                }
            }
        }
        if (!this.isDesktopDevice()) {
            return "";
        }
        else {
            if(!nodeIdSent) {
                this.sendNodeId("", true);
            }
        }
    }

    /**
     * Generate a node Id.
     * @param
     * @returns {string} the node id
     */
    App.prototype.generateNodeId = function(){
        var nodeId = "map" + this.randomInt(100, 10000000);
        return nodeId;
    }

    /**
     * Insert a new node.
     * If mapId not specified, use current map.
     *
     * @param [mapId]
     * @returns {string} The new node id
     */

    App.prototype.insertNode = function (mapId, nodeId, text, imagePath, x, y) {

        if (!this.canView  || !this.canManipulate) return "";

        if (mapId==null)
            mapId = this.currentMapId;
        this.mapController = this.getMapController(mapId);

        /* image */
        var img,
            width = Node.DEFAULT_SIZE.width,
            height = Node.DEFAULT_SIZE.height;

        _this = this;
        if (imagePath != undefined && imagePath!="") {
            img = new Image();
            var processImage = function(mapAssetPath) {
                                    img.src = mapAssetPath + '/' + imagePath;
                                    img.onload = function () {
                                                    /* si fa in modo che la width fitti la dimensione del nodo) */
                                                    /* capita la dimensione la si sistema all'interno del nodo */
                                                    var min_image_height,
                                                        scaleX = parseInt(img.naturalWidth / (width), 10);

                                                    scaleX < 1 ? scaleX = 1 : scaleX = scaleX;
                                                    min_image_height = img.naturalHeight / scaleX;

                                                    height < min_image_height ? height = min_image_height : height = height;

                                                    node = _this.insertStandardNode(mapId, nodeId, text, imagePath, width, height, x, y);

                                                    //call U/R stack:
                                                    node.btbid = mapId;
                                                    console.log('A new node is created, thus we call onjs_createVariant', JSON.stringify(node));
                                                    nativeApp.onjs_createVariant(JSON.stringify(node));
                                                    if (!_this.isDesktopDevice()) {
                                                        return node.id;
                                                    }
                                                }
                               }

            if (!this.isDesktopDevice()) {
                var mapAssetPath = document.nativeService.findAssetBaseUrl(mapId);
                processImage(mapAssetPath);
            }
            else {
                document.nativeService.findAssetBaseUrl(mapId, function(mapAssetPath) {
                    processImage(mapAssetPath);
                });
            }
        }else{
            node = _this.insertStandardNode(mapId, nodeId, text, imagePath, width, height);
            //call U/R stack:
            node.btbid = mapId;
            console.log('A new node is created, thus we call onjs_createVariant', JSON.stringify(node));
            nativeApp.onjs_createVariant(JSON.stringify(node));
            if (!this.isDesktopDevice()) {
                return node.id;
            }
            else {
                this.sendNodeId(node.id, false);
            }
        }

    };

    App.prototype.insertStandardNode = function (mapId, nodeId, text, imagePath, widthNode, heightNode, xCoord, yCoord){

        // TODO Move this source into mapController

        var mc = this.getMapController(mapId),
            zoom = (mc.getZoom());

        if (mc){
            for (var nodeIdOld in mc.mapView.nodeViews)
            {
                mc.mapView.nodeViews[nodeIdOld].node.nodeData.textProperties.editing = false;
            }
        }

        // si calcola la y in cui piazzare il nodo, in modo che sia sotto ad un nodo esistente
        var mapExtend = mc.calculateMapExtend();
        var x, y;

        if (xCoord != null && xCoord!=undefined && yCoord!=null && yCoord!=undefined){
            var relativex = xCoord + window.scrollX;
            var relativey = yCoord + window.scrollY - $('.titleBar:visible').outerHeight();
            x = relativex/zoom.scaleX;
            y = relativey/zoom.scaleY;
        }else{
            x = ($(window).width()*window.devicePixelRatio/(zoom.scaleX/window.devicePixelRatio)) / 2;
            if  (mapExtend.maxY == 0){
                y = heightNode +20 ;
            }else{
                y = mapExtend.maxY  + heightNode+20;

            }
        }


        var nodeType = this.lastNodeData.nodeType;

        if (!nodeId || nodeId==undefined)
            nodeId = "map" + this.randomInt(100, 10000000);
        var node = new Node();
        node.id = nodeId;

        node.nodeData.nodeType = nodeType;

        var nodeComment = $(".commentReadBtn-template").clone();
        nodeComment.removeClass("commentReadBtn-template")
        nodeComment.addClass("commentReadBtn")
        nodeComment.addClass(nodeId)

        $("#comment_contents").append(nodeComment);

        node.nodeData.penWidth = 4;
        // Hack - If first node -> set as Node Root
        node.nodeData.mode = this.nodesCount === 0 ? Node.MODE.ROOT : Node.MODE.CHILD;
        //node.nodeData.tempMode = this.nodesCount === 0 ? Node.MODE.ROOT : Node.MODE.CHILD;


        node.nodeData.deepFillColor = "#FFFFFF";
        node.nodeData.borderRadius = 5;
        node.nodeData.penColor = this.lastNodeData.penColor;
        node.nodeData.penColorId = this.lastNodeData.penColorId;
        node.nodeData.textProperties.bold = this.lastNodeData.textProperties.bold;
        node.nodeData.textProperties.italic = this.lastNodeData.textProperties.italic;
        node.nodeData.textProperties.size = this.lastNodeData.textProperties.size;
        node.nodeData.brushColor = rgb2rgba(this.lastNodeData.penColor,node.nodeData.mode);
        node.nodeData.textProperties.editing = true;

        node.image.name = imagePath;

        node.setCenter(x/window.devicePixelRatio, y/window.devicePixelRatio);
        node.setSize(widthNode, heightNode);
        node.setTextMaxSize(10, 10);
        if (text) {
            node.titleText = text;
        }

        mc.addNode(node);
        this.nodes[this.nodesCount] = nodeId;

        this.nodesCount++;

        this.lastNodeData.penColor = node.nodeData.penColor;
        this.lastNodeData.penColorId = node.nodeData.penColorId;

        this.fixCanvasSize();
        
        if (this.device == App.NATIVE_APP_DEVICE.IOS)
        {
            setTimeout(function(){
                mc.redraw();
                if(y/window.devicePixelRatio > document.body.scrollHeight-100)
                    window.scrollTo(0,document.body.scrollHeight);
            }, 0);
        }
        else
        {    
            mc.redraw();

            if(y/window.devicePixelRatio > document.body.scrollHeight-100)
                window.scrollTo(0,document.body.scrollHeight);
        }
        //
        return node;

    }

    App.prototype.updateExistingNode = function(mapId, nodeId, imagePath){

        // has to add an image, first has to find the image
        if (mapId==null)
            mapId = this.currentMapId;

        mc = this.getMapController(mapId);
        nodeView = mc.getNodeView(nodeId);
        node = nodeView.node;

        // ottengo una versione del nodo prima di ogni cambiamento
        var from_node = new Node(node.toJson());

        _this = this;
        node.comment.editing = false;
        node.nodeData.textProperties.editing = false;
        if (imagePath != undefined && imagePath!="") {
            document.app.hideEditor();
            document.app.hideCommentOverlay();
            img = new Image();

            var processImage = function(mapAssetPath) {
                                    img.src = mapAssetPath + '/' + imagePath;
                                    img.onload = function () {
                                                    /* si fa in modo che la width fitti la dimensione del nodo) */
                                                    /* capita la dimensione la si sistema all'interno del nodo */
                                                    var scaleX = parseInt(img.naturalWidth / (node.geometry.width - 50), 10);
                                                    scaleX < 1 ? scaleX = 1 : scaleX = scaleX;
                                                    var min_image_height = img.naturalHeight / scaleX;
                                                    var height = node.geometry.height;
                                                    height < min_image_height ? height = min_image_height : height = node.geometry.height;
                                                    node.setSize(node.geometry.width, height);
                                                    if (node.image==null) //urhe refactor
                                                        node.image = {
                                                            name: "",
                                                            transparent: false,
                                                            custom: false
                                                        };
                                                    node.image.name = imagePath;
                                                    mc.stage.removeChild(nodeView.widget);
                                                    nodeView = mc.mapView.addNodeView(node, mc.stage);
                                                    nodeView.drawWidget();
                                                    mc.setNodeHandlers(nodeView);

                                                    mc.updateMap(node.id);
                                                    _this.fixCanvasSize();

                                                    if (_this.device == App.NATIVE_APP_DEVICE.IOS)
                                                    {
                                                        setTimeout(function(){
                                                            mc.redraw();
                                                        }, 0)
                                                    }
                                                    else
                                                    {
                                                        mc.redraw();
                                                    }
                                                    //document.app.hideCommentOverlay();
                                                    /* CALL U/R stack */
                                                    mc.setUpdateModelVariant(from_node, node);
                                                    document.app.onNodeStateChange(node);
                                                    if (!_this.isDesktopDevice()) {
                                                        return node;
                                                    }
                                                }
                               }

            if (!this.isDesktopDevice()) {
                var mapAssetPath = document.nativeService.findAssetBaseUrl(mapId);
                processImage(mapAssetPath);
            }
            else {
                document.nativeService.findAssetBaseUrl(mapId, function(mapAssetPath) {
                    processImage(mapAssetPath);
                });
            }
        }
    }


    /**
     * Delete a node.
     * If mapId not specified, use current map.
     *
     * @param nodeId
     * @param [mapId]
     */
    App.prototype.deleteNode = function (node) {
        //call U/R stack:
        node.btbid = this.currentMapId;
        console.log('A node has been deleted thus we need to send onjs_deleteVariant ', JSON.stringify(node));
        nativeApp.onjs_deleteVariant(JSON.stringify(node));
        this._innerDeleteNode(node);
    }

    App.prototype._innerDeleteNode = function (node) {
        $("."+node.id).remove();
        this.mapController.removeNodeView(node);
        this.mapController.removeNode(node);
        this.nodesCount--;
    };

    App.prototype.editNodeText = function (nodeId, mapId) {
        this.mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.bringNodeToFront = function (nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.sendNodeToBack = function (nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.selectAllNodes = function (mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.setNodeShape = function (shape, nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.setNodeTextPosition = function (position, nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.setNodeLineThickness = function (thickness, nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.setNodeFramedImage = function (framed, nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.setNodeBackgroundColor = function (color, nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.setNodeBorderColor = function (color, nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.setNodeTextColor = function (color, nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };


    App.prototype.setText = function(node,_text) {
        $("#node_content_"+node.id).html(_text);
    };

    // EDGES

    /**
     * Insert a new edge.
     * If mapId not specified, use current map.
     *
     * @param sourceNodeId
     * @param targetNodeId
     * @param [mapId]
     */
    App.prototype.insertEdge = function (sourceNodeId, targetNodeId, mapId) {

        if (!this.canView || !this.canManipulate) return "";

        var m = mapId || this.currentMapId;
        var edgeMapSourceTarget = sourceNodeId+targetNodeId;
        var edgeMapTargetSource = targetNodeId+sourceNodeId;
        var nodeView = this.mapController.getNodeView(sourceNodeId);
        if (nodeView === undefined || this.mapController.connectionExists(edgeMapSourceTarget) || this.mapController.connectionExists(edgeMapTargetSource)) return;
        var color = nodeView.node.nodeData.penColor;

        this.mapController.addConnection(edgeMapSourceTarget);

        var edgeId = "e" + this.randomInt(100,10000000);
        var edge = new Edge();
        edge.edgeData.penColor = this.lastEdgeData.penColor;
        edge.edgeData.penColorId = this.lastEdgeData.penColorId;
        edge.edgeData.type = this.lastEdgeData.edgeType;
        //edge.edgeData.textProperties.bold = this.lastEdgeData.textProperties.bold
        //edge.edgeData.textProperties.italic = this.lastEdgeData.textProperties.italic
        edge.id = edgeId;
        edge.source = sourceNodeId;
        edge.target = targetNodeId;

        edge.titleText = "";
        this.mapController.addEdge(edge.toJson());
        
        if (this.device == App.NATIVE_APP_DEVICE.IOS)
        {
            setTimeout(function(){
                this.mapController.redraw();
            }, 0);
        }
        else
        {
            this.mapController.redraw();    
        }
        

        //calling U/R stack:
        var jsedge = edge.toJson();
        jsedge.btbid = this.currentMapId;
        console.log('A new edge is created, thus we call onjs_createVariant', JSON.stringify(jsedge));
        nativeApp.onjs_createVariant(JSON.stringify(jsedge));
    };

    App.prototype.deleteEdge = function (edgeId, mapId) {
        var mapController = this.getMapController(mapId);

        //calling U/R stack:
        var edge = mapController.map.edges[edgeId];
        var jsedge = edge.toJson();
        jsedge.btbid = this.currentMapId;
        console.log('An edge has been deleted thus we need to send onjs_deleteVariant ', JSON.stringify(jsedge));
        nativeApp.onjs_deleteVariant(JSON.stringify(jsedge));

        $("#edge_content_"+edgeId).remove();
        mapController.removeEdgeView(edgeId);
        mapController.removeEdge(edgeId);
        mapController.redraw();
    };

    App.prototype.editEdgeText = function (edgeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.bringNodeToFront = function (nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.sendNodeToBack = function (nodeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.selectAllEdges = function (mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.setEdgeLineColor = function (color, colorId, edgeId, mapId) {
        var mapController = this.getMapController(mapId);
        var edgeView = mapController.getEdgeView(edgeId);
        edgeView.edge.edgeData.penColor = color;
        edgeView.edge.edgeData.penColorId = colorId;
        edgeView.drawWidget();
        mapController.redraw();
    };

    App.prototype.setEdgeLineThickness = function (thickness, edgeId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.setEdgeDirection = function (direction, edgeId, mapId) {
        var mapController = this.getMapController(mapId);
        var edgeView = mapController.getEdgeView(edgeId);
        edgeView.edge.edgeData.type = direction;
        edgeView.edge.directed = true;
        if (direction ==  Edge.EDGETYPE.NONE){
            edgeView.edge.directed = false;
        }
        edgeView.drawWidget();
        mapController.redraw();
    };

    /////////
    // TABLES

    App.prototype.insertTable = function (mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.deleteTable = function (tableId, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    /**
     *
     * @param mapId
     * @param tableId
     * @param where (middle or bottom)
     */
    App.prototype.insertTableColumn = function (where, mapId, tableId) {
    };

    /**
     *
     * @param mapId
     * @param tableId
     * @param where (middle or bottom)
     */
    App.prototype.insertTableRow = function (where, mapId, tableId) {
    };

    App.prototype.deleteTableColumn = function (columnId, tableId, mapId) {
    };

    App.prototype.deleteTableRow = function (rowId, tableId, mapId) {
    };

    App.prototype.insertTableImage = function (image, rowId, columnId, tableId, mapId) {
    };

    // TODO Chiedere a Fabrizio:
    // TODO Tabelle: Modifica dimensione nodo implica ridimensionamento riga e colonna corrispondente (una cella è un nodo con vincoli, ad es. immagine sempre riquadrata, testo sempre dentro riquadro


    ///////
    // MISC

    App.prototype.startProgressBar = function () {
        var me=this;
        this.progressbar.progressbar({
            value: false,
            change: function() {
                me.progressLabel.text( me.progressbar.progressbar( "value" ) + "%" );
            },
            complete: function() {
                //me.progressLabel.text( "Complete!" );
            }
        });
    };

    App.prototype.stopProgressBar = function () {
        this.progressbar.progressbar("destroy");
    };

    App.prototype.setProgressBarValue = function (value) {
        this.progressbar.progressbar( "value", value );
    };

    App.prototype.undo = function (mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.redo = function (mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.search = function (pattern, mapId) {
        var mapController = this.getMapController(mapId);
        // TODO
    };

    App.prototype.visit = function (mapId) {
    };

    /**
     * Changes zoom factor
     * If mapId not specified, use current map.
     *
     * @param {number} value
     * @param [mapId]
     */
    App.prototype.setZoom = function (value, mapId) {
        /*console.log("setZoom", value);*/
        this.mapController = this.getMapController(mapId);
        this.mapController.setZoom(value);
        var newZoom = this.mapController.getZoom();
        this.placeCommentInAllNodes(mapId, newZoom.scaleX, newZoom.scaleY);
        this.fixCanvasSize();
        this.mapController.redraw();
    };

    // TODO Archivio immagini

    // TODO Configurazione

    // PRIVATE FUNCTIONS

    App.prototype.changeCanvasInteractionMode = function(mode) {
        this.$canvas.removeClass();
        this.$canvas.addClass(mode);
    };

    App.prototype.randomInt = function(min, max) {
        return Math.floor(Math.random() * (max - min + 1)) + min;
    };

    /**
     * Get a map controller.
     * If mapId not specified, use current map controller.
     *
     * @param [mapId]
     * @returns {MapController}
     */
    App.prototype.getMapController = function(mapId, next) {

        var id = mapId || this.currentMapId;

        var mapController = this.mapControllers[id];
        if (!mapController)
        {
            var _mc = this;
            var initMapController = function(mapAssetPath) {
                                        mapController = new MapController(_mc.$canvas, id, mapAssetPath);
                                        _mc.currentMapId = id;
                                        _mc.mapControllers[id] = mapController;
                                        if (next)
                                        {
                                            next(mapController);
                                        }
                                    }

            if (this.isDesktopDevice())
            {
                document.nativeService.findAssetBaseUrl(id, function(mapAssetPath) {
                    initMapController(mapAssetPath);
                });
            }
            else
            {
                var mapAssetPath = document.nativeService.findAssetBaseUrl(id);
                initMapController(mapAssetPath);
            }
        }
        else
        {
            if (this.isDesktopDevice() && next)
            {
                next(mapController);
            }
            else
            {
                return mapController;
            }
        }

        //this.setMapTitle(id);
    };

    App.prototype.getMapImages = function(){
        var mc = this.getMapController();
        if(mc)
            return mc.getMapImages();
    };


    App.prototype.isEmpty = function(mapId) {
        var id = mapId || this.currentMapId;
        var mapController = this.mapControllers[id];

        if (mapController==null || !mapController)
        {
            return true;
        }

        // number nodes
        if (mapController.mapView.nodeViews && Object.keys(mapController.mapView.nodeViews).length>0)
        {
            return false;
        }
        //@todo valutare la situazione sul numero degli edge
        return true;
    }


    //@todo capire cosa fare di questo metodo
    App.prototype.handleTitleMap = function(mapId){
        var id = mapId || this.currentMapId;

        var titleBarLabel = $('#titleText');
        var inputBarLabel = $('#inputText .titleNodeLabel');

        function updateHtmlTitle(obj, id, newTitle){
            titleBarLabel.toggle();
            inputBarLabel.toggleClass('selected');
            obj.parent().find('.saveBtn').toggle();
            obj.parent().find('.deleteBtn').toggle();
            obj.parent().find('.editTitleBtn').toggle();
            document.app.updateMapTitle(id, newTitle);
        }

        $(".editTitleBtn").unbind("click").bind("click", function() {
            var newTitle = titleBarLabel.text();
            updateHtmlTitle($(this), id, newTitle);
            inputBarLabel.val(newTitle);
        });

        /*
         Eliminato: BTB-850 (Editor Mappe) Eliminare il pulsante cancella nella barra del titolo
         $(".titleBar .deleteBtn").unbind("click").bind("click", function() {
         // click on delete hide the input and reset the previous label
         updateHtmlTitle($(this), id, null);
         });*/

        $(".titleBar .saveBtn").unbind("click").bind("click", function() {
            // click on delete hide the input and reset the previous label
            var newTitle = inputBarLabel.val();
            updateHtmlTitle($(this), id, newTitle);
            titleBarLabel.text(newTitle);
        });


        //keypress on the input label
        inputBarLabel.unbind("keypress").bind("keypress", function(e) {
            if (e.which === 13) {
                var newTitle = inputBarLabel.val();
                updateHtmlTitle($('.titleBar .editTitleBtn'), id, newTitle );
                titleBarLabel.text(newTitle);
                inputBarLabel.blur();
            }
        });
    }


    App.prototype.findBestZoom = function(){
        var canvas = this.$canvas[0];
        canvas.width = (window.innerWidth-20)*window.devicePixelRatio;
        canvas.height = window.innerHeight*window.devicePixelRatio;
        if (this.mapController){
            var newZoom = 1;
            if (this.mapController.mapExtend.maxX != 0){
                // calcolo il nuovo zoom solo se non ci sta nella nuova configurazione width & height

                var extendedWidth = this.mapController.mapExtend.maxX;
                var extendedHeight = this.mapController.mapExtend.maxY;
                console.log('Width canvas ', this.mapController.canvas.width, 'Width extend ', extendedWidth);
                console.log('Height canvas ', this.mapController.canvas.height, 'Height extend ', extendedHeight );

                if (this.mapController.canvas.width > extendedWidth &&  this.mapController.canvas.height > extendedHeight)
                {
                    newZoom = this.mapController.getZoom().scaleX*100; // scaleX e scaleY sono uguali
                    console.log('findBestZoom: zoom is not changing', newZoom);
                }
                else
                {
                    var scaleX = this.mapController.canvas.width / extendedWidth;

                    scaleX = Math.min(1, scaleX);

                    var scaleY = this.mapController.canvas.height / extendedHeight;

                    scaleY = Math.min(1, scaleY);
                    console.log(Math.min(scaleX, scaleY) * 100);
                    newZoom = Math.round((Math.min(scaleX, scaleY) * 100));

                    this.mapController.setZoom(newZoom);
                    console.log('findBestZoom: zoom is changing', newZoom);
                    /* then it has to call a callback to set the zoom in the zoom controller */
                    nativeApp.onjs_zoomChanged(newZoom);

                }

            }else{
                newZoom = 100;
                console.log(newZoom);
                this.mapController.setZoom(newZoom);
                /* then it has to call a callback to set the zoom in the zoom controller */
                nativeApp.onjs_zoomChanged(newZoom);
            }

        }
    };



    App.prototype.fixCanvasSize = function() {
        var canvas = this.$canvas[0];
        canvas.width = (window.innerWidth-20)*window.devicePixelRatio;
        canvas.height = (window.innerHeight)*window.devicePixelRatio;
        if (this.mapController){
            // esiste il mapController
            this.mapController.calculateMapExtend();
            var maxWidth = this.mapController.mapExtend.maxX*this.mapController.getZoom().scaleX/window.devicePixelRatio;
            var maxHeight = this.mapController.mapExtend.maxY*this.mapController.getZoom().scaleY/window.devicePixelRatio;
            //console.log(maxWidth);
            //console.log(maxHeight);
            if (canvas.width<maxWidth)
                canvas.width = maxWidth;
            if (canvas.height<maxHeight)
                canvas.height = maxHeight;
        }
        this.$canvas.width(canvas.width/window.devicePixelRatio);
        this.$canvas.height(canvas.height/window.devicePixelRatio);
    };


    App.prototype.hideTitle = function(){
        $('.titleBar').hide();
        $('#mapCanvas').css('margin-top', 0);    
        //this.placeCommentInAllNodes(this.currentMapId, this.mapController.getZoom().scaleX, this.mapController.getZoom().scaleY);
    }

    App.prototype.printMap = function()
    {
        var img = new Image;

        img.id = 'tempPrintImage';    /// give an id so we can remove it in next step
        img.onload = print;           /// onload handler triggers your print method
        img.src = canvas.toDataURL(); /// set canvas image as source
        document.body.appendChild(img);

        function print()
        {
            console.log("printing...")
        }

    };


    /* perform the print out of a canvas */
    App.prototype.getImagePrint = function()
    {

        var output= "";

        var canvas = this.$canvas[0];
        if (this.mapController){

            var oldmapControllerOffset = this.mapController.stageOffset;
            var oldZoom = this.mapController.getZoom();
            var oldInteractionMode = this.mapController.interactionMode;
            this.modeDirect();
            this.mapController.stageOffset = {x:0,y:0};
            this.mapController.stage.x = 0;
            this.mapController.stage.y = 0;
            this.mapController.calculateMapExtend();
            this.mapController.setZoom(100);


            var maxWidth = this.mapController.mapExtend.maxX/window.devicePixelRatio;
            if (maxWidth == 0){
                maxWidth = (window.innerWidth)*window.devicePixelRatio;
            }

            var lineHeight = 22;
            var x = 0;
            var y = 60;
            var text = this.getMapTitle();
            var context = canvas.getContext('2d');

            var offSetY = this.wrapText(context, text, x, y, maxWidth, lineHeight, false);

            var maxHeight = (this.mapController.mapExtend.maxY+offSetY)/window.devicePixelRatio;

            if (canvas.width<maxWidth)
                canvas.width = maxWidth;
            if (canvas.height<maxHeight)
                canvas.height = maxHeight;


            this.$canvas.width(canvas.width);
            this.$canvas.height(canvas.height);

            this.mapController.stageOffset = {x:0,y:offSetY};
            this.mapController.stage.x = 0;
            this.mapController.stage.y = offSetY;
            this.mapController.redraw();

            x = canvas.width / 2;
            this.wrapText(context, text, x, y, maxWidth, lineHeight, true);
            output = canvas.toDataURL("image/png");

            /* ridisegno il livello di zoom */
            /**/


            this.mapController.stageOffset = oldmapControllerOffset;
            this.mapController.stage.x = oldmapControllerOffset.x;
            this.mapController.stage.y = oldmapControllerOffset.y;
            this.setZoom(oldZoom.scaleX/window.devicePixelRatio*100);
            this.mapController.redraw();
            if (oldInteractionMode == MapController.MODE.HAND){
                this.modeHand();

            }


        }

        if (!this.isDesktopDevice()) {
            return output;
        }
        else {
            this.sendMapAsImage(output);
        }
    };


    App.prototype.wrapText = function(context, text, x, y, maxWidth, lineHeight, print) {
        var words = text.split(' ');
        var line = '';
        context.font = 'bold 18px BooktabBaseFont';
        context.fillStyle = '#9B9B9B';
        context.textAlign = 'center';
        for(var n = 0; n < words.length; n++) {
            var testLine = line + words[n] + ' ';
            var metrics = context.measureText(testLine);
            var testWidth = metrics.width;

            if (testWidth > maxWidth && n > 0) {
                if (print=true) context.fillText(line, x, y);
                line = words[n] + ' ';
                y += lineHeight;
            }
            else {
                line = testLine;
            }
        }

        if (print=true) context.fillText(line, x, y);
        return y;
    };

    App.prototype.escapeInputFromApp = function(_itm)
    {
        if (typeof _itm == "string"){
            var regex = /\s*style\s*=\s*"([^"]*)"\s*/gi;
            _itm = _itm.replace(regex, " style=\\\"$1\\\"");
        }
        return _itm;
    }

    /**
     * Used by undo/redo queue to communicate an object creation
     * @param {_itm} item json definition
     */
    App.prototype.createVariant = function(_itm)
    {
        console.log ('RECEIVED A CREATE VARIANT FROM APP. The json is below');
        console.log(_itm);
        _itm = this.escapeInputFromApp(_itm);
        var obj = JSON.parse(_itm);
        if(obj.nodeData !== undefined) {
            if(obj.nodeData.textProperties && obj.nodeData.textProperties.editing)
                obj.nodeData.textProperties.editing = false;
            console.log('the create variant is for a node');
            this.mapController.addNode(obj);
            this.nodes[this.nodesCount] = obj.id;
            this.nodesCount++;
            this.lastNodeData.penColor = obj.nodeData.penColor;
        }
        else // edge:
        {
            console.log('the create variant is for an edge');
            this.mapController.addEdge(obj);
            this.lastEdgeData.penColor = obj.edgeData.penColor;
            if (this.device == App.NATIVE_APP_DEVICE.IOS)
            {
                var mc = this.mapController;
                setTimeout(function(){
                    mc.redraw();
                }, 0);
            }
        }
        if (this.device != App.NATIVE_APP_DEVICE.IOS)
        {
            this.mapController.redraw();
        }
        console.log ('CREATE VARIANT FROM APP DONE!');
    };

    /**
     * Used by undo/redo queue to communicate an object update
     * @param {_itm} item json definition
     */
    App.prototype.updateVariant = function(_itm)
    {
        console.log ('RECEIVED AN UPDATE VARIANT FROM APP. The json is below');
        console.log(_itm);
        _itm = this.escapeInputFromApp(_itm);
        var obj = JSON.parse(_itm);
        if(obj.nodeData !== undefined) {
            if(obj.nodeData.textProperties && obj.nodeData.textProperties.editing)
                obj.nodeData.textProperties.editing = false;
            console.log('the update variant is for a node');
            $('#comment_contents .commentReadBtn.' + obj.id).remove();
            this.updateExistingNode(null, obj.id,  obj.image.name);
            if (this.device == App.NATIVE_APP_DEVICE.IOS)
            {
                this.mapController.updateNodeForVariant(obj);
            }
            else
            {    
                this.mapController.updateNode(obj);
            }
            this.mapController.updateEdgesLinkedTo(obj.id)
            this.lastNodeData.penColor = obj.nodeData.penColor;
            if (this.device == App.NATIVE_APP_DEVICE.IOS)
            {
                var mc = this.mapController;
                setTimeout(function(){
                    mc.redraw();
                }, 0);
            }
        }
        else //edge:
        {
            console.log('the update variant is for an edge');
            this.mapController.updateEdgeFromJson(obj);
            this.lastEdgeData.penColor = obj.edgeData.penColor;
            if (this.device == App.NATIVE_APP_DEVICE.IOS)
            {
                var mc = this.mapController;
                setTimeout(function(){
                    mc.redraw();
                }, 0);
            }
        }
        
        // NB: per effetto del timeout, probabilmente i due pezzi di codice non si possono uniformare.
        if (this.device != App.NATIVE_APP_DEVICE.IOS)
        {
            this.mapController.redraw();    
        }
        
        console.log ('UPDATE VARIANT FROM APP DONE!');
    };

    /**
     * Used by undo/redo queue to communicate an object deletion
     * @param {_itm} item json definition
     */
    App.prototype.deleteVariant = function(_itm)
    {
        console.log ('RECEIVED A DELETE VARIANT FROM APP. The json is below');
        console.log(_itm);
        _itm = this.escapeInputFromApp(_itm);
        var obj = JSON.parse(_itm);
        if(obj.nodeData !== undefined) {
            console.log('the delete variant is for a node');
            var node = this.mapController.map.nodes[obj.id];
            this._innerDeleteNode(node);
            this.hideEditor();
        }
        else // edge:
        {
            console.log('the delete variant is for an edge');
            $("#edge_content_"+obj.id).remove();
            this.mapController.removeEdgeView(obj.id);
            this.mapController.removeEdge(obj.id);
            this.hideLinkEditor();
            if (this.device == App.NATIVE_APP_DEVICE.IOS)
            {
                setTimeout(function(){
                    mc.redraw();
                },0);
            }
        }

        if (this.device != App.NATIVE_APP_DEVICE.IOS)
        {
            this.mapController.redraw();    
        }
        console.log ('DELETE VARIANT FROM APP DONE!');
    }

    /* Converts a map to Json and sends it to the nativeApp
     */
    App.prototype.sendJson = function(mapId) {
        if (nativeApp.onjs_toJson) {
            nativeApp.onjs_toJson(
                JSON.stringify(this.getMapController(mapId).toJson(
                    mapId))
            );
        }
    };

    /* Sends some map properties to the native app */
    App.prototype.sendMapProps = function() {
        if (nativeApp.onjs_getMapProps) {
            nativeApp.onjs_getMapProps(JSON.stringify({
                empty: this.isEmpty(),
                mapId: this.currentMapId
            }));
        }
    };

    /* Sends the node id to the native app after creating it */
    App.prototype.sendNodeId = function(nodeId, isNodeFromPoint) {
        if (nativeApp.onjs_getNodeId) {
            nativeApp.onjs_getNodeId(nodeId, isNodeFromPoint);
        }
    };

    /* Sends the map as an image (as data) to the native app for printing */
    App.prototype.sendMapAsImage = function(imageData) {
        if (nativeApp.onjs_getMapAsImage) {
            nativeApp.onjs_getMapAsImage(imageData);
        }
    };

    App.prototype.refreshNode = function(mapId, nodeId)
    {
        this.mapController = this.getMapController(mapId);
        if (this.mapController)
        {
            // devo trovare il nodo della mappa corrispondente
            //console.log(this.mapController.mapView);
            for (mynodeId in this.mapController.mapView.nodeViews)
            {
                if (mynodeId == nodeId) {
                    this.mapController.mapView.nodeViews[mynodeId].updateWidget();
                    break;
                }
            }
        }
    };

    App.prototype.isDesktopDevice = function()
    {
        return (this.device == App.NATIVE_APP_DEVICE.DESKTOP);
    };

    return App;
});
