/**
 * Created by fabrizio on 02/04/14.
 */

define(['jquery','mapUtils'], function ($,MapUtils) {

    var Edge = function (json)
    {
        // Edge properties
        this.id = "";
        this.directed = false;
        this.source = "";
        this.target = "";
        this.edgeData = {
            type: Edge.EDGETYPE.NONE,
            penColor: "#0000FF",
            penColorId: 1,
            penWidth: Edge.PENWIDTH.WIDTH4,
            drawOver: false,
            curve: false,
            ctrlPx: 0,
            ctrlPy: 0,
            language: "",
            textProperties: {
                bold: false,
                italic: false
            }
        };
        this.titleText = "";

        // Transient data (not to be JSON-ized)
        this.sourceRef = null; // Reference to source model object
        this.targetRef = null; // Reference to target model object
        // Load data (if passed)
        if (json)
        {
            this.loadJson(json)
        }

        var me = this;
        function _inner(_obj, _name)
        {
            Object.defineProperty(_obj,_name,{
                get: function(){
                    return _obj["_"+_name];
                },
                set: function(newVal){
                    var oldJson = JSON.stringify(me.toJson());
                    _obj["_"+_name] = newVal;
                    var newJson = JSON.stringify(me.toJson());
                    me.notifyChanged(oldJson,newJson);
                },
                configurable: true
            });
        };
        function configureNestedProperty(obj){
            if( typeof(obj) != "object") return;
            for(var name in obj)
            {
                if(name.slice(0,1) != "_" && obj.hasOwnProperty(name))
                {
                    obj["_"+name] = obj[name];
                    _inner(obj,name);
                    configureNestedProperty(obj["_"+name]);
                }
            }
        }

        configureNestedProperty(this);


    }

    Edge.prototype.notifyChanged = function(oldJson,newJson)
    {
        //console.log("edgeChanged",oldJson,newJson);
        $("#edge_content_"+this.id).trigger("edgeChanged",oldJson,newJson);
    }


    Edge.prototype.jsonFilter = function(key, value)
    {
        if (key == "sourceRef" || key == "targetRef")
        {
            return undefined;
        } else
        {

        if (key.slice(0,1) == "_") {
            return undefined;
        } else {
            return value;
        }

        }
    };

    Edge.prototype.loadJson = function (json)
    {
        if (json.id) { this.id = json.id }
        if (json.directed) { this.directed = json.directed }
        if (json.source) { this.source = json.source }
        if (json.target) { this.target = json.target }
        if (json.edgeData) { $.extend(this.edgeData,json.edgeData) }
        if (json.titleText) {
            this.titleText = json.titleText;
        }else {
            this.titleText = "";
        }
        // settings the defaults, i.e. the colors
        this.setPenWidth(this.edgeData.penWidth);
        this.setPenColor(this.edgeData.penColor);

    }

    Edge.prototype.toJson = function()
    {
        var jsonString = JSON.stringify(this,this.jsonFilter,2);
        return JSON.parse(jsonString);
    }

    Edge.prototype.setTextPoint = function (srcP,tgtP,x,y)
    {
        var p = {x: x, y: y};
        var ctrlP = MapUtils.prototype.getControlPoint(srcP,tgtP,p);
        this.edgeData.ctrlPx = ctrlP.x;
        this.edgeData.ctrlPy = ctrlP.y;
    }

    Edge.prototype.setCtrlPoint = function (srcP,tgtP,x,y)
    {
        var p = {x: x, y: y};
        var ctrlP = p;
        this.edgeData.ctrlPx = ctrlP.x;
        this.edgeData.ctrlPy = ctrlP.y;
    }


    Edge.prototype.setPenWidth = function(width){
        switch (width) {
            case Edge.PENWIDTH.DASH:
            case Edge.PENWIDTH.WIDTH1:
            case Edge.PENWIDTH.WIDTH2:
            case Edge.PENWIDTH.WIDTH3:
            case Edge.PENWIDTH.WIDTH4:
                break;
            default:
                width = Edge.PENWIDTH.WIDTH4;
                break;
        }
        this.edgeData.penWidth = width;
    }


    Edge.prototype.setPenColor = function(rgb){
        /* has to find if it is rgb or rgba and then check if it in the appropriate form */
        rgb = rgba2rgb(rgb);
        penColorId = 1;
        switch (rgb){
            case Edge.BRUSHCOLORS.GRAY:
                penColorId = 1;
                break;
            case Edge.BRUSHCOLORS.PURPLE:
                penColorId = 2;
                break;
            case Edge.BRUSHCOLORS.RED:
                penColorId = 3;
                break;
            case Edge.BRUSHCOLORS.DARK_ORANGE:
                penColorId = 4;
                break;
            case Edge.BRUSHCOLORS.ORANGE:
                penColorId = 5;
                break;
            case Edge.BRUSHCOLORS.YELLOW:
                penColorId = 6;
                break;
            case Edge.BRUSHCOLORS.GREEN:
                penColorId = 7;
                break;
            case Edge.BRUSHCOLORS.DARK_GREEN:
                penColorId = 8;
                break;
            case Edge.BRUSHCOLORS.CYAN:
                penColorId = 9;
                break;
            case Edge.BRUSHCOLORS.BLUE:
                penColorId = 10;
                break;
                break;
            default:
                rgb = Edge.BRUSHCOLORS.GRAY;
                break;
        }
        this.edgeData.penColor = rgb;
        this.edgeData.penColorId = penColorId;
    }


    // Static Enums --------------------------------------------------------------------------------
    Edge.EDGETYPE =
    {
        NONE: "none",
        FORWARD: "forward",
        BACKWARD: "backward",
        BOTH: "both"
    }

    Edge.PENWIDTH =
    {
        DASH: "dash",
        WIDTH1: "1",
        WIDTH2: "2",
        WIDTH3: "3",
        WIDTH4: "4"
    }
    // ---------------------------------------------------------------------------------------------


    // default colors for the app
    Edge.BRUSHCOLORS = {
        GRAY:           rgba2rgb("rgb(189, 195, 199)"),
        PURPLE:         rgba2rgb("rgb(155, 89, 182)"),
        RED:            rgba2rgb("rgb(208, 2, 27)"),
        DARK_ORANGE:    rgba2rgb("rgb(230, 126, 34)"),
        ORANGE:         rgba2rgb("rgb(243, 156, 18)"),
        YELLOW:         rgba2rgb("rgb(248, 231, 28)"),
        GREEN:          rgba2rgb("rgb(153, 204, 51)"),
        DARK_GREEN:     rgba2rgb("rgb(46, 204, 113)"),
        CYAN:           rgba2rgb("rgb(51, 187, 204)"),
        BLUE:           rgba2rgb("rgb(41, 128, 185)")
    }


    return Edge;

})
