/**
 * Created by fabrizio on 02/04/14.
 */

define(['node','edge'], function (Node,Edge)
{

	var Map = function (json, assetBasePath)
	{
		// Map properties
        this._assetBasePath = assetBasePath;
		this.map =
		{
			mapProps: {
 				mapName: "Titolo",
                _preMapName: "Titolo",
				author: "",
				className: "",
				date: "",
				comments: ""
			}
		}
		// Nodes collection
		this.nodes = {}; // Map id -> model
		// Edges collection
		this.edges = {}; // Map id -> model
		// Images collection
		this.images = {};
		// Load a JSON Map data
		if (json)
		{
			this.loadJson(json);
		}
	}

	Map.prototype.loadJson = function(json)
	{

		if (!json) throw "Map.loadJson: json is null";
		// Get map properties
		this.map.mapProps = json.mapProps;
		// Create node models
		if (json.nodes)
		{
			for (var i = 0; i < json.nodes.length; i++)
			{
				this.addNode(json.nodes[i]);

			}
		}
		// Create edge models
		if (json.edges)
		{
			for (var i = 0; i < json.edges.length; i++)
			{
				this.addEdge(json.edges[i]);
			}
		}


	}

	Map.prototype.toJson = function()
	{
		var json = {};
		// Add map properties
		json.mapProps = this.map.mapProps;
		json.nodes = [];
		json.edges = [];
		// Add nodes
		for (var nodeId in this.nodes)
		{
			var node = this.nodes[nodeId];
			json.nodes.push(node.toJson());
		}
		// Add edges
		for (var edgeId in this.edges)
		{
			var edge = this.edges[edgeId];
			json.edges.push(edge.toJson());
		}
		return json;
	}

	Map.prototype.addNode = function(json)
	{
		if (!json) throw "Map.addNode: json is null";
		var node = new Node(json, this._assetBasePath);
        this.nodes[node.id] = node;
		if(node.image && node.image.name)
			this.images[node.id] = node.image;
		return node;
	}

	Map.prototype.addEdge = function(json)
	{
		if (!json) throw "Map.addEdge: json is null";
		var edge = new Edge(json);

		this.edges[edge.id] = edge;
		return edge;
	}

	return Map;

})
