/**
 * Created by fabrizio on 02/04/14.
 */



define(['jquery'], function($) {


	var Node = function(json, assetBasePath) {

		this.id = "";
		this.nodeData = {
			nodeType: Node.NODETYPE.RECTANGLE,
			frame: false,
			textLayout: Node.TEXTLAYOUT.INSIDE,
			textProperties: {
				bold: false,
				italic: false,
				size: Node.LABELSIZE.DEFAULT,
				editing: false
			},
			brushColor: Node.BRUSHCOLORS.WHITE,
			penColor: Node.BRUSHCOLORS.GRAY,
			penColorId: 1,
			penWidth: Node.PENWIDTH.WIDTH4,
			children: 0,
			language: "",
			mode: Node.MODE.CHILD,
			deepFillColor: Node.BRUSHCOLORS.WHITE,
			borderRadius: 5
		};

		this.nodeTable = {
			tableID: "",
			tableRow: -1,
			tableCol: -1
		};
		this.geometry = {
			x: 0,y: 0,
			width: 0,height: 0,
			x1: 0, y1: 0,
			x2: 0, y2: 0,
			imgW: 0, imgH: 0,
			txtW: 0, txtH: 0
		};
		this.image = {
			name: "",
			transparent: false,
			custom: false
		};
		this.titleText = "";
		this.deepText = "";
		this.links = [];
		this.comment = {
			text: "",
			editing: false
		};

		// Load json if passed
		if (json) {
			this.loadJson(json);
		}

		this._assetBasePath = assetBasePath;

        var me = this;
                    function _inner(_obj, _name)
                    {
                        Object.defineProperty(_obj,_name,{
                                get: function(){
                                    return _obj["_"+_name];
                                },
                                set: function(newVal){
                                    var oldJson = JSON.stringify(me.toJson());
                                    _obj["_"+_name] = newVal;
                                    var newJson = JSON.stringify(me.toJson());
                                    me.notifyChanged(oldJson,newJson);
                                },
                                configurable: true
                            });
                    };
        function configureNestedProperty(obj){
            if( typeof(obj) != "object") return;
            for(var name in obj)
            {
                if(name.slice(0,1) != "_" && obj.hasOwnProperty(name))
                {
                    obj["_"+name] = obj[name];
                    _inner(obj,name);
                    configureNestedProperty(obj["_"+name]);
                }
            }
        }

        configureNestedProperty(this);


	};

    Node.prototype.notifyChanged = function(oldJson,newJson)
    {
        //console.log("nodeChanged",oldJson,newJson);
        //$(document).trigger("nodeChanged",oldJson,newJson);
        $("#node_content_"+this.id).trigger("nodeChanged",oldJson,newJson);
    }

	Node.prototype.loadJson = function(json) {
		if (json.id) { this.id = json.id; }
		if (json.nodeData) { $.extend(this.nodeData,json.nodeData); }
		if (json.nodeTable) { $.extend(this.nodeTable,json.nodeTable); }
		if (json.geometry) { $.extend(this.geometry,json.geometry); }
		if (json.image) { $.extend(this.image,json.image); }
		if (json.titleText) {
            this.titleText = json.titleText;
        }else {
            this.titleText = "";
        }
        if (json.deepText) {
            this.deepText = json.deepText;
        }else {
            this.deepText = "";
        }

		if (json.links) { this.links = json.links; }

        /* here we set the default values
         * we force to map input json from exported map into well defined colors and penwidth */
        this.setPenColor(this.nodeData.penColor);
        this.setBrushColor(this.nodeData.penColor, this.nodeData.mode);
        this.setPenWidth(this.nodeData.penWidth);
        this.setNodeType(this.nodeData.nodeType);
        this.setTextLayout(this.nodeData.textLayout);

	};

	Node.prototype.jsonFilter = function(key, value)
	{
		if (key.slice(0,1) == "_") {
			return undefined;
		} else {
			return value;
		}
	}

	Node.prototype.toJson = function() {
		var jsonString = JSON.stringify(this,this.jsonFilter,2);
		return JSON.parse(jsonString);
	};

	Node.prototype.getCenter = function() {
		return {x: this.geometry.x, y: this.geometry.y};
	};

	Node.prototype.setCenter = function(x, y) {
		// Calculate movement
		var deltaX = x - this.geometry.x;
		var deltaY = y - this.geometry.y;
		// Apply deltas
		this.geometry.x += deltaX;
		this.geometry.y += deltaY;
		/*this.geometry.x1 += deltaX;
		this.geometry.y1 += deltaY;
		this.geometry.x2 += deltaX;
		this.geometry.y2 += deltaY;*/
	};

	Node.prototype.getSize = function() {
		return {width: this.geometry.width, height: this.geometry.height};
	};

	Node.prototype.setSize = function(width, height) {
		this.geometry.width = width;
		this.geometry.height = height;
		//this.geometry.x1 = this.geometry.x-(this.geometry.width/2);
		//this.geometry.x2 = this.geometry.x+(this.geometry.width/2);
		// TODO: consider node layout and move x1/x2/y1/y2
	};

    // given a rgb string return the color, if rgb is not in the set
    Node.prototype.setPenColor = function(rgb){
        /* has to find if it is rgb or rgba and then check if it in the appropriate form */
        rgb = rgba2rgb(rgb);
        this.nodeData.penColorId = 1;

        switch (rgb){
            case Node.BRUSHCOLORS.GRAY:
                this.nodeData.penColorId = 1;
                break;
            case Node.BRUSHCOLORS.PURPLE:
                this.nodeData.penColorId = 2;
                break;
            case Node.BRUSHCOLORS.RED:
                this.nodeData.penColorId = 3;
                break;
            case Node.BRUSHCOLORS.DARK_ORANGE:
                this.nodeData.penColorId = 4;
                break;
            case Node.BRUSHCOLORS.ORANGE:
                this.nodeData.penColorId = 5;
                break;
            case Node.BRUSHCOLORS.YELLOW:
                this.nodeData.penColorId = 6;
                break;
            case Node.BRUSHCOLORS.GREEN:
                this.nodeData.penColorId = 7;
                break;
            case Node.BRUSHCOLORS.DARK_GREEN:
                this.nodeData.penColorId = 8;
                break;
            case Node.BRUSHCOLORS.CYAN:
                this.nodeData.penColorId = 9;
                break;
            case Node.BRUSHCOLORS.BLUE:
                this.nodeData.penColorId = 10;
                break;
            case Node.BRUSHCOLORS.WHITE:
                this.nodeData.penColorId = 11;
                break;
                                break;
            default:
                rgb = Node.BRUSHCOLORS.GRAY;
                this.nodeData.penColorId = 1;
                break;
        }
        this.nodeData.penColor = rgb;
    }

    Node.prototype.setBrushColor = function(rgb, mode){
        this.nodeData.brushColor = rgb2rgba(rgb, mode);
    }

    Node.prototype.setPenWidth = function(width){
        switch (width) {
            case Node.PENWIDTH.DASH:
            case Node.PENWIDTH.WIDTH1:
            case Node.PENWIDTH.WIDTH2:
            case Node.PENWIDTH.WIDTH3:
            case Node.PENWIDTH.WIDTH4:
                    break;
            default:
                width = Node.PENWIDTH.WIDTH4;
                break;
        }
        this.nodeData.penWidth = width;
    }


    Node.prototype.setNodeType = function(nodeType){
        /* il mapping has to be */
        /* rectangle/roundrect
            ellipse
            image -> roundrect
            text -> roundrect white brushcolor, and white pen
         */

        switch(nodeType){
            case Node.NODETYPE.IMAGE:
            case Node.NODETYPE.RECTANGLE:
                this.nodeData.nodeType = Node.NODETYPE.ROUNDRECT;
                break;
            case Node.NODETYPE.TEXT:
                this.nodeData.nodeType = Node.NODETYPE.ROUNDRECT;
                this.setPenColor(Node.BRUSHCOLORS.WHITE);
                this.setBrushColor(Node.BRUSHCOLORS.WHITE);
                break;
        }

    }

    Node.prototype.setTextLayout = function(textLayout){
        /* always inside... may be in the future it will be changed */
        this.nodeData.textLayout = 'inside';
    }

    Node.prototype.setTextMaxSize = function(deltaW, deltaH){
        !deltaW ? deltaW = Node.DEFAULT_DELTA_TEXT.deltaW : deltaW = deltaW;
        !deltaH ? deltaH = Node.DEFAULT_DELTA_TEXT.deltaH : deltaH = deltaH;
        this.geometry.txtW = this.geometry.width - deltaW;
        this.geometry.txtH = this.geometry.height - deltaH;
    }

	// Static Enums --------------------------------------------------------------------------------

	Node.NODETYPE = {
		IMAGE: "image",
		RECTANGLE: "rectangle",
		ROUNDRECT: "roundrect",
		ELLIPSE: "ellipse",
		TEXT: "text"
	};

	Node.TEXTLAYOUT = {
		NONE: "none",
		INSIDE: "inside",
		UNDER: "under",
		OVER: "over"
	};

	Node.PENWIDTH = {
		DASH: "dash",
		WIDTH1: "1",
		WIDTH2: "2",
		WIDTH3: "3",
		WIDTH4: "4"
	};

	Node.MODE = {
		ROOT: true,
		CHILD: false
	};

	Node.LABELSIZE = {
		MAX: 5,
		DEFAULT: 3
	};

    // default colors for the app
    Node.BRUSHCOLORS = {
        GRAY:           rgba2rgb("rgb(189, 195, 199)"),
        PURPLE:         rgba2rgb("rgb(155, 89, 182)"),
        RED:            rgba2rgb("rgb(208, 2, 27)"),
        DARK_ORANGE:    rgba2rgb("rgb(230, 126, 34)"),
        ORANGE:         rgba2rgb("rgb(243, 156, 18)"),
        YELLOW:         rgba2rgb("rgb(248, 231, 28)"),
        GREEN:          rgba2rgb("rgb(153, 204, 51)"),
        DARK_GREEN:     rgba2rgb("rgb(46, 204, 113)"),
        CYAN:           rgba2rgb("rgb(51, 187, 204)"),
        BLUE:           rgba2rgb("rgb(41, 128, 185)"),
        WHITE:          rgba2rgb("rgb(255, 255, 255)")
    }

    Node.LABELCOLOR = {
        ROOT: 'labelROOT',
        CHILD: 'labelCHILD'
    }


    Node.DEFAULT_SIZE = {
        width: 250,
        height: 80
    }

    Node.DEFAULT_DELTA_TEXT = {
        deltaW: 10,
        deltaH: 10
    }

	// ---------------------------------------------------------------------------------------------

	return Node;

});
