/**
 * Created by fabrizio on 16/04/14.
 */

define (['node'], function(Node) {

	// Geometry calc service

	var GeometryService = function()
	{
	};

    /**
     * Calculate both intersection between the edge line and the node border of the source and target nodes
     * @param edgeView
     * @param sourceNodeView
     * @param targetNodeView
     */
	GeometryService.prototype.calculateEdgeIntersections = function(edgeView,sourceNodeView,targetNodeView)
	{
		if (edgeView && sourceNodeView && targetNodeView)
		{
			// Calculate REAL intersections... All in stage coordinates
			edgeView.sourceP = this.calculateEdgeIntersection(edgeView,sourceNodeView,targetNodeView);
			edgeView.targetP = this.calculateEdgeIntersection(edgeView,targetNodeView,sourceNodeView);
		}
	};

    /**
     * Calculate the intersection between the edge line and node border
     * @param edgeView is the representation of the edge
     * @param thisNodeView is the representation of the node we are calculating the intersection
     * @param otherNodeView is the representation of the node the edge comes from
     * @returns {{ok: boolean, x: number, y: number}} the calculated intersection and a state flag.
     */
    GeometryService.prototype.calculateEdgeIntersection = function(edgeView,thisNodeView,otherNodeView)
	{
		var result = {ok:true, x:0, y:0};
		var center;
		var lorP;
		var loeP;
		// Check if parameters view and relative models are assigned
		if (edgeView && edgeView.edge &&
			thisNodeView && thisNodeView.node &&
			otherNodeView && otherNodeView.node)
		{
			var edge = edgeView.edge;
			var thisNode = thisNodeView.node;
			//
			var extP = {x:0, y:0};
			if (edge.edgeData.curve) {
				extP = {x: edge.edgeData.ctrlPx, y: edge.edgeData.ctrlPy};
			} else {
				extP = otherNodeView.getCenter();
			}
			center = thisNodeView.getCenter();
			var size = thisNodeView.getSize();
			size.width = size.width/2;
			size.height = size.height/2;
			var radius = 0;
			if (thisNode.nodeData.nodeType==Node.NODETYPE.ROUNDRECT) {
				radius = thisNodeView.roundRadius;
			}
			if (thisNode.nodeData.textLayout==Node.TEXTLAYOUT.UNDER ||
				thisNode.nodeData.textLayout==Node.TEXTLAYOUT.OVER) {
				center = thisNodeView.getWidgetCenter();
				lorP = this.getLineOnRectPoint(extP.x,extP.y,center.x,center.y,size.width,size.height,radius);
				if (!lorP.ok) {
					result.ok = false;
				} else {
					result.ok = true;
					result.x = lorP.x;
					result.y = lorP.y;
				}
			} else {
				if (thisNode.nodeData.nodeType==Node.NODETYPE.ELLIPSE) {
					loeP = this.getLineOnEllipsePoint(extP.x,extP.y,center.x,center.y,size.width,size.height);
					if (!loeP) {
						result.ok = false;
					} else {
						result.ok = true;
						result.x = loeP.x;
						result.y = loeP.y;
					}
				} else {
					lorP = this.getLineOnRectPoint(extP.x,extP.y,center.x,center.y,size.width,size.height,radius);
					if (!lorP) {
						result.ok = false;
					} else {
						result.ok = true;
						result.x = lorP.x;
						result.y = lorP.y;
					}
				}
			}
		}
        else
        {
            console.log('calculateEdgeIntersection called with wrong params');
        }
		return result;
	};


    /**
     * Calculates the solution of 2nd degree equation with given parameters
     * @param a
     * @param b
     * @param c
     * @returns {{ok: boolean, x1: number, x2: number}}
     */
    //function TArrow.SolveEquation(A,B,C: extended; var x1,x2: extended): boolean;
	GeometryService.prototype.solveEquation = function(a,b,c)
	{
		var result = {ok:false, x1:0.0, x2:0.0};
		result.x1=b*b-4*a*c;
		if (result.x1>=0) {
			result.x1=Math.sqrt(result.x1);
			a=a*2;
			result.x2=(-b+result.x1)/a;
			result.x1=(-b-result.x1)/a;
			result.ok=true;
		}
		return result;
	};

    /**
     * TODO Fabri commentare e testare...
     * @param px
     * @param py
     * @param cx
     * @param cy
     * @param lx
     * @param ly
     * @returns {{ok: boolean, x: number, y: number}}
     */
	//function(pX,pY,cX,cY,lX,lY: integer; var RX,RY: integer): boolean;
	GeometryService.prototype.getLineOnEllipsePoint = function(px,py,cx,cy,lx,ly)
	{
		var m=0.0;
		var q=0.0;
		var lx2=0.0;
		var ly2=0.0;
		var result = {ok:true, x:0.0, y:0.0};
		if (py==cy) {
			result.y = cy;
			if (px<cx) {result.x=cx-lx} else {result.x=cx+lx}
		} else {
			if (px==cx) {
				result.x=cx;
				if (py<cy) {result.y=cy-ly} else {result.y=cy+ly}
			} else {
				result.ok=false;
				m=(py-cy)/(px-cx);
				lx2=lx*lx;
				ly2=ly*ly;
				if ((m*m*lx2+ly2)!=0) {
					q=lx2*ly2/(m*m*lx2+ly2);
					if (q>=0) {
						q=Math.sqrt(q);
						result.x=Math.round(q);
						result.y=Math.round(q*m);
						if (px<cx) {
							result.x=cx-result.x;
							result.y=cy-result.y;
						} else {
							result.x=cx+result.x;
							result.y=cy+result.y;
						}
						result.ok=true;
					}
				}
			}
		}
		return result;
	};


    /**
     * TODO Fabri commentare e testare...
     * @param x1
     * @param y1
     * @param x2
     * @param y2
     * @param cx
     * @param cy
     * @param r
     * @returns {{ok: boolean, ax: number, ay: number, bx: number, by: number}}
     */
	//function TArrow.GetLineOnCirclePoints(x1,y1,x2,y2,CX,CY,R: integer;	var AX,AY,BX,BY: integer): boolean;
	GeometryService.prototype.getLineOnCirclePoints = function(x1,y1,x2,y2,cx,cy,r)
	{
		var result = {ok:false, ax:0.0, ay:0.0, bx:0.0, by:0.0};
		var m =(y2-y1)/(x2-x1);
		var d =(m*(cx-x1))-(cy-y1);
		var solveEq = this.solveEquation(m*m+1,2*m*d,d*d-r*r);
		if (solveEq.ok)	{
			result.ax=cx+Math.round(solveEq.x1);
			result.ay=cy+Math.round(m*solveEq.x1+d);
			result.bx=cx+Math.round(solveEq.x2);
			result.by=cy+Math.round(m*solveEq.x2+d);
			result.ok=true;
		}
		return result;
	};
    /**
     * TODO Fabri commentare e testare...
     * @param px
     * @param py
     * @param cx
     * @param cy
     * @param lx
     * @param ly
     * @param r
     * @returns {{ok: boolean, x: number, y: number}}
     */
    //function TArrow.GetLineOnRectPoint(PX,PY,CX,CY,LX,LY,R: integer; var RX,RY: integer): boolean;
	GeometryService.prototype.getLineOnRectPoint = function(px,py,cx,cy,lx,ly,r)
	{
		var result = {ok:true, x:0.0, y: 0.0};
		var sx = 0;
		var sy = 0;
		var locP;
		if (py==cy) {
			result.y=cy;
			if (px<cx) {result.x=cx-lx} else {result.x=cx+lx}
		} else {
			if (px==cx) {
				result.x=cx;
				if (py<cy) {result.y=cy-ly} else {result.y=cy+ly}
			} else {
				if (px<cx) {sx=-1} else {sx=1}
				if (py<cy) {sy=-1} else {sy=1}
				result.x=cx+sx*ly*Math.abs(px-cx)/Math.abs(py-cy); // thrown away: ~~ : integer division (like delphi "div")
				if ((px<cx && result.x>=(cx-lx)) || (px>cx && result.x<=(cx+lx))) {
					if (py<cy) {result.y=cy-ly} else {result.y=cy+ly}
					if (r!=0) {
						if (Math.abs(result.x-cx)>(lx-r)) {
							result.ok=false;
							result.x=cx+sx*lx-sx*r;
							result.y=cy+sy*ly-sy*r;
							locP = this.getLineOnCirclePoints(cx,cy,px,py,result.x,result.y,r);
							if (locP.ok) {
								if (px<cx) {
									result.x=locP.ax;
									result.y=locP.ay;
								} else {
									result.x=locP.bx;
									result.y=locP.by;
								}
								result.ok=true;
							}
						}
					}
				} else {
					result.y=cy+(sy*lx*Math.abs(py-cy)/Math.abs(px-cx)); // ~~
					if (px<cx) {result.x=cx-lx} else {result.x=cx+lx;}
					if (r!=0) {
						if (Math.abs(result.y-cy)>(ly-r)) {
							result.ok=false;
							result.x=cx+sx*lx-sx*r;
							result.y=cy+sy*ly-sy*r;
							locP = this.getLineOnCirclePoints(cx,cy,px,py,result.x,result.y,r);
							if (locP.ok) {
								if (px<cx) {
									result.x=locP.ax;
									result.y=locP.ay;
								} else {
									result.x=locP.bx;
									result.y=locP.by;
								}
								result.ok=true;
							}
						}
					}
				}
			}
		}
		return result;
	};

//	GeometryService.prototype.sign = function(x) {
//		// Stack Overflow: megafast...
//		//return typeof x === 'number' ? x ? x < 0 ? -1 : 1 : x === x ? 0 : NaN : NaN;
//		// Stack Overflow: fastest
//		return x ? x < 0 ? -1 : 1 : 0;
//	}

	// Static Enums --------------------------------------------------------------------------------

	// ---------------------------------------------------------------------------------------------

	return GeometryService;

});