/**
 * Created by fabrizio on 27/03/14.
 */

define ([], function() {

	// Service static class for general purpose map calculations

	var MapUtils = function()
	{
	}

	/**
	 * Normalize the area given by 4 coordinates
	 * minimum values at top-left, maximum at bottom-right
	 * @param x1 number first x
	 * @param y1 number first y
	 * @param x2 number second x
	 * @param y2 number second y
	 * @returns normalized area {x1: number, y1: number, x2: number, y2: number}
	*/
	MapUtils.prototype.normalizeArea = function(x1,y1,x2,y2)
	{
		var res = {x1:0, y1:0, x2:0, y2:0};
		if (x1<=x2) {
			res.x1 = x1;
			res.x2 = x2;
		} else {
			res.x1 = x2;
			res.x2 = x1;
		}
		if (y1<=y2) {
			res.y1 = y1;
			res.y2 = y2;
		} else {
			res.y1 = y2;
			res.y2 = y1;
		}
		return res;
	}

	/**
	 * Extend an area to include the given point
	 * @param area initial area to extend {x1: number, y1: number, x2: number, y2: number}
	 * @x x number coordinate of the additional point
	 * @y y number coordinate of the additional point
	 * @returns extended area {x1: number, y1: number, x2: number, y2: number}
	 */
	MapUtils.prototype.extendArea = function(area,x,y)
	{
		if (x<area.x1) {
			area.x1 = x;
		}
		else if (x>area.x2)
		{
			area.x2 = x;
		}
		if (y<area.y1) {
			area.y1 = y;
		}
		else if (y>area.y2)
		{
			area.y2 = y;
		}
		return area;
	}

	/**
	 * Calculate text point from start, end and control points
	 * @param startP start point {x: number, y: number}
	 * @param endP end point {x: number, y: number}
	 * @param ctrlP control point {x: number, y: number}
	 * @returns {x: number, y: number} text point
	 */
	MapUtils.prototype.getTextPoint = function(startP,endP,ctrlP)
	{
		var textP = {x:0, y:0};
		if (startP && endP && ctrlP)
		{
			var avgP = {x: (startP.x+endP.x)/2, y:(startP.y+endP.y)/2};
			var dx = ctrlP.x-avgP.x;
			var dy = ctrlP.y-avgP.y;
			textP = {x: avgP.x+(dx*3/4), y: avgP.y+(dy*3/4)};
		}
		return textP;
	}

	/**
	 * Calculate control point from start, end and text points
	 * @param startP start point {x: number, y: number}
	 * @param endP end point {x: number, y: number}
	 * @param textP text point {x: number, y: number}
	 * @returns {x: number, y: number} control point
	 */
	MapUtils.prototype.getControlPoint = function(startP,endP,textP)
	{
		var ctrlP = {x:0, y:0};
		if (startP && endP && textP)
		{
			var avgP = {x: (startP.x+endP.x)/2, y:(startP.y+endP.y)/2};
			var dx = textP.x-avgP.x;
			var dy = textP.y-avgP.y;
			ctrlP = {x: avgP.x+(dx*4/3), y: avgP.y+(dy*4/3)};
		}
		return ctrlP;
	}

	return MapUtils;

})