/**
 * Created by fabrizio on 30/04/14.
 */

define ([], function() {

	// Text flowing service class

	var NL = "\n";
	var TEXT_NODE = 3;

	var TextService = function(easelContainer)
	{
		if (!easelContainer) throw "TextService: easelContainer is null!"
		this.easelContainer = easelContainer; // Text elements easel container (holds total width and height)
		this.shape = null;
		this.firstP = false;
		this.nextIsLi = false;
		// Output data structure
		this.words = []; // Words array (text,style,ascent,textWidget,shapeWidget)
		this.lines = []; //Lines array (height,width,maxAscent,textAlign,lineWidget,words[])

	};

	/**
	 * Set a new html element to be rendered and create text widgets
	 * @param htmlElement Html element to render
	 */
	TextService.prototype.setHtmlText = function (htmlElement)
	{
        //console.log("htmlElement",htmlElement)
		this.firstP = true;
		this.clear();
		this.orderedListCounter = 1;
		this.flattenHtmlText(htmlElement);
		this.createTextWidgets();
	};

	/**
	 * Clear data structure and relative widgets
	 */
	TextService.prototype.clear = function ()
	{
		this.clearContainer();
		this.clearData();
	}

	TextService.prototype.clearContainer = function ()
	{
		// Clear container
		this.easelContainer.removeAllChildren();
		// Add widget shape
		this.easelContainer.addChild(this.shape);
		// Clear line assignments (text must be reflown)
		this.lines = [];
	}

	TextService.prototype.clearData = function ()
	{
		// Clear data references (Garbage Collector, please save us!)
		this.words = [];
	}

	/**
	 * Convert a HTML text element into a words array
	 * @param htmlElement HTML element to parse
	 */

	TextService.prototype.flattenHtmlText = function(htmlElement, prevStyle)
	{
		if(!this.orderedListCounter) this.orderedListCounter = 1;
		if(htmlElement.nodeName.toLowerCase()=='ol'){
			this.orderedListCounter = 1;
		}
        //console.log("htmlElement--->",htmlElement)
		var computedStyle = window.getComputedStyle(htmlElement.parentNode);
		var curStyle = {};
		Object.keys(computedStyle).forEach(function(key){
			curStyle[key] = computedStyle[key];
		});

		if(prevStyle && prevStyle.textDecoration && prevStyle.textDecoration!="none" && curStyle.textDecoration=="none")
			curStyle.textDecoration = prevStyle.textDecoration;

		var curWords = null;
		var curWord = null;
		var j = 0;
		if (htmlElement.nodeType==TEXT_NODE)
		// HTML text element
		{
			if (this.nextIsLi)
			{
				this.nextIsLi = false;
				this.words.push({text: NL, style: curStyle, ascent: 0, textWidget: null, shapeWidget: null});
				var goingUp = true;
				var el = htmlElement;
				var isOL = false;
				while(goingUp){
					el = el.parentNode;
					if(!el) goingUp = false;
					else if(el.nodeName.toLowerCase()=='ol'){
						isOL = true;
						goingUp = false;
					}
				}
				if(isOL){
					this.words.push({text: this.orderedListCounter+". ", style: curStyle, ascent: 0, textWidget: null, shapeWidget: null});
					this.orderedListCounter++;
				}
				else
					this.words.push({text: "• ", style: curStyle, ascent: 0, textWidget: null, shapeWidget: null});
			}
			var text = htmlElement.nodeValue;
			if (curStyle.textTransform=="uppercase")
			{
				// Tutto maiuscolo
				text = text.toUpperCase();
			}
			// Split words
			curWords = text.split(" ");
			for (j=0; j<curWords.length; j++)
			{
                //console.log(curWords[j]);
                curWordWithoutBR=curWords[j].split(NL);
                //console.log(curWordWithoutBR);
                for (k=0; k<curWordWithoutBR.length; k++){
                    curWord = curWordWithoutBR[k];

                    //console.log(curWord);
                    if (curWord!="")
                    {
                            this.words.push({text: curWord, style: curStyle, ascent: 0, textWidget: null, shapeWidget: null});

                    }

                    if ((curWordWithoutBR.length>1 && k < curWordWithoutBR.length-1)){
                        this.words.push({text: NL, style: curStyle, ascent: 0, textWidget: null, shapeWidget: null});
                    }
                }

                if (j<curWords.length-1){
                    this.words.push({text: " ", style: curStyle, ascent: 0, textWidget: null, shapeWidget: null});
                }


			}
		}
		else if (htmlElement.nodeName.toLowerCase()=="br" ||
			(htmlElement.nodeName.toLowerCase()=="p") && !this.firstP)
		{
			// Push a line break
			this.words.push({text:NL,style:curStyle,ascent:0,textWidget:null,shapeWidget:null});
		}
		else if (htmlElement.nodeName.toLowerCase()=="p")
		{
			if (this.firstP)
			{
				this.firstP = false;
			}
			else
			{
				// Push a line break (after first <p>)
				this.words.push({text:NL,style:curStyle,ascent:0,textWidget:null,shapeWidget:null});
			}
		}
		else if (htmlElement.nodeName.toLowerCase()=="li")
		{
			// Manage list items (SM: only unordered lists)
			this.nextIsLi = true;
		}
		var children=htmlElement.childNodes;
		for (var i=0;i<children.length;i++)
		{
			// Recur on children list
			var child=children[i];
			this.flattenHtmlText(child, curStyle);
		}
	};

	/**
	 * Create text widgets (easeljs text widgets) grouped in line containers in the easelContainer
	 */
	TextService.prototype.createTextWidgets = function ()
	{
		// Total area shape
		this.shape = new createjs.Shape();
		// Create new widgets
		for (var i=0; i<this.words.length; i++)
		{
			var word = this.words[i];
			// Create text widget with constructor limited style
			var textWidget = new createjs.Text(word.text, word.style.font, word.style.color);

			// Create shape widget for extended styles
			var shapeWidget = new createjs.Shape();
			// Apply sizes from text widget
			shapeWidget.x = 0;
			shapeWidget.y = 0;
			shapeWidget.width = textWidget.getBounds().width;
			shapeWidget.height = textWidget.getBounds().height;
			// Base alignment for text elements
			textWidget.textBaseline = "middle"; // alphabetic needs to be shifted down, but ok for different font sizes!
			// Calculate ascent for current font style
			var ref = new createjs.Text("M",word.style,"#000000");
			word.ascent = ref.getBounds().width; // Approx: ascent="M".width (?!?)
			// Apply other styles to text
			textWidget.color = word.style.color;
			// Apply extended styles using the back shape
			// Background color

			//console.log("shapeWidgetColor->",word.style.backgroundColor)

			shapeWidget.graphics
				.setStrokeStyle(1,"round")
				.beginStroke(word.style.backgroundColor)
				.beginFill(word.style.backgroundColor)
				.drawRect(0,0,shapeWidget.width, shapeWidget.height);
			// Underline
			if (word.style.textDecoration && word.style.textDecoration.slice(0,9)=="underline")
			{
				var f = 1.5;
				if(shapeWidget.height>10) f = 1.2;
				if(shapeWidget.height>12) f = 0.9;
				if(shapeWidget.height>16) f = 0.8;
				if(shapeWidget.height>20) f = 0.7;
				if(shapeWidget.height>24) f = 0.65;
				if(shapeWidget.height>28) f = 0.6;
				if(shapeWidget.height>36) f = 0.55;
				if(shapeWidget.height>48) f = 0.5;
				if(shapeWidget.height>72) f = 0.4;

				shapeWidget.graphics
					.setStrokeStyle(1, "round")
					.beginStroke(word.style.color)
					.moveTo(0, shapeWidget.height*f)
					.lineTo(shapeWidget.width, shapeWidget.height*f);

			}
			// Strike through
			else if (word.style.textDecoration && word.style.textDecoration.slice(0,12)=="line-through")
			{
				var f = 1.5;
				if(shapeWidget.height>10) f = 1.2;
				if(shapeWidget.height>12) f = 0.9;
				if(shapeWidget.height>16) f = 0.8;
				if(shapeWidget.height>20) f = 0.7;
				if(shapeWidget.height>24) f = 0.65;
				if(shapeWidget.height>28) f = 0.6;
				if(shapeWidget.height>36) f = 0.55;
				if(shapeWidget.height>48) f = 0.5;
				if(shapeWidget.height>72) f = 0.4;

				f = f/2;

				var h = textWidget.getBounds().height;
				shapeWidget.graphics
					.setStrokeStyle(1, "round")
					.beginStroke(word.style.color)
					.moveTo(0, shapeWidget.height*f)
					.lineTo(shapeWidget.width, shapeWidget.height*f);
			}
			// Link to widgets
			word.textWidget = textWidget;
			word.shapeWidget = shapeWidget;
		}
	};

	/**	 *
	 * Arrange text inside a given width
	 * @param maxWidth maximum width for text reflowing/arranging
	 */
	TextService.prototype.arrangeText = function (maxWidth)
	{
		/*---------------------------------------------------------------------------*/
		var startNewLine = function ()
		{
			// Create new text line container
			currentLineContainer = new createjs.Container();
			currentLine = {
				width: 0, height: 0,
				maxAscent: 0,
				textAlign: null,
				lineWidget: currentLineContainer,
				words: []
			}
		};
		/*---------------------------------------------------------------------------*/
		var addWordToCurrentLine = function (word)
		{
			// Update text alignment (if not yet defined)
			if (!currentLine.textAlign)
			{
				currentLine.textAlign = word.style.textAlign;
			}
			// Update width
			currentLine.width += word.textWidget.getBounds().width;
			// Update height
			if (word.textWidget.getBounds().height > currentLine.height)
			{
				currentLine.height = word.textWidget.getBounds().height;
			}
			// Update max ascent
			if (word.ascent > currentLine.maxAscent)
			{
				currentLine.maxAscent = word.ascent;
			}
			// Add word to line words list
			currentLine.words.push(word);
			// Update line width
			currentLine.width = currentX + word.textWidget.getBounds().width;
			// Update total width
			if (currentX + word.textWidget.getBounds().width > totalWidth)
			{
				totalWidth = currentX + word.textWidget.getBounds().width;
			}
			if (currentX == 0 && word.text.trim() == "")
			{
				// First item in line is a space: make invisible
				word.textWidget.visible = false;
				word.shapeWidget.visible = true;
			}
			else
			{
				// Add widgets
				word.textWidget.visible = true;
				word.shapeWidget.visible = true;
				currentLine.lineWidget.addChild(word.shapeWidget);
				currentLine.lineWidget.addChild(word.textWidget);
				// Place widgets
				word.textWidget.x = currentX;
				word.textWidget.y = 0; // baseline: alphabetic
				word.shapeWidget.x = currentX;
				// Calculate Y offset for shape box
				word.shapeWidget.y = word.textWidget.y - word.ascent;
				// Update current line height
				if (word.textWidget.getBounds().height > word.textWidget.getBounds().height)
				{
					currentLine.height = word.textWidget.getBounds().height;
				}
				currentX += word.textWidget.getBounds().width;
			}
		};
		/*---------------------------------------------------------------------------*/
		var commitCurrentLine = function (me)
		{
			// Align current line container based on style
			// Update sizes and position for reflowing algorythm
			// Add line container to easel container
			me.easelContainer.addChild(currentLineContainer);
			currentLineContainer.y = currentY;
			if(currentLine.height>=24 && currentLine.height<50) currentLineContainer.y += currentLine.height/4;
			//if(currentLine.height>=30 && currentLine.height<40) currentLineContainer.y += currentLine.height/3;
			if(currentLine.height>=50) currentLineContainer.y += currentLine.height/2.5;
			if(currentY>0)
				currentLine.maxAscent = currentLine.height * 0.75;
			// Add current line
			me.lines.push(currentLine);
			// Update for next line
			currentX = 0;
			currentY += currentLine.height;
			totalHeight += currentLine.height;
		};
		/*---------------------------------------------------------------------------*/
		var alignVertContainer = function (line)
		{
			// Vertical alignment (max ascent)
			for (var i=0; i<line.words.length; i++)
			{
				var word = line.words[i];
				var deltaY = line.maxAscent;
				if (word.style.verticalAlign=="sub")
				{
					// Subscript
					var descent = word.textWidget.getBounds().height - word.ascent;
					deltaY += descent;

				}
				else if (word.style.verticalAlign=="super")
				{
					// Superscript
					deltaY -= word.textWidget.getBounds().height - word.ascent;
				}
				word.textWidget.y += deltaY;
				word.shapeWidget.y += deltaY;
			}
		};
		/*---------------------------------------------------------------------------*/
		var hideLastSpaces = function (line)
		{
			if (line.words.length>0)
			{
				var word = line.words[line.words.length-1];
				if (word.text.trim()=="")
				{
					// Spaces in last place: hide and compensate
					line.width -= word.textWidget.getBounds().width;
					word.textWidget.visible = false;
					word.shapeWidget.visible = false;
				}
			}
		};
		/*---------------------------------------------------------------------------*/
		var alignHorizContainer = function (line)
		{
			// Check for spaces in last place and avoid it
			hideLastSpaces(line);
			// Align text line
			var deltaX = 0;
			switch (line.textAlign)
			{
				case "left" :
				{
					deltaX = 0;
					break;
				}
				case "center" :
                {
					deltaX = (totalWidth - line.width) / 2;
					break;
				}
				case "right" :
				{
					deltaX = totalWidth - line.width;
					break;
				}
			}
			line.lineWidget.x = deltaX;
		};
		/*---------------------------------------------------------------------------*/
		// Remove old controls from easel container
		this.clearContainer();
		var currentY = 0;
		var currentX = 0;
		var totalWidth = 0;
		var totalHeight = 0;
		var i = 0;
		//
		this.easelContainer.width = 0;
		this.easelContainer.height = 0;
		var currentLine = null;
		var currentLineContainer = null;
		var previousIsNL = false;
		for (i = 0; i < this.words.length; i++)
		{
			if (!currentLineContainer)
			{
				startNewLine();
			}
			var word = this.words[i];
			word.textWidget.visible = true;
			word.shapeWidget.visible = true;
			if (word.text == NL)
			{
				// New line: start a new line (no element draw)
				word.textWidget.visible = false;
				word.shapeWidget.visible = false;
				if(previousIsNL){
					currentLine.height = 18;
					currentLine.width = 10;
					currentLine.maxAscent = 8;
					word.textWidget.visible = true;
					word.shapeWidget.visible = true;
				}
				commitCurrentLine(this);
				startNewLine();
				previousIsNL = true;
			}
			else if (currentX == 0 && word.text.trim() == "")
			{
				// First item in line is a space: make invisible
				word.textWidget.visible = false;
				word.shapeWidget.visible = false;
			}
			else
			{
				previousIsNL = false;
				// Normal word: put in flow
				if (currentX + word.textWidget.getBounds().width > maxWidth)
				{
					// Max width exceeded: wrap to new line
					commitCurrentLine(this);
					startNewLine();
				}
				// Put word in current line:
				addWordToCurrentLine(word);
			}
		}
		// Commit last line if not empty
		if (currentX > 0)
		{
			commitCurrentLine(this);
		}
		// Align all lines
		for (i=0; i<this.lines.length; i++)
		{
			var line = this.lines[i];
			// Vertical alignment (different font sizes, shift down for max ascent)
			alignVertContainer(line);
			// Horizontal alignment (text alignment)
			alignHorizContainer(line);
		}
		// Set sizes of the result text container
		this.easelContainer.width = totalWidth;
		this.easelContainer.height = totalHeight;
		if (this.shape)
		{
			// External text area
			this.shape.x = 0;
			this.shape.y = 0;
			this.shape.width = totalWidth;
			this.shape.height = totalHeight;
			// Debug: External text frame
//			this.shape.graphics
//				.beginStroke("#FF0000")
//				.setStrokeStyle(1,"round")
//				.drawRect(0,0,this.shape.width,this.shape.height);
		}
	};

	// Static Enums --------------------------------------------------------------------------------
	TextService.TEXT_ALIGN = {
		LEFT: 'left',
		CENTER: 'center',
		RIGHT: 'right'
	};

	// ---------------------------------------------------------------------------------------------

	return TextService;

});
