/**
 * Created by fabrizio on 02/04/14.
 */

define (['jquery','edge','mapUtils','eventsService','textService','easeljs'],	function($,Edge,MapUtils,EventsService,TextService) {

    var EdgeView = function(edge,map,stage)
    {
        var me = this;
        //
        this.stage = stage;
        this.dragging = false;
        this.dragDispP = {x:0,y:0};
        this.simpleClick = true;
        //
        this.edge = edge;
        //
        this.source = map.nodes[edge.source];
        this.target = map.nodes[edge.target];

        // Intersection points (calculated by mapController)
        this.sourceP = {x:0, y:0};
        this.targetP = {x:0, y:0};

        // View components
        this.srcP = {x:0, y:0}; // Source node point (local coordinates)
        this.tgtP = {x:0, y:0}; // Target node point (local coordinates)
        this.ctlP = {x:0, y:0}; // Control point (local coordinates)
        this.txtP = {x:0, y:0}; // Text point (local coordinates)

        // View components
        this.widget = new createjs.Container(); // Widget container
        this.area = new createjs.Shape(); // External node area
        this.shapeText = new createjs.Shape();
        this.text = new createjs.Container(); // Text container

        // Text service
        this.textService = new TextService(this.text);

        // Create DOM title text element
        this.titleTextElement = $('<div id="edge_content_' + edge.id + '" class="edge-content"></div>');
        this.titleTextElement.append(this.edge.titleText);
        $('#edge_contents #edge_content_'+edge.id).remove();
        $('#edge_contents').append(this.titleTextElement);
        if (this.edge.titleText)
        {
            this.textService.setHtmlText(this.titleTextElement[0]);
            this.textService.arrangeText(10000,TextService.TEXT_ALIGN.LEFT);
        }

        this.shapeText.cursor = "pointer";
        this.text.cursor = "pointer";
        // View assembly
        this.widget.addChild(this.area);
        this.widget.addChild(this.shapeText);
        this.widget.addChild(this.text);

        // Low-level events
        this.widget.on("mousedown", function(evt) {me.onWidgetMouseDown(evt.stageX, evt.stageY);});
        this.shapeText.on("mousedown", function(evt) {me.onTextMouseDown(evt.stageX, evt.stageY);});
        this.text.on("mousedown", function(evt) {me.onTextMouseDown(evt.stageX, evt.stageY);});
        this.widget.on("pressmove", function(evt) {me.onTextPressMove(evt.stageX, evt.stageY);});
        this.widget.on("pressup"  , function(evt) { me.onTextPressUp  (evt.stageX, evt.stageY);});
    };

    EdgeView.prototype.updateWidget = function(source, target)
    {

        var me = this;
        // Update edge look basing on model data
        this.source = source;
        this.target = target;
        if (this.edge && this.source && this.target)
        {
            // Update position and sizes (absolute coordinates)
            // View components (relative coordinates)
            var ed = this.edge.edgeData;
            var srcC = this.source.getCenter();
            var tgtC = this.target.getCenter();
            var edgeArea = MapUtils.prototype.normalizeArea(srcC.x, srcC.y, tgtC.x, tgtC.y);
            if (ed.curve)
            {
                // Curve line: include control points (points are coincident)
                edgeArea = MapUtils.prototype.extendArea(edgeArea,ed.ctrlPx,ed.ctrlPy);
            }
            this.widget.x = edgeArea.x1;
            this.widget.y = edgeArea.y1;
            this.widget.width = (edgeArea.x2-edgeArea.x1);
            this.widget.height = (edgeArea.y2-edgeArea.y1);
            //
            this.area.x = 0;
            this.area.y = 0;
            this.area.width = this.widget.width;
            this.area.height = this.widget.height;
            // Based on sourceP/targetP that must be pre-calculated by mapController
            // Convert points to local area coordinates
            this.srcP = this.stage.localToLocal(this.sourceP.x,this.sourceP.y,this.area);
            this.tgtP = this.stage.localToLocal(this.targetP.x,this.targetP.y,this.area);
            this.ctlP = this.stage.localToLocal(ed.ctrlPx,ed.ctrlPy,this.area);
            if (ed.curve)
            {
                // Curve edge: use control point for text
                this.txtP = MapUtils.prototype.getTextPoint(this.srcP,this.tgtP,this.ctlP);
            }
            else
            {
                // Linear edge: use average point for text
                this.txtP = {x: (this.srcP.x+this.tgtP.x)/2, y: (this.srcP.y+this.tgtP.y)/2};
            }
            //
            //if (this.edge.titleText)
            //{
            $('#edge_contents #edge_content_'+this.edge.id).remove();


            var classForTitleTextElement="";


            this.titleTextElement = $('<div id="edge_content_' + this.edge.id + '" class="edge-content"></div>');
            this.titleTextElement.addClass('edgeLabel');
            this.titleTextElement.append(this.edge.titleText);
            classForTitleTextElement = this.edge.edgeData.textProperties.bold ? 'labelBold' : '';
            this.titleTextElement.addClass(classForTitleTextElement);
            classForTitleTextElement = this.edge.edgeData.textProperties.italic ? 'labelItalic' : '';
            this.titleTextElement.addClass(classForTitleTextElement);
            $('#edge_contents').append(this.titleTextElement);
            //console.log(classForTitleTextElement);
            this.textService.setHtmlText(this.titleTextElement[0]);
            this.textService.arrangeText(10000,TextService.TEXT_ALIGN.LEFT);


            //}
            this.text.x = this.txtP.x - (this.text.width / 2);
            this.text.y = this.txtP.y - (this.text.height / 2);

            this.titleTextElement.on("edgeChanged",function(evt,fromJ,toJ)
            {
                if (me.onModelChanged) me.onModelChanged(evt,fromJ,toJ);
            });

            this.drawWidget();
        }
    };

    EdgeView.prototype.drawWidget = function()
    {

        this.area.graphics.clear();

        var ed = this.edge.edgeData;

//		// Debug: External edge area
//		this.area.graphics
//			.beginStroke("#000000")
//			.setStrokeStyle(1,"round")
//			.drawRect(0,0,this.area.width,this.area.height);

        this.text.x = this.txtP.x - (this.text.width / 2);
        this.text.y = this.txtP.y - (this.text.height / 2);

        this.shapeText.graphics.clear();
        if (this.edge.titleText !=""){
            this.shapeText.graphics.beginFill(ed.penColor).drawRoundRect(this.text.x-8,this.text.y-6,this.text.width+16,this.text.height+12, 3);
        }
        // Draw line
        if (ed.curve)
        {
            // Draw Bézier curve
            this.area.graphics
                .beginStroke(ed.penColor)
                .setStrokeStyle(parseInt(ed.penWidth, 10),"round")
                .moveTo(this.srcP.x,this.srcP.y)
                .bezierCurveTo(this.ctlP.x,this.ctlP.y,this.ctlP.x,this.ctlP.y,this.tgtP.x,this.tgtP.y);
        }
        else
        {
            // Draw linear connector
            this.area.graphics
                .beginStroke(ed.penColor)
                .setStrokeStyle(parseInt(ed.penWidth, 10),"round")
                .moveTo(this.srcP.x,this.srcP.y)
                .lineTo(this.tgtP.x,this.tgtP.y);
        }
        // Draw arrow tips
        var otherP;
        if (this.edge.edgeData.type==Edge.EDGETYPE.FORWARD ||
            this.edge.edgeData.type==Edge.EDGETYPE.BOTH) {
            otherP={x:0,y:0};
            if (this.edge.edgeData.curve) {
                otherP=this.ctlP;
            } else {
                otherP=this.srcP;
            }
            this.drawTip(this.tgtP, otherP);
        }
        if (this.edge.edgeData.type==Edge.EDGETYPE.BACKWARD ||
            this.edge.edgeData.type==Edge.EDGETYPE.BOTH) {
            if (this.edge.edgeData.curve) {
                otherP=this.ctlP;
            } else {
                otherP=this.tgtP;
            }
            this.drawTip(this.srcP,otherP);
        }
    };

    EdgeView.prototype.drawTip = function(p1,p2)
    {
        var ed = this.edge.edgeData;
        var tp0 = {x:0,y:0};
        var tp1 = {x:0,y:0};
        var tp2 = {x:0,y:0};
        var a=Math.atan2(p2.x-p1.x,p2.y-p1.y);
        var a1=a-(Math.PI/9);
        var a2=a+(Math.PI/9);
        var thickness=ed.penWidth*6;
        var color=ed.penColor;
        tp0.x=p1.x+Math.round(Math.sin(a1)*thickness);
        tp0.y=p1.y+Math.round(Math.cos(a1)*thickness);
        tp1.x=p1.x;
        tp1.y=p1.y;
        tp2.x=p1.x+Math.round(Math.sin(a2)*thickness);
        tp2.y=p1.y+Math.round(Math.cos(a2)*thickness);
        this.area.graphics
            .beginStroke(color)
            .setStrokeStyle(1,"round")
            .beginFill(color)
            .moveTo(tp0.x,tp0.y)
            .lineTo(tp1.x,tp1.y)
            .lineTo(tp2.x,tp2.y)
            .endFill();
    };

    EdgeView.prototype.setLinear = function()
    {
        this.edge.edgeData.curve = false;
    };

    EdgeView.prototype.onWidgetMouseDown = function(x, y)
    {
        // TODO: Select edge, temporarily setLinear
        this.simpleClick = true;
        // if the edge has no text show the editor
        if (!this.edge.titleText){
            document.app.hideOverlayContents();
            document.app.hideCommentOverlay();
            document.app.showLinkEditor(this.edge);
        }

    };

    EdgeView.prototype.onTextMouseDown = function(x, y)
    {
        // Calculate click displacement
        this.dragging = true;
        //var deltaP = this.shapeText.globalToLocal(x,y);
        //this.dragDispP = {x: this.txtP.x - deltaP.x, y: this.txtP.y - deltaP.y};
        this.dragDispP = {x:0, y:0}; // TODO: calcolare offset, compensare fattore trascinamenti lunghi

        document.app.hideOverlayContents();
        document.app.hideCommentOverlay();
        document.app.showLinkEditor(this.edge);

    };

    EdgeView.prototype.onTextPressMove = function(x, y)
    {
        // Execute dragging
        if (this.dragging)
        {
            // hide the editor
            document.app.hideLinkEditor(this.edge);

            // Convert edge to curve

            this.simpleClick = false;
            // Update widget position on stage
            var tP = this.stage.globalToLocal(x,y);
            // Fire event for notification
            if (this.onEdgeMidPointDrag) {
                var evt = EventsService.prototype.createEdgeMidPointDragEvent(
                    this.edge.id,
                        tP.x + this.dragDispP.x,
                        tP.y + this.dragDispP.y,
                    false,
                    false);
                this.onEdgeMidPointDrag(evt);
            }
        }
    };

    EdgeView.prototype.onTextPressUp = function(x, y)
    {
        // End drag
        this.dragging = false;
        document.app.fixCanvasSize();
        this.stage.update();
        var evt = EventsService.prototype.createEdgeModifiedEvent(
            this.edge.id,
            false,
            false);
        this.onEdgeUpdated(evt);

        document.app.hideOverlayContents();
        document.app.hideCommentOverlay();
        document.app.showLinkEditor(this.edge);

    };

    return EdgeView;

});
