/**
 * Created by fabrizio on 02/04/14.
 */

define (['jquery','node','eventsService','textService','easeljs'], function($,Node,EventsService,TextService) {

    /**
     *
     * @param node Node model
     * @param stage EaselJs container
     * @constructor
     */
    var NodeView = function(node, stage) {
        var me = this;
        //
        this.dragging = false;
        this.stage = stage;
        this.dragDispP = {x:0,y:0};
        //

        // Bools for edit
        this.editingMode = false;


        this.node = node;

        // View components
        this.widget = new createjs.Container(); // Widget container

        this.area = new createjs.Shape(); // External node area
        this.shape = new createjs.Shape(); // Node shape (or image frame)
        this.image = new createjs.Bitmap(new Image()); // Image (to be rescaled)
        this.text = new createjs.Container(); // Text container
        this.hit = new createjs.Shape(); // Widget hit area
        this.background = new createjs.Shape();
        //this.commentIcon = new createjs.Bitmap("assets/images/ico/commentread.png")

        // Text service
        this.text.x = (this.node.geometry.width - this.node.geometry.txtW)/2;
        this.textService = new TextService(this.text);
        // Rounded rectangle radius (if applicable)
        this.roundRadius = 5;

        this.shape.width = this.node.geometry.width;
        this.shape.height = this.node.geometry.height;

        // View assembly
        this.widget.addChild(this.background);
        this.widget.addChild(this.area);
        this.widget.addChild(this.shape);
        this.widget.addChild(this.image);
        this.widget.addChild(this.text);

        this.widget.hitArea = this.hit;

        // Low-level events
        this.image.image.loadedOk = true;
        this.image.image.onload = function() {me.imageLoaded();};
        this.image.image.onerror = function() {
                // sostituzione con placeholder
            me.image.image.loadedOk = false;
            me.image.image.src = NodeView.DEFAULT_MISSING_IMG
        };
        this.stage.enableMouseOver(10);
        this.widget.cursor = "pointer";
        this.widget.on("mouseover",function(evt){me.onMouseRollOver(evt.stageX, evt.stageY);});
        this.widget.on("mousedown", function(evt) {me.onMouseDown(evt.stageX, evt.stageY);});
        this.widget.on("pressmove", function(evt) {me.onPressMove(evt.stageX, evt.stageY);});
        this.widget.on("pressup"  , function(evt) {me.onPressUp  (evt.stageX, evt.stageY);});
        this.widget.on("click",function(evt){ me.onDBLClick(evt.stageX, evt.stageY);});
    };


    NodeView.DEFAULT_MISSING_IMG = "assets/images/ico/missing-icon.png"

    NodeView.prototype.updateWidget = function() {
        // Update widget based on model and text size
        var me = this;
        if (this.node)
        {
            //gli elementi che cambiano la dimensione del nodo
            // mi serve sapere la dimensione minima del testo
            // Load html text
            var titleTextHtml;
            //var textSource = this.getAssetPath("resources/" + this.node.id + "/title/element.html");

            titleTextHtml = this.node.titleText;

            // Create DOM title text element
            $('#node_contents').find("#node_content_"+this.node.id).remove();
            this.titleTextElement = $('<div id="node_content_' + this.node.id + '" class="node-content"></div>');
            //this.titleTextElement.css("width",this.node.geometry.width);
            //this.titleTextElement.css("height",this.node.geometry.height);
            this.titleTextElement.addClass("nodeLabel");
            if (this.node.nodeData.textProperties.bold) {
                this.titleTextElement.addClass("labelBold");
            }
            if (this.node.nodeData.textProperties.italic) {
                this.titleTextElement.addClass("labelItalic");
            }
            this.titleTextElement.addClass("labelSize" + this.node.nodeData.textProperties.size);
            var labelClassColor = Node.LABELCOLOR.CHILD;
            if (this.node.nodeData.mode == Node.MODE.ROOT) labelClassColor = Node.LABELCOLOR.ROOT;
            if (isWhite(this.node.nodeData.penColor)) labelClassColor = Node.LABELCOLOR.CHILD; /* se sfondo bianco label in nero */

            this.titleTextElement.addClass(labelClassColor);

            this.titleTextElement.append(titleTextHtml);
            $('#node_contents').append(this.titleTextElement);
            this.textService.setHtmlText(this.titleTextElement[0]);
            this.textService.arrangeText(this.node.geometry.txtW);

            this.titleTextElement.on("nodeChanged",function(evt,fromJ,toJ)
            {
                if (me.onModelChanged) me.onModelChanged(evt,fromJ,toJ);
            });


            if (this.text.height+10> this.node.geometry.height ) {
                this.node.geometry.height = this.text.height + 20;
            }
            if (this.text.width> this.node.geometry.width ) {
                this.node.geometry.width = this.text.width + 20;
            }


            // Update position and size (absolute)
            var nd = this.node.nodeData;
            var ni = this.node.image;

            if (ni != null && ni.name!=""){
                var imageSource = this.getAssetPath(ni.name);

                if (!this.image.image.src.endsWith(imageSource))
                {
                    this.image.image.src = imageSource;

                    //console.log('assegno la source', imageSource);
                }else{
                    //TOD DEFINIRE COSA FARE IN QUESTO CASO
                    //console.log('source già assegnata');
                }
            }else{
                this.widget.removeChild(this.image);
                this.image = null;
            }

            // inoltre la dimensione del nodo dovrebbe essere tale da ospitare
            // sia l'immagine + l'altezza del nodo ma si deve essere certi che l'immagine sia caricata
            if (this.image!=null && this.image.name != ""){
                var naturalWidth = this.node.geometry.width;
                var naturalHeight = this.node.geometry.height - this.text.height;
                if (this.image.image.naturalWidth!=0){
                    naturalWidth = this.image.image.naturalWidth;
                    naturalHeight = this.image.image.naturalHeight;
                }
                var scaleX = parseInt(naturalWidth / (this.node.geometry.width), 10);
                scaleX < 1 ? scaleX = 1 : scaleX = scaleX;
                var min_image_height =  naturalHeight / scaleX;
                var min_height_node = min_image_height;
                /*if (this.text.height!=0)  BTD-3010
                    min_height_node += (this.text.height+20)*/
                if (this.node.geometry.height<min_height_node){
                    this.node.geometry.height = min_height_node;
                }
            }

            var ng = this.node.geometry;

            // Calculate widget extent (in stage absolute coordinates)
            this.widget.x = ng.x-(ng.width/2);
            this.widget.y = ng.y-(ng.height/2);
            this.widget.width = ng.width;
            this.widget.height = ng.height;
            // Widget components: relative coordinates
            // Area: same as widget
            this.area.x = 0;
            this.area.y = 0;
            this.area.width = this.widget.width;
            this.area.height = this.widget.height;
            // Hit: same as widget
            this.hit.x = this.area.x;
            this.hit.y = this.area.y;
            this.hit.width = this.area.width;
            this.hit.height = this.area.height;
            //console.log('node image', ni);

            // Text visibility
            //this.text.visible = (nd.textLayout!=Node.TEXTLAYOUT.NONE);
            this.text.visible = true;
            this.text.x = (this.node.geometry.width-this.text.width)/2;
            //console.log('altezza', this.text.height);
            this.text.y = (this.node.geometry.height-this.text.height)/2+2;

            if (this.image!=null && this.image.image.src!="" ){
                //console.log('this.image', this.image);
                this.shape.x = this.area.x;
                this.shape.y = this.area.y;
                this.shape.width = this.area.width;
                this.shape.height = this.area.height;

                this.image.x = this.shape.x;
                this.image.y = this.shape.y;
                this.image.width = this.shape.width;
                this.image.height = this.shape.height;
                if (this.text.height != undefined && this.text.height!=0){
                    this.image.height = this.shape.height - this.text.height-20;
                }else{
                    this.image.height = this.shape.height;
                }
                this.imageLoaded();
                //this.text.y =  this.image.height + (this.node.geometry.height- this.image.height - this.text.height)/2;
                this.text.y = this.image.height;
            }else{

                this.shape.x = this.area.x;
                this.shape.y = this.area.y;
                this.shape.width = this.area.width;
                this.shape.height = this.area.height;
                //console.log('nuova posizione di y ', this.text.y);
            }

            //posizione della label
            this.titleTextElement.css("width",this.node.geometry.width);
            this.titleTextElement.css("height",this.node.geometry.height);

            //devo mettere i commenti
            if (this.node.deepText.length>0){
                // creo il commento e lo metto a true
                if ($("#comment_contents ."+this.node.id).size()<=0){
                    var nodeComment = $(".commentReadBtn-template").clone();
                    nodeComment.removeClass("commentReadBtn-template")
                    nodeComment.addClass("commentReadBtn");
                    nodeComment.addClass(this.node.id);
                    $("#comment_contents").append(nodeComment);
                }

            }

            this.drawWidget();
        }


        /* ------ qui inizia era la vecchia gestione anastatis ----- *
         this.text.x = (this.node.geometry.width-this.text.width)/2;
         this.text.y = (this.node.geometry.height-this.text.height)/2;

         // Shape: depends on node type, text layout or framed image
         // Image: depends on node type, text layout and frame
         // Text: depends on text layout

         if  (nd.nodeType==Node.NODETYPE.IMAGE && nd.frame)
         {
         // Framed image
         this.shape.x = this.area.x;
         this.shape.y = this.area.y;
         this.shape.width = this.area.width;
         this.shape.height = this.area.height;
         //
         this.image.x = 0;
         this.image.width = this.area.width;
         if (nd.textLayout==Node.TEXTLAYOUT.INSIDE || nd.textLayout==Node.TEXTLAYOUT.NONE)
         {
         this.image.y = 0;
         this.image.height = this.area.height;
         //
         this.text.y = (this.area.height-this.text.height)/2;
         }
         else if (nd.textLayout==Node.TEXTLAYOUT.UNDER)
         {
         this.image.y = 0;
         this.image.height = this.area.height-this.text.height;
         //
         this.text.y = this.area.height-this.text.height;
         }
         else if (nd.textLayout==Node.TEXTLAYOUT.OVER)
         {
         this.image.y = (this.area.height-this.text.height);
         this.image.height = this.area.height-this.text.height;
         //
         this.text.y = 0;
         }
         }
         else if (nd.textLayout==Node.TEXTLAYOUT.INSIDE || nd.textLayout==Node.TEXTLAYOUT.NONE)
         {
         this.shape.x = this.area.x;
         this.shape.y = this.area.y;
         this.shape.width = this.area.width;
         this.shape.height = this.area.height;


         if (this.image.image) {
         this.image.x = this.shape.x;
         this.image.y = this.shape.y;
         this.image.width = this.shape.width;
         if (this.text.height != undefined)
         this.image.height = this.shape.height - this.text.height;
         else
         this.image.height = this.shape.height;
         this.imageLoaded();

         }

         this.text.y = (this.area.height -this.text.height)/2;

         }
         else if (nd.textLayout==Node.TEXTLAYOUT.UNDER)
         {
         // Text under
         this.shape.x = this.area.x;
         this.shape.y = this.area.y;
         this.shape.width = this.area.width;
         this.shape.height = this.area.height-this.text.height;
         //

         if (this.image.image) {
         this.image.x = this.shape.x;
         this.image.y = this.shape.y;
         this.image.width = this.shape.width;
         this.image.height = this.shape.height;
         this.imageLoaded();
         }
         //
         this.text.y = this.shape.height;
         }
         else if (nd.textLayout==Node.TEXTLAYOUT.OVER)
         {
         // Text over
         this.shape.x = this.area.x;
         this.shape.y = text.height;
         this.shape.width = this.area.width;
         this.shape.height = this.area.height-this.text.height;
         //
         if (this.image.image) {
         this.image.x = this.shape.x;
         this.image.y = this.shape.y;
         this.image.width = this.shape.width;
         this.image.height = this.shape.height;
         this.imageLoaded();
         }
         //
         this.text.y = 0;
         }
         */
        /* ------ qui finisce era la vecchia gestione anastatis ----- */




    };

    NodeView.prototype.getTextNode = function() {
        return this.node.titleText;
    };

    // redefine this on controller side
    //NodeView.prototype.onModelChanged = function(evt, fromJ,toJ){
    //};

    NodeView.prototype.setLabelPropForRootNode = function(mode){
        var labelClassColor = mode === true ? Node.LABELCOLOR.ROOT : Node.LABELCOLOR.CHILD;
        if (isWhite(this.node.nodeData.penColor)) labelClassColor = Node.LABELCOLOR.CHILD;
        var titledElement = $('#node_content_'+this.node.id);
        titledElement.removeClass(Node.LABELCOLOR.CHILD).removeClass(Node.LABELCOLOR.ROOT).addClass(labelClassColor);
    };


    NodeView.prototype.cleanData = function() {
        this.area.graphics.clear();
        this.shape.graphics.clear();
        this.background.graphics.clear();
    };

    NodeView.prototype.drawWidget = function() {

        this.cleanData();
        var nd = this.node.nodeData;
//		// Debug: external rectangle (-1px)
//		this.area.graphics
//			.beginStroke("#000000")
//			.setStrokeStyle(2,"round")
//			.drawRect(-1, -1, this.area.width+2, this.area.height+2);
        // Hit area

        this.hit.graphics
            .beginFill("#000000")
            .drawRect(0, 0, this.hit.width, this.hit.height);

        switch (nd.nodeType)
        {
            case Node.NODETYPE.RECTANGLE:
                this.shape.graphics
                    .beginStroke(nd.penColor)
                    .setStrokeStyle(parseInt(nd.penWidth, 10),"round")
                    .beginFill(nd.brushColor)
                    .drawRect(0, 0, this.shape.width, this.shape.height);
                this.background.graphics.beginFill("#FFFFFF").drawRect(0, 0, this.shape.width, this.shape.height);
                break;
            case Node.NODETYPE.ROUNDRECT:
                this.shape.graphics
                    .beginStroke(nd.penColor)
                    .setStrokeStyle(parseInt(nd.penWidth, 10),"round")
                    .beginFill(nd.brushColor)
                    .drawRoundRect(0, 0, this.shape.width, this.shape.height,this.roundRadius);
                this.background.graphics.beginFill("#FFFFFF").drawRoundRect(0, 0, this.shape.width, this.shape.height,this.roundRadius);
                break;
            case Node.NODETYPE.ELLIPSE:
                this.shape.graphics
                    .beginStroke(nd.penColor)
                    .setStrokeStyle(parseInt(nd.penWidth, 10),"round")
                    .beginFill(nd.brushColor)
                    .drawEllipse(0, 0, this.shape.width, this.shape.height);
                this.background.graphics.beginFill("#FFFFFF").drawEllipse(0, 0, this.shape.width, this.shape.height);

                break;
            case Node.NODETYPE.IMAGE:
                if (nd.frame)
                {
                    this.shape.graphics
                        .beginStroke(nd.penColor)
                        .setStrokeStyle(parseInt(nd.penWidth, 10),"round")
                        .beginFill(nd.brushColor)
                        .drawRect(0, 0, this.shape.width, this.shape.height);
                }
                break;
            case Node.NODETYPE.TEXT:
                // Debug text-only nodes
//				this.shape.graphics
//					.beginStroke("#ff0000")
//					.setStrokeStyle(1,"round")
//					.drawRect(0, 0, this.shape.width, this.shape.height);
                break;
        }
    };

    NodeView.prototype.onMouseRollOver = function(x,y) {

    };

    NodeView.prototype.onMouseOut = function() {

    };

    NodeView.prototype.onMouseDown = function(x, y) {
        // Calculate click displacement from top-left corner
        this.dragging = true;
        this.dragDispP = this.widget.globalToLocal(x,y);
    };

    NodeView.prototype.onPressMove = function(x, y) {
        //console.log('onPressMove');

        if (this.dragging) {
            // Update widget position on stage
            var P = this.stage.globalToLocal(x, y);

            this.widget.x = P.x - this.dragDispP.x;
            this.widget.y = P.y - this.dragDispP.y;
            // Fire event for notification
            if (this.onNodeDrag) {
                var evt = EventsService.prototype.createNodeDragEvent(
                    this.node.id,
                        this.widget.x + (this.widget.width / 2),
                        this.widget.y + (this.widget.height / 2),
                    false,
                    false);
                this.onNodeDrag(evt);
            }

        }
    };

    NodeView.prototype.onPressUp = function (x, y) {
        //console.log('onPressUp');
        this.dragging = false;
        if(window.devicePixelRatio===1)
            document.app.fixCanvasSize();
        this.stage.update();
        // Fire event for notification
        if (this.onNodeEndDrag) {
            var evt = EventsService.prototype.createNodeEndDragEvent(
                this.node.id,
                    this.widget.x+(this.widget.width/2),
                    this.widget.y+(this.widget.height/2),
                false,
                false);
            this.onNodeEndDrag(evt);

        }
    };

    NodeView.prototype.onDBLClick = function(x,y) {
        //console.log('onDBLClick');
        if (this.onNodeDoubleClicked) {
            var evt = EventsService.prototype.createDBLClickEvent(
                this.node.id,
                    this.widget.x + (this.widget.width / 2),
                    this.widget.y + (this.widget.height / 2),
                false,
                false);
            this.onNodeDoubleClicked(evt);
        }
    };

    NodeView.prototype.imageLoaded = function() {
        // Rescale image
        var scaleX = this.image.width / this.image.image.naturalWidth;
        var scaleY = this.image.height / this.image.image.naturalHeight;
        var scale = Math.min(scaleX,scaleY);
        var nw = this.image.image.naturalWidth * scale;
        var nh = this.image.image.naturalHeight * scale;
        // aggiungere qui i controlli per non sgranare l'immagine

        this.image.x=(this.image.width - nw) / 2;
        this.image.y=(this.image.height - nh) / 2;
        this.image.scaleX = scale;
        this.image.scaleY = scale;
        //
        //console.log('update image');
        this.stage.update();
    };

    NodeView.prototype.getID = function() {
        var nodeId = "";
        if (this.node) {
            nodeId = this.node.id;
        }
        return nodeId;
    };

    NodeView.prototype.getCenter = function() {
        if (this.node && this.node.nodeData.nodeType==Node.NODETYPE.TEXT) {
            return {x: this.widget.x+(this.widget.width/2),
                y: this.widget.y+(this.widget.height/2)};
        } else {
            return {x: this.widget.x+this.shape.x+(this.shape.width/2),
                y: this.widget.y+this.shape.y+(this.shape.height/2)};
        }
    };

    NodeView.prototype.getWidgetCenter = function() {
        return {x: this.widget.x+(this.widget.width/2),
            y: this.widget.y+(this.widget.height/2)};
    };

    NodeView.prototype.getSize = function() {
        return {width: this.getWidth(), height: this.getHeight()};
    };

    NodeView.prototype.getWidth = function() {
        return this.widget.width;
    };

    NodeView.prototype.getHeight = function() {
        return this.widget.height;
    };

    NodeView.prototype.off = function() {
        this.widget.mouseEnabled = false;
    };

    NodeView.prototype.on = function() {
        this.widget.mouseEnabled = true;
    };

    /**
     * @param fileName Asset file name
     * @returns {string} Asset absolue path
     */
    NodeView.prototype.getAssetPath = function(fileName) {
        return this.node._assetBasePath + "/" + fileName;
    };

    return NodeView;

});
