/**
 * Created by mtassetti on 03/06/14.
 */

define(['geometryService','jquery','map','node','edge','nodeView','edgeView'], function (GeometryService, $,Map,Node,Edge,NodeView,EdgeView) {

    describe('constructor', function() {
        it('default constructor', function() {
            var geometryService = new GeometryService();
            expect(geometryService).not.toBeUndefined()
        });
    });

    describe('calculateEdgeIntersection', function() {

        beforeEach(function() {
           this.geometryService = new GeometryService();
        });

        it('squares aligned over diagonal', function() {
            var n1Json={
                geometry:{x: 50,y: 50,width: 100,height: 100}
            };
            var n2Json={
                geometry:{x: 250,y: 250,width: 100,height: 100}
            }

            var views=createViews(n1Json,n2Json);
            var edgeView=views[0]
            var node1View=views[1]
            var node2View=views[2]

            var point1=this.geometryService.calculateEdgeIntersection(edgeView,node1View,node2View);
            var point2=this.geometryService.calculateEdgeIntersection(edgeView,node2View,node1View);
            expect(point1.x).toEqual(100);
            expect(point1.y).toEqual(100);
            expect(point2.x).toEqual(200);
            expect(point2.y).toEqual(200);
        });

        it('streched rectangles aligned over diagonal', function() {
            var n1Json={
                geometry:{x: 50,y: 50,width: 300,height: 100}
            };
            var n2Json={
                geometry:{x: 250,y: 250,width: 300,height: 100}
            }

            var views=createViews(n1Json,n2Json);
            var edgeView=views[0]
            var node1View=views[1]
            var node2View=views[2]

            var point1=this.geometryService.calculateEdgeIntersection(edgeView,node1View,node2View);
            var point2=this.geometryService.calculateEdgeIntersection(edgeView,node2View,node1View);
            expect(point1.x).toEqual(100);
            expect(point1.y).toEqual(100);
            expect(point2.x).toEqual(200);
            expect(point2.y).toEqual(200);
        });

        it('streched rectangles not aligned over diagonal', function() {
            var n1Json={
                geometry:{x: 350,y: 125,width: 100,height: 50}
            };
            var n2Json={
                geometry:{x: 200,y: 200,width: 100,height: 50}
            }

            var views=createViews(n1Json,n2Json);
            var edgeView=views[0]
            var node1View=views[1]
            var node2View=views[2]

            var point1=this.geometryService.calculateEdgeIntersection(edgeView,node1View,node2View);
            var point2=this.geometryService.calculateEdgeIntersection(edgeView,node2View,node1View);
            expect(point1.x).toEqual(300);
            expect(point1.y).toEqual(150);
            expect(point2.x).toEqual(250);
            expect(point2.y).toEqual(175);
        });

        it('rouded aligned over diagonal', function() {
            var n1Json={
                geometry:{x: 50,y: 50,width: 100,height: 100},
                nodeData: {
                    nodeType:Node.NODETYPE.ELLIPSE
                }
            };
            var n2Json={
                geometry:{x: 250,y: 250,width: 100,height: 100},
                nodeData: {
                    nodeType:Node.NODETYPE.ELLIPSE
                }
            }

            var views=createViews(n1Json,n2Json);
            var edgeView=views[0]
            var node1View=views[1]
            var node2View=views[2]

            var point1=this.geometryService.calculateEdgeIntersection(edgeView,node1View,node2View);
            var point2=this.geometryService.calculateEdgeIntersection(edgeView,node2View,node1View);
            expect(point1.x).toEqual(85);
            expect(point1.y).toEqual(85);
            expect(point2.x).toEqual(215);
            expect(point2.y).toEqual(215);
        });
    });


    var createViews=function(n1Json,n2Json)
    {
        n1Json.id='n1';
        n1Json.id='n2';
        var edgeJson={
            id:'e1',source:'n1',target:'n2'
        };
        var node1=new Node(n1Json);
        var node2=new Node(n2Json);
        var edge=new Edge(edgeJson);
        var map=new Map()
        map.nodes[node1.id] = node1;
        map.nodes[node2.id] = node2;
        map.edges[edge.id] = edge;

        var node1View=new NodeView(node1);
        var node2View=new NodeView(node2);
        var edgeView=new EdgeView(edge,map);

        node1View.updateWidget();
        node2View.updateWidget();
        return [edgeView,node1View,node2View];
    }

});

