


define(['jquery'],function() {

    var App = function ($_mainContainer) {

        this.mainContentWrapper = $('#main-content-wrapper', $_mainContainer);
        this.mainContent = $('#main-content', $_mainContainer);
        this.inputSearch = $('#search-input', $_mainContainer);
        this.buttonSearch =  $('#search-button', $_mainContainer);
        this.messageError = $('#message-error-template', $_mainContainer);

        this.mediaTemplate = $('#media-template .media-container', $_mainContainer);

        this.eventOnClose();
        this.eventOnSearch();

        this.adaptContent();

        window.addEventListener('resize', function(){
            document.app.adaptContent();
        });

    };

    App.prototype.adaptContent = function()
    {
        this.mainContentWrapper.css('height', $('body').height() - $('#fixed-content').outerHeight());

    };

    App.prototype.refresh = function()
    {
        /* delete the current content of navigation media */
        this.mainContent.empty();
        this.inputSearch.val("");

    };

    App.prototype.eventOnClose = function()
    {
        $('#panel-close').click(function(event)
        {
            document.nativeService.onjs_close();
        });
    };

    App.prototype.eventOnSearch = function()
    {
        this.buttonSearch.bind('click', function(event)
        {

            document.app.searchMedia(false);
        });

        this.inputSearch.bind('keypress', function(event)
        {
            if (event.which == 13)
            {
                event.preventDefault();
                document.app.searchMedia(false);
            }
        });
    };

    App.prototype.searchMedia = function(refresh)
    {
        //this.mainContent.empty();
        var keywords = this.inputSearch.val().trim();
        if (keywords.length>0)
        {
            document.nativeService.onjs_keywordSearched(keywords, refresh);
        }
        else
        {
            // Voglio poter "azzerare" la ricerca
            document.app.refresh();
        }
    };

    App.prototype.displayMedia = function(media, refresh)
    {
        if (refresh == undefined) refresh = false;
        if (!refresh)
            this.mainContent.empty();

        if (media == undefined || media.length <= 0)
        {
            this.mainContent.empty();
            var mediaError = $(this.messageError).clone().attr('id', 'message-error').addClass('visibile').html("Non ci sono risultati.");
            this.mainContent.append(mediaError);
        }

        for (var medium in media){
            var mediaContainer;
            if ($('#media-'+medium, this.mainContent).length>0)
            {
                mediaContainer = $('#media-'+medium, this.mainContent);
            }
            else
            {
                mediaContainer = $(this.mediaTemplate).clone().addClass('show');
                mediaContainer.attr('id', "media-"+medium);
                mediaContainer.attr("data-media-id", media[medium].id);
                this.mainContent.append(mediaContainer);
                mediaContainer.unbind('click').bind('click', function(event){
                    event.preventDefault();
                    document.nativeService.onjs_mediaSelected($(this).attr("data-media-id"));
                });
            }
            /*populating the data */
            $('.title-obj', $(mediaContainer)).html(media[medium].title);

            mediaContainer.addClass(media[medium].type);
            if (media[medium].fileExists)
            {
                //set a background image
                mediaContainer.removeClass("noPreview").addClass("preview");
                if (media[medium].type == "img")
                    $('.media-obj', mediaContainer).css('background-image', "url("+media[medium].filePath+")");
            }else{
                mediaContainer.removeClass("preview").addClass("noPreview");
                $('.media-obj', mediaContainer).css('background-image', "");
            }

        }
    };

    App.prototype.refreshLastSearch = function()
    {
        document.app.searchMedia(true);
    };

    return App;
});