var SpeechRecognition = window.SpeechRecognition || 
                        window.webkitSpeechRecognition || 
                        window.mozSpeechRecognition || 
                        window.oSpeechRecognition || 
                        window.msSpeechRecognition;

var VocalDriving = function(options) {
	var vd = this;
	vd.commands = {};
	vd.keyword = '';
	vd.sr = null;
	vd.lang = 'en-EN';
	vd.callbacks = {};
	vd.userOptions = {
		lang: true,
		keyword: true,
		commands: true,
	};
	vd.userCallbacks = {
		onAudioStart: true,
		onAudioEnd: true,
		onSpeechStart: true,
		onSpeechEnd: true,
		onResult: true,
		onError: true,
		onEnd: true
	};

	vd.helpers = {
		finalstr: '',
		speechend: false,
		timeout: null,
	};

	vd.reset = function() {
		if (!SpeechRecognition) return false;
		this.helpers.finalstr = '';
		if (!this.sr) {
			this.sr = new SpeechRecognition();
	        this.sr.continuous = true;
			this.sr.interimResults = true;
			this.sr.maxAlternatives = 1;
			this.sr.lang = vd.lang;
			vd.attachEvents();
		}
		return true;
	};

	vd.attachEvents = function() {
		var vd = this;
		this.sr.onaudiostart = function(ev) { vd.onAudioStart(ev) };
		this.sr.onaudioend = function(ev) { vd.onAudioEnd(ev) };
		this.sr.onsoundstart = function(ev) { vd.onSoundStart(ev) };
		this.sr.onsoundend = function(ev) { vd.onSoundEnd(ev) };
		this.sr.onspeechstart = function(ev) { vd.onSpeechStart(ev) };
		this.sr.onspeechend = function(ev) { vd.onSpeechEnd(ev) };
		this.sr.onresult = function(ev) { vd.onResult(ev) };
		this.sr.onerror = function(ev) { vd.onError(ev) };
		this.sr.onend = function(ev) { vd.onEnd(ev) };
	};

	vd.onAudioStart = function() {
		if (this.callbacks['onAudioStart']) this.callbacks['onAudioStart'](event);
	};

	vd.onAudioEnd = function() {
		if (this.callbacks['onAudioEnd']) this.callbacks['onAudioEnd'](event);
	};

	vd.onSoundStart = function() {
		if (this.callbacks['onSoundStart']) this.callbacks['onSoundStart'](event);
	};

	vd.onSoundEnd = function() {
		if (this.callbacks['onSoundEnd']) this.callbacks['onSoundEnd'](event);
	};

	vd.onSpeechStart = function() {
		if (this.callbacks['onSpeechStart']) this.callbacks['onSpeechStart'](event);
	};

	vd.onSpeechEnd = function() {
		if (this.callbacks['onSpeechEnd']) this.callbacks['onSpeechEnd'](event);
	};

	vd.onError = function() {
		if (this.callbacks['onError']) this.callbacks['onError'](event);
	};

	vd.onEnd = function() {
		if (this.callbacks['onEnd']) this.callbacks['onEnd'](event);
		var canStart = this.reset();
		if (canStart) this.start();
	};

	vd.onResult = function(event) {
		var vd = this;
		if (this.callbacks['onResult']) this.callbacks['onResult'](event);
		var finalStr = false;
	    for (var i = event.resultIndex; i < event.results.length; ++i) {
	    	if (event.results[i].isFinal) {
	        	vd.helpers.finalstr += event.results[i][0].transcript;
	        	finalStr= true;
	      	}
	    }
		if (finalStr) {
			console.log(vd.helpers.finalstr);
			vd.validateCommand(vd.helpers.finalstr);
		}
	};

	vd.validateCommand = function(cmd) {
		cmd = cmd.toLowerCase();
		var valid = (this.keyword!="") ? (cmd.indexOf(this.keyword)!=-1) : (cmd!="");
    	if (valid) {
    		if (this.keyword!="") cmd = cmd.substring(cmd.indexOf(this.keyword), cmd.length);
    		this.parseCommand(cmd);
		} else {
			var canStart = this.reset();
			if (canStart) this.start();
		}
	};

	vd.parseCommand = function(cmd) {
		cmd = cmd.toLowerCase();
		var ar = cmd.split(' ');
		if (this.keyword!="") ar.shift();
		var newstr = ar.join(' ');
    	var exec = this.checkCommand(newstr);
    	if (exec) {
    		exec.command.callback.apply(null, exec.attributes);
    	}
    	var canStart = this.reset();
		if (canStart) this.start();
	};

	vd.checkCommand = function(str, iterator) {
		var commands = this.commands;
		var foundCommand = null;
		var currentCommand;
		var k = iterator || 0;
		while (k<commands.length && foundCommand==null) {
			currentCommand = commands[k];
			var cmd = commands[k].command;
			var j = 0;
			while (j<cmd.length && foundCommand==null) {
				if (str.indexOf(cmd[j])!=-1) {
					foundCommand = cmd[j];
				} else j++;
			}
			k++;
		}
		if (foundCommand) {
			var io = str.indexOf(foundCommand);
			var nstr = str.substr(io, str.length);
			nstr = nstr.replace(foundCommand+' ', '');
			var ar = nstr.split(' ');
			var ctr = vd.checkAttributes(str, foundCommand, currentCommand);
			if (ctr!=false) return ctr;
			else this.checkCommand(str, k);
		} else {
			return false;
		}
	};

	vd.checkAttributes = function(str, activator, command) {
		var foundAttribute = true;
		var attributes = [];
		var k = 0;
		var validators = command['attribute'];
		while (k<validators.length && str!="") {
			var validator = validators[k];
			var foundAttribute = validator(str);
			if (foundAttribute) {
				var io = str.indexOf(foundAttribute);
				str = str.substr(io, str.length);
				str = str.replace(foundAttribute+' ', '');
				attributes.push(foundAttribute);
			}
			k++;
		}
		var validAttributes = (attributes.length == validators.length);
		if (validAttributes) {
			return {
				command: command,
				activator: activator,
				attributes: attributes
			};
		} else {
			return false;
		}
	};

	vd.start = function() {
		this.helpers.speechend = false;
		this.sr.start();
	};

	vd.init = function(options) {
		var vd = this;
		for (var k in options) {
			if (vd.userOptions[k]) this[k] = options[k];
			if (vd.userCallbacks[k]) this.callbacks[k] = options[k];
		}
		var canStart = this.reset();
		if (canStart) this.start();
	};

	this.init(options);
}