(function() {
	'use strict';

	PeriodicTable.element = Backbone.Model.extend({

		defaults: {
			name: null,
			number: null,
			symbol: null,
			color: null,
			image_url: null,
			image_description: null,
			more_text: null,
			group: null,
			atomic_mass: null,
			oxidation_numbers: null,
			melting_temperature: null,
			boiling_temperature: null,
			electronic_configuration: null,
			standard_physical_state: null,
			atomic_radius: null,
			first_ionisation_energy: null,
			electronegativity: null,
			electronic_affinity: null,
			density: null,
			discovery_year: null,
			active: false,
			clickable: true,
			deleted: 0,
			elementView: null,
			currentStatus: null,
		},

		initialize: function() {
			var model = this;
			Backbone.on('periodictable:shown', this.onReady, this);
		},

		onReady: function() {
			var model = this;
			model.set('complete_configuration', model.generateCompleteConfiguration());
			model.set('configuration_array', model.generateConfigurationToArray());
			model.set('configuration_array_number', model.generateConfigurationToArrayNumber());
			model.set('levels_matrix', model.generateMatrixLevels());
			model.set('number_of_levels', model.getLevelsNumber());
			model.set('levels_division', model.generateLevelsDivision());
			model.set('spanned_configuration', model.spanifyConfiguration());
		},

		hasMoreText: function() {
			return this.get('more_text').length>1;
		},

		generateCompleteConfiguration: function() {
			var str = this.get('electronic_configuration');
			str = str
				.replace('[Rn]','[Xe]4f<sup>14</sup>5d<sup>10</sup>6s<sup>2</sup>6p<sup>6</sup>')
				.replace('[Xe]','[Kr]4d<sup>10</sup>5s<sup>2</sup>5p<sup>6</sup>')
				.replace('[Kr]','[Ar]3d<sup>10</sup>4s<sup>2</sup>4p<sup>6</sup>')
				.replace('[Ar]','[Ne]3s<sup>2</sup>3p<sup>6</sup>')
				.replace('[Ne]','[He]2s<sup>2</sup>2p<sup>6</sup>')
				.replace('[He]','1s<sup>2</sup>')
				.replace(/<\/sup><sup>/g,'');
			return str;
		},

		generateConfigurationToArray: function() {
			var str = this.get('complete_configuration') || this.generateCompleteConfiguration();
			str = str
					.replace(new RegExp('</sup>', 'g'), '/')
					.replace(new RegExp('<sup>', 'g'), '')
					.replace(new RegExp('s', 'g'), '/0/')
					.replace(new RegExp('p', 'g'), '/1/')
					.replace(new RegExp('d', 'g'), '/2/')
					.replace(new RegExp('f', 'g'), '/3/')
					.replace(new RegExp('g', 'g'), '/4/')
					.replace(new RegExp('h', 'g'), '/5/')
					.replace(new RegExp('i|l', 'g'), '/');
			var arr = str.split('/');
			arr.splice(arr.length-1);
			return arr;
		},

		generateConfigurationToArrayNumber: function() {
			var str = this.get('complete_configuration') || this.generateCompleteConfiguration();
			str = str
				.replace(new RegExp('</sup>', 'g'), '/')
				.replace(new RegExp('<sup>', 'g'), '')
				.replace(new RegExp('s|p|d|f|g|h|i|l', 'g'), '/');
			var arr = str.split('/');
			arr.splice(arr.length-1);
			return arr;
		},

		generateMatrixLevels: function() {
			var arr = this.get('configuration_array') || this.generateConfigurationToArray();
			var matrix = [];
			for (var k=0; k<arr.length; k=k+3){
				var parsed = parseInt(arr[k])-1;
				matrix[parsed] = matrix[parsed] || [];
				matrix[parsed][parseInt(arr[k+1])] = parseInt(arr[k+2])
			}
			return matrix;
		},

		getLevelsNumber: function(){
			var arr = this.get('configuration_array_number') || this.generateConfigurationToArrayNumber();
			var max=1;
			for (var i=0; i<arr.length; i=i+2){
				var parsed = parseInt(arr[i]);
				if (parsed>max) max = parsed;
			}
			return max;
		},

		generateLevelsDivision: function(){
			var matrix = [];
			var arr = this.get('configuration_array_number') || this.generateConfigurationToArrayNumber();
			var arl = arr.length;
			var nl = this.get('number_of_levels') || this.getLevelsNumber();
			for (var k=0; k<nl; k++){
				var narr = [];
				var z = 0;
				for (var i=0; i<arl; i=i+2){
					if (parseInt(arr[i])==(k+1)) {
						if ((i+1)==(arl-1)) {
							narr[z]=parseInt(arr[i+1])*1000;
						}
						else narr[z]=parseInt(arr[i+1]);
						z++;
					}
				}
				matrix[k] = narr;
			}
			return matrix;
		},

		spanifyConfiguration: function() {
			var str = this.get('complete_configuration') || this.generateCompleteConfiguration();
			str = str
				.replace(new RegExp('<sup>', 'g'), '<aaa>')
				.replace(new RegExp('</sup>', 'g'), '</aaa>')
				.replace(new RegExp('s', 'g'), '/S/')
				.replace(new RegExp('p', 'g'), '/P/')
				.replace(new RegExp('d', 'g'), '/D/')
				.replace(new RegExp('f', 'g'), '/F/')
				.replace(new RegExp('g', 'g'), '/G/')
				.replace(new RegExp('/S/', 'g'), '<span class="s">s</span>')
				.replace(new RegExp('/P/', 'g'), '<span class="p">p</span>')
				.replace(new RegExp('/D/', 'g'), '<span class="d">d</span>')
				.replace(new RegExp('/F/', 'g'), '<span class="f">f</span>')
				.replace(new RegExp('/G/', 'g'), '<span class="g">g</span>')
				.replace(new RegExp('<aaa>', 'g'), '<sup>')
				.replace(new RegExp('</aaa>', 'g'), '</sup> ');
			return str;
		},
	});

}());
