(function() {
	'use strict';

	PeriodicTable.ClassesView = Backbone.View.extend({
		/*** attributes ***/
		isActive: false,

		events: {
			"click #classi span": "switchClasses"
		},

		realClasses: [],

		elements: [],
		classes: null,
		spans: null,
		ing: null,

		initialize: function() {
			this.elements = this.$('.elemento').get();
			var box = document.getElementById('classibox');
			this.classes = document.getElementById('classi');
			this.spans = this.classes.querySelectorAll('span');
			this.ing = this.el.querySelectorAll('.ingclasse');
			box.style['left'] = (this.$('#labelbox [rel="classes"]').offset().left - $(this.classes).offset().left) + 'px';
		},

		setActive: function(bool) {
			if (bool!=this.isActive) {
				this.isActive = bool;
				if (bool) this.active();
				else this.deactive();
				return this;
			}
		},

		active: function() {
			this.listenTo(periodicTableView.zElements,'change:active',this.onElementTap);
			var spans = this.spans;
			for (var k = 0; k < spans.length; k++) {
				spans[k].classList.remove('classeact');
			}
			var activeEl = periodicTableView.getActiveElement();
			if (activeEl) {
				var group = activeEl.get('group');
				if (group!='unkown') this.onElementTap(activeEl);
				else activeEl.set('active', false);
			}
			periodicTableView.focusView.close();
			this.semplifyClasses();
			var unclickable = periodicTableView.zElements.where({group : 'unkown'});
			for (var i = 0; i<unclickable.length; i++ ) unclickable[i].set('clickable', false);
		},

		reset: function() {
			Backbone.trigger("class:close");
			var ing = this.ing;
			for (var k = 0; k < ing.length; k++) {
				ing[k].style['display'] = 'none';
			}
			var elements = this.elements;
			for (var k = 0; k < elements.length; k++) {
				var el = elements[k];
				el.removeAttribute('style');
				if (el.classList.contains('showGroup')) {
					el.classList.remove('showGroup');
				}
			}
			var spans = this.spans;
			for (var k = 0; k < spans.length; k++) {
				spans[k].classList.remove('classeact');
			}
			periodicTableView.zElements.each(function(model) {
				model.set('active', false)
			});
		},

		deactive: function() {
			this.stopListening();
			this.restoreSemplifiedClasses();
			var ing = this.ing;
			for (var k = 0; k < ing.length; k++) {
				ing[k].style['display'] = 'none';
			}
			var elements = this.elements;
			for (var k = 0; k < elements.length; k++) {
				var el = elements[k];
				el.removeAttribute('style');
				if (el.classList.contains('showGroup')) {
					el.classList.remove('showGroup');
				}
			}
			var spans = this.spans;
			for (var k = 0; k < spans.length; k++) {
				spans[k].classList.remove('classeact');
			}
			var unclickable = periodicTableView.zElements.where({group : 'unkown'});
			for (var i = 0; i<unclickable.length; i++ ) unclickable[i].set('clickable', true);
		},

		onElementTap: function(model) {
			if (this.isActive && model.get('active')) {
				document.body.classList.remove('showCredits');
				var cl = model.get("group");
				this.openClass(cl);
				periodicTableView.audio.play("elemento");
			}
		},

		openClass: function(cl) {			
			var ing = this.ing;
			for (var k = 0; k < ing.length; k++) {
				ing[k].style['display'] = 'none';
			}
			var spans = this.spans;
			for (var k = 0; k < spans.length; k++) {
				var sp = spans[k];
				var rel = sp.getAttribute('rel');
				if (rel == cl) {
					sp.classList.add('classeact');
				} else {
					sp.classList.remove('classeact');
				}
			}
			var elements = this.elements;
			for (var k = 0; k < elements.length; k++) {
				var el = elements[k];
				el.removeAttribute('style');
				if (el.classList.contains('showGroup')) {
					el.classList.remove('showGroup');
				}
			}
			Backbone.trigger("class:open", cl);
			var els = periodicTableView.zElements.where({ group: cl });
			for (var i=0; i<els.length; i++){
				var model = els[i];
				var ev = model.get('elementView');
				ev.el.style['opacity'] = 1;
			};
			
			document.getElementById(cl+'_ing').style['display'] = 'block';
		},

		switchClasses: function(ev) {
			periodicTableView.audio.play("scheda");
			var cl = ev.currentTarget.getAttribute('rel');
			var activeElement = periodicTableView.getActiveElement();
			if (activeElement) activeElement.set('active', false)
			this.openClass(cl);
			return false;
		},

		semplifyClasses: function() {
			var view = this;
			if (TPCONF['semplifyClasses']) {
				for (var i = 0; i<periodicTableView.zElements.models.length; i++) {
					var model = periodicTableView.zElements.models[i];
					var group = model.get('group');
					view.realClasses.push(group);
					if (group == 'alogeni') model.set('group', 'nonmetal');
					if (group == 'metalc' || group == 'metald' || group == 'postmetal') model.set('group', 'metal');
				}
			}
		},

		restoreSemplifiedClasses: function() {
			var view = this;
			if (TPCONF['semplifyClasses']) {
				for (var i = 0; i<periodicTableView.zElements.models.length; i++) {
					var model = periodicTableView.zElements.models[i];
					model.set('group', view.realClasses[i]);
				}
				view.realClasses = [];
			}
		},
	});

}());