(function() {
	'use strict';

	PeriodicTable.ConfigurationView = Backbone.View.extend({
		/*** attributes ***/
		isActive: false,
		levelsCompleted: [],
		electrons: {},
		protons: [],
		levels: {},
		electronsObject: {},
		lastPositions: {},
		lastElectrons: [],
		lastMatrices: [],
		levelRadius: 26.88,
		canvasWidth: 420,
		centerOffsetX: 210,
		sublevelsSymbols: ['s','p','d','f'],
		electronsNumberForLevel: [2,2,6,2,6,2,10,6,2,10,6,2,14,10,6,2,14,7],
		elementsToDisable: [110,111,112,113,114,115,116,117,118],
		levelsComplete: [[2],[2,6],[2,6,10],[2,6,10,14],[2,6,10,14],[2,6,10,14],[2,6,10,14]],
		sublevelsLength: [2,6,10,14],
		quanticMatrix:[ [0,0], [-1,0,1,-1,0,1], [-2,-1,0,1,2,-2,-1,0,1,2], [-3,-2,-1,0,1,2,3,-3,-2,-1,0,1,2,3]],
		colours: ['#f2055c','#0f8aa6','#bf8821','#8c0335'],
		lastComplete: null,
		nucleus: null,
		notationSpan: null,
		focusPanel: null,
		atom: null,
		tables: null,
		spins: null,
		levelLabels: {},
		canvas: null,

		initialize: function() {
			Backbone.on('focus:close', this.deSelect, this);
			this.nucleus = document.getElementById('nucleo');
			this.notationSpan = this.el.querySelector('#notaz span');
			this.focusPanel = document.getElementById('approfondimento');
			this.canvas = document.getElementById('disegno');
			this.atom = document.getElementById('atomo');
			this.tables = this.el.querySelectorAll('#spin_ing table');
			this.spins = this.el.querySelectorAll('.spin1');

			this.levelLabels['l'] = document.getElementById('l');
			this.levelLabels['n'] = document.getElementById('n');
			this.levelLabels['m'] = document.getElementById('m');
		},

		setActive: function(bool) {
			if (bool!=this.isActive) {
				this.isActive = bool;
				if (bool) this.active();
				else this.deactive();
				return this;
			}
		},

		active: function() {
			this.listenTo(periodicTableView.zElements,'change:active',this.onElementTap);
			periodicTableView.modalToShow('ce_ing');
			for (var i=0; i<this.elementsToDisable.length; i++) {
				var num = this.elementsToDisable[i];
				var model = periodicTableView.zElements.where({ number: num })[0];
				model.set('clickable',false);
				model.get('elementView').setClass('nonconsiderare schiarita','');
			};
			this.resetConfiguration();
			var activeEl = periodicTableView.getActiveElement();
			if (activeEl && activeEl.get('clickable')) {
				this.onElementTap(activeEl);
				this.focusPanel.classList.add('app_ce');
			} else {
				if (activeEl) {
					activeEl.set('active', false);
					periodicTableView.focusView.$el.hide();
					periodicTableView.focusView.close();
				}
				this.focusPanel.classList.add('app_ce');
			}
		},

		reset: function() {
			periodicTableView.focusView.close();
			periodicTableView.zElements.each(function(model) {
				model.set('active', false)
			});
		},

		deactive: function() {
			periodicTableView.switchColours('default');
			this.stopListening();
			periodicTableView.zElements.each(function(model) {
				model.set('clickable', true)
			});
		},

		onElementTap: function(model) {
			var view = this;
			var time = 0;
			if (this.timeout!=undefined) time = 150;
			setTimeout(function() {
				clearTimeout(view.timeout);
				view.timeout = undefined;
				if (view.isActive && model.get('active')) {
					document.body.classList.remove('showCredits');
					document.body.classList.add('mode6');
					periodicTableView.audio.play("elemento");
					periodicTableView.focusView.open('configuration',model);
				}
			}, time);
		},

		resetConfiguration: function() {
			var elements = this.el.querySelectorAll('.elettronec, .orbitale, .protone');
			for (var i = 0; i < elements.length; i++) {
				var el = elements[i];
				el.style['display'] = 'none';
				el.classList.remove('livello_schiarito');
				el.classList.remove('eletV');
			}
			this.electrons = {};
			this.livels = {};
			this.electronsInside = 0;
		},

		completeConfiguration: function(model) {
			var ne = model.get('number');
			var view = this;
			var str='';
			var bir=null;
			var completed = false;
			if(view.electronsInside!=ne) {
				str = model.get('complete_configuration');
				bir = model.get('levels_matrix');
			} else {
				completed = true;
			}
			var z = 1;
			var liv = 1;
			if (view.levels[liv+' '+z]==undefined) { 
				view.drawLevel(liv,z,view.colours[z-1]);
				view.lastComplete =  (liv+'_'+z);
			}
			if(view.electronsInside<ne){
				view.electronsInside++;
				var boxco = view.spiralNucleus(view.electronsInside);
				if (view.protons[view.electronsInside]==undefined) {
					var np = $('<div class="protone" style="left: '+boxco[0]+'px; top:'+boxco[1]+'px; z-index:'+(109-view.electronsInside)+'" ></div>');
					view.nucleus.appendChild(np[0]);
					view.protons[view.electronsInside] = np;
				}
				else { 
					view.protons[view.electronsInside].show().removeClass('popoutelet'); 
				}
				var currentModel = periodicTableView.zElements.where({ number: view.electronsInside })[0];
				var str2 = currentModel.get('complete_configuration');
				var bir2 = currentModel.get('levels_matrix');
				this.notationSpan.innerHTML = currentModel.get('spanned_configuration');
				var arra = currentModel.get('configuration_array_number');
				var m = currentModel.get('levels_division');
				view.completeSpinGraph(m);
				var liv = 0;
				var z = 0;
				var zt = view.calculateSublevel(view.electronsInside);
				var ar_lamp = [];
				var n=0;
				var sliv;
				for (var i=1; i<=bir2.length; i++){
					var liv = bir2[i-1];
					for (var k=1; k<=liv.length; k++){
						if (view.levels[i+' '+k]==undefined) {
							view.drawLevel(i,k,view.colours[k-1]);
							view.lastComplete = i+' '+k;
						} else {		
							view.levels[i+' '+k].style['display'] = 'block';
						}
						sliv = liv[k-1];
						for (var j=1; j<=sliv; j++){
							n = n+1;
							var position = view.electronPosition(i,k,j);
							if (view.electrons[i+'_'+k+'_'+j]==undefined) {
								var strO = i+'_'+k+'_'+j; 
								switch (strO) {
									case "3_3_4" : 
										view.electronIn(position,4,1,2,view.lastPositions["4_1_2"][0],view.lastPositions["4_1_2"][1]);
										var myEl = view.electrons[strO] = view.electrons["4_1_2"];
										myEl.setAttribute('orbitale',"3_3");
										myEl.setAttribute('livello',"3");
										view.electrons["4_1_2"] = undefined;
										break;
									case "3_3_9" : 
										view.electronIn(position,4,1,2,view.lastPositions["4_1_2"][0],view.lastPositions["4_1_2"][1]);
										var myEl = view.electrons[strO] = view.electrons["4_1_2"];
										myEl.setAttribute('orbitale',"3_3");
										myEl.setAttribute('livello',"3");
										view.electrons["4_1_2"] = undefined;
										break;	
									case "4_3_3" : 
										view.electronIn(position,5,1,2,view.lastPositions["5_1_2"][0],view.lastPositions["5_1_2"][1]);
										var myEl = view.electrons[strO] = view.electrons["5_1_2"];
										myEl.setAttribute('orbitale',"4_3");
										myEl.setAttribute('livello',"4");
										view.electrons["5_1_2"] = undefined;
										break;
									case "4_3_6" : 
										view.electronIn(position,5,1,2,view.lastPositions["5_1_2"][0],view.lastPositions["5_1_2"][1]);
										var myEl = view.electrons[strO] = view.electrons["5_1_2"];
										myEl.setAttribute('orbitale',"4_3");
										myEl.setAttribute('livello',"4");
										view.electrons["5_1_2"] = undefined;
										break;
									case "4_3_9" : 
										view.electronIn(position,5,1,1,view.lastPositions["5_1_1"][0],view.lastPositions["5_1_1"][1]);
										var myEl = view.electrons[strO] = view.electrons["5_1_1"];
										myEl.setAttribute('orbitale',"4_3");
										myEl.setAttribute('livello',"4");
										view.electrons["5_1_1"] = undefined;
										view.levels[5+' '+1].style['display'] = 'none';
										break;
									case "4_4_2" : 
										view.electronIn(position,5,3,1,view.lastPositions["5_3_1"][0],view.lastPositions["5_3_1"][1]);
										var myEl = view.electrons[strO] = view.electrons["5_3_1"];
										myEl.setAttribute('orbitale',"4_4");
										myEl.setAttribute('livello',"4");
										view.electrons["5_3_1"] = undefined;
										view.levels[5+' '+3].style['display'] = 'none';
										break;
									case "4_4_8" : 
										view.electronIn(position,5,3,1,view.lastPositions["5_3_1"][0],view.lastPositions["5_3_1"][1]);
										var myEl = view.electrons[strO] = view.electrons["5_3_1"];
										myEl.setAttribute('orbitale',"4_4");
										myEl.setAttribute('livello',"4");
										view.electrons["5_3_1"] = undefined;
										view.levels[5+' '+3].style['display'] = 'none';
										break;
									case "5_3_8" : 
										view.electronIn(position,6,1,2,view.lastPositions["6_1_2"][0],view.lastPositions["6_1_2"][1]);
										var myEl = view.electrons[strO] = view.electrons["6_1_2"];
										myEl.setAttribute('orbitale',"5_3");
										myEl.setAttribute('livello',"5");
										view.electrons["6_1_2"] = undefined;
										break;
									case "5_4_1" : 
										view.electronIn(position,6,3,2,view.lastPositions["6_3_2"][0],view.lastPositions["6_3_2"][1]);
										var myEl = view.electrons[strO] = view.electrons["6_3_2"];
										myEl.setAttribute('orbitale',"5_4");
										myEl.setAttribute('livello',"5");
										view.electrons["6_3_2"] = undefined;
										break;
									case "5_4_5" : 
										view.electronIn(position,6,3,1,view.lastPositions["6_3_1"][0],view.lastPositions["6_3_1"][1]);
										var myEl = view.electrons[strO] = view.electrons["6_3_1"];
										myEl.setAttribute('orbitale',"5_4");
										myEl.setAttribute('livello',"5");
										view.electrons["6_3_1"] = undefined;
										view.levels[6+' '+3].style['display'] = 'none';
										break;
									case "5_4_8" : 
										view.electronIn(position,6,3,1,view.lastPositions["6_3_1"][0],view.lastPositions["6_3_1"][1]);
										var myEl = view.electrons[strO] = view.electrons["6_3_1"];
										myEl.setAttribute('orbitale',"5_4");
										myEl.setAttribute('livello',"5");
										view.electrons["6_3_1"] = undefined;
										view.levels[6+' '+3].style['display'] = 'none';
										break;
									case "6_3_8" : 
										view.electronIn(position,7,1,2,view.lastPositions["7_1_2"][0],view.lastPositions["7_1_2"][1]);
										var myEl = view.electrons[strO] = view.electrons["7_1_2"];
										myEl.setAttribute('orbitale',"6_3");
										myEl.setAttribute('livello',"6");
										view.electrons["7_1_2"] = undefined;
										break;
									default: 
										view.electronIn(position,i,k,j);
										break;
								}
								view.lastPositions[strO] = position;
								view.lastElectrons.push(i+'_'+k+'_'+j);
							}
						}
					}
				}
				var le = view.lastElectrons[view.lastElectrons.length-1].split('_');
				var i = parseInt(le[0]);
				var k = parseInt(le[1]);
				var j = parseInt(le[2]);
				this.levelLabels['l'].innerHTML = (k-1);
				this.levelLabels['n'].innerHTML = i;
				var tot = view.sublevelsLength[k-1];
				this.levelLabels['l'].innerHTML = (view.quanticMatrix[k-1][j-1]);
				var ar_opac = [];
				view.isLevelComplete(view.lastMatrices);
				view.lastMatrices = m;
			}
			if(view.electronsInside>ne){
				view.electronsInside--;
				var currentModel = periodicTableView.zElements.where({ number: view.electronsInside })[0];
				var str2 = currentModel.get('complete_configuration');
				var bir2 = currentModel.get('levels_matrix');
				this.notationSpan.innerHTML = currentModel.get('spanned_configuration');
				var protons = this.el.querySelectorAll('.protone:not(.popoutelet)');
				protons[protons.length-1].classList.add('popoutelet');
				var arra = currentModel.get('configuration_array_number');
				var m = currentModel.get('levels_division');
				view.completeSpinGraph(m);
				var l_sl = view.calculateLevelAndSublevel(str);
				var liv = 0;
				var z = 0;
				var zt = view.calculateSublevel(view.electronsInside);
				var ar_lamp = [];
				var n=0;
				
				var leIndex = view.lastElectrons.pop();
				view.electronOut(leIndex);
				view.electrons[leIndex] = undefined;		
				for (var i=1; i<=bir2.length; i++){
					var liv = bir2[i-1];
					for (var k=1; k<=liv.length; k++){
						sliv = liv[k-1];
						for (var j=1; j<=sliv; j++){
							n = n+1;
							var position = view.electronPosition(i,k,j);
							var strO = i+'_'+k+'_'+j;
							switch (strO+'-'+view.electronsInside) {
								case "3_3_3-23" : 
									var np = view.electronPosition(4,1,2);
									view.electronIn(np,3,3,4,view.lastPositions["3_3_4"][0],view.lastPositions["3_3_4"][1]);
									var nstrO = '4_1_2';
									var myEl = view.electrons[nstrO] = view.electrons["3_3_4"];
									myEl.setAttribute('orbitale',"4_1");
									myEl.setAttribute('livello',"4");
									view.electrons["3_3_4"] = undefined;
									view.lastElectrons.pop();
									break;
								case "3_3_8-28" : 
									var np = view.electronPosition(4,1,2);
									view.electronIn(np,3,3,9,view.lastPositions["3_3_9"][0],view.lastPositions["3_3_9"][1]);
									var nstrO = '4_1_2';
									var myEl = view.electrons[nstrO] = view.electrons["3_3_9"];
									myEl.setAttribute('orbitale',"4_1");
									myEl.setAttribute('livello',"4");
									view.electrons["3_3_9"] = undefined;
									view.lastElectrons.pop();
									break;	
								case "4_3_2-40" : 
									var np = view.electronPosition(5,1,2);
									view.electronIn(np,4,3,3,view.lastPositions["4_3_3"][0],view.lastPositions["4_3_3"][1]);
									var nstrO = '5_1_2';
									var myEl = view.electrons[nstrO] = view.electrons["4_3_3"];
									myEl.setAttribute('orbitale',"5_1");
									myEl.setAttribute('livello',"5");
									view.electrons["4_3_3"] = undefined;
									view.lastElectrons.pop();
									break;
								case "4_3_5-43" : 
									var np = view.electronPosition(5,1,2);
									view.electronIn(np,4,3,6,view.lastPositions["4_3_6"][0],view.lastPositions["4_3_6"][1]);
									var nstrO = '5_1_2';
									var myEl = view.electrons[nstrO] = view.electrons["4_3_6"];
									myEl.setAttribute('orbitale',"5_1");
									myEl.setAttribute('livello',"5");
									view.electrons["4_3_6"] = undefined;
									view.lastElectrons.pop();
									break;
								case "4_3_8-45" :
									var np = view.electronPosition(5,1,1);
									view.electronIn(np,4,3,9,view.lastPositions["4_3_9"][0],view.lastPositions["4_3_9"][1]);
									var nstrO = '5_1_1';
									var myEl = view.electrons[nstrO] = view.electrons["4_3_9"];
									myEl.setAttribute('orbitale',"5_1");
									myEl.setAttribute('livello',"5");
									view.electrons["4_3_9"] = undefined;
									view.lastElectrons.pop();
									view.levels[5+' '+1].style['display'] = 'block';
									break;
								case "4_4_1-58" : 
									var np = view.electronPosition(5,3,1);
									view.electronIn(np,4,4,2,view.lastPositions["4_4_2"][0],view.lastPositions["4_4_2"][1]);
									var nstrO = '5_3_1';
									var myEl = view.electrons[nstrO] = view.electrons["4_4_2"];
									myEl.setAttribute('orbitale',"5_3");
									myEl.setAttribute('livello',"5");
									view.electrons["4_4_2"] = undefined;
									view.levels[5+' '+3].style['display'] = 'block';
									view.lastElectrons.pop();
									break;
								case "5_3_1-64" : 
									var np = view.electronPosition(5,3,1);
									view.electronIn(np,4,4,8,view.lastPositions["4_4_8"][0],view.lastPositions["4_4_8"][1]);
									var nstrO = '5_3_1';
									var myEl = view.electrons[nstrO] = view.electrons["4_4_8"];
									myEl.setAttribute('orbitale',"5_3");
									myEl.setAttribute('livello',"5");
									view.electrons["4_4_8"] = undefined;
									view.lastElectrons.pop();
									view.levels[5+' '+3].style['display'] = 'block';
									break;
								case "5_3_7-77" : 
									var np = view.electronPosition(6,1,2);
									view.electronIn(np,5,3,8,view.lastPositions["5_3_8"][0],view.lastPositions["5_3_8"][1]);
									var nstrO = '6_1_2';
									var myEl = view.electrons[nstrO] = view.electrons["5_3_8"];
									myEl.setAttribute('orbitale',"6_1");
									myEl.setAttribute('livello',"6");
									view.electrons["5_3_8"] = undefined;
									view.lastElectrons.pop();
									view.levels[6+' '+1].style['display'] = 'block';
									break;
								case "5_3_2-90" : 
									var np = view.electronPosition(6,3,2);
									view.electronIn(np,5,4,1,view.lastPositions["5_4_1"][0],view.lastPositions["5_4_1"][1]);
									var nstrO = '6_3_2';
									var myEl = view.electrons[nstrO] = view.electrons["5_4_1"];
									myEl.setAttribute('orbitale',"6_3");
									myEl.setAttribute('livello',"6");
									view.electrons["5_4_1"] = undefined;
									view.lastElectrons.pop();
									view.levels[6+' '+3].style['display'] = 'block';
									break;
								case "5_3_1-93" : 
									var np = view.electronPosition(6,3,1);
									view.electronIn(np,5,4,5,view.lastPositions["5_4_5"][0],view.lastPositions["5_4_5"][1]);
									var nstrO = '6_3_1';
									var myEl = view.electrons[nstrO] = view.electrons["5_4_5"];
									myEl.setAttribute('orbitale',"6_3");
									myEl.setAttribute('livello',"6");
									view.electrons["5_4_5"] = undefined;
									view.lastElectrons.pop();
									view.levels[6+' '+3].style['display'] = 'block';
									break;
								case "6_3_1-96" : 
									var np = view.electronPosition(6,3,1);
									view.electronIn(np,5,4,8,view.lastPositions["5_4_8"][0],view.lastPositions["5_4_8"][1]);
									var nstrO = '6_3_1';
									var myEl = view.electrons[nstrO] = view.electrons["5_4_8"];
									myEl.setAttribute('orbitale',"6_3");
									myEl.setAttribute('livello',"6");
									view.electrons["5_4_8"] = undefined;
									view.lastElectrons.pop();
									view.levels[6+' '+3].style['display'] = 'block';
									break;
								case "6_3_9" : 
									var np = view.electronPosition(7,1,2);
									view.electronIn(np,6,3,8,view.lastPositions["6_3_8"][0],view.lastPositions["6_3_8"][1]);
									var nstrO = '7_1_2';
									var myEl = view.electrons[nstrO] = view.electrons["6_3_8"];
									myEl.setAttribute('orbitale',"7_1");
									myEl.setAttribute('livello',"7");
									view.electrons["6_3_8"] = undefined;
									view.lastElectrons.pop();
									break;
							}
						}
					}
				}
				var le = view.lastElectrons[view.lastElectrons.length-1].split('_');
				var i = parseInt(le[0]);
				var k = parseInt(le[1]);
				var j = parseInt(le[2]);
				this.levelLabels['l'].innerHTML = (k-1);
				this.levelLabels['n'].innerHTML = i;
				var tot = view.sublevelsLength[k-1];
				this.levelLabels['m'].innerHTML = (view.quanticMatrix[k-1][j-1]);
				view.isLevelComplete(m);
				view.isLevelToHide(m);
				view.lastMatrices = m;
				
			}

			//window.requestAnimationFrame(function() {
				if (!completed) {
					view.timeout = setTimeout(function(){
						view.completeConfiguration(model);
					},150);
				}
			//})
		},

		drawLevel: function(liv,sliv,color) {
			if (sliv==undefined) sliv = 1;
			if (liv==undefined) liv = 1;
			var dist = sliv*10;
			var new_level_div = document.createElement('div');
			new_level_div.setAttribute('class','orbitale');
			new_level_div.setAttribute('id',('or'+liv+'_'+sliv));
			new_level_div.setAttribute('livello',liv);
			new_level_div.style.borderRadius = (this.levelRadius*liv*2+dist+'px');
			this.levelRadius = Math.round(this.levelRadius);
			var style = new_level_div.style;
			style['top'] = (this.centerOffsetX-dist) + 'px';
			style['left'] = (this.centerOffsetX-dist) + 'px';
			style['width'] = (this.levelRadius*liv*2+dist) + 'px';
			style['height'] = (this.levelRadius*liv*2+dist) + 'px';
			style['border'] = 'solid 1px '+color;
			style['marginLeft'] = (-this.levelRadius*liv+dist*.5) + 'px';
			style['marginTop'] = (-this.levelRadius*liv+dist*.5) + 'px';
			this.canvas.appendChild(new_level_div);
			this.levels[liv+' '+sliv] = new_level_div;
			new_level_div = this.levels[sliv];
		},

		spiralNucleus: function(nec) {
			var mr = 4;
			var dif = 1;
			var z = 0;
			var liv = 0;
			if (nec>=2 && nec<=9) {z=nec-2; liv=0;}
			if (nec>=10 && nec<=25) {z=nec-10; liv=1;}
			if (nec>=26 && nec<=50) {z=nec-26; liv=2;}
			if (nec>=51 && nec<=75) {z=nec-51; liv=3;}
			if (nec>=76 && nec<=109) {z=nec-76; liv=4;}
			z++;
			var r_circ = mr*liv;
			if (liv>0) r_circ -= 1/liv;
			var coX = r_circ*Math.cos(Math.PI/2+(Math.PI)+z);
			var coY = r_circ*Math.sin(Math.PI/2+(Math.PI)+z);
			var zi = 10-liv;
			return [coX,coY,zi];
		},

		electronPosition: function(liv,sl,ne) {
			ne--;
			var ang=1.5*Math.PI;
			var lung2 = (this.sublevelsLength[sl-1]);
			var dif=2*Math.PI/lung2;
			var r_circ =this.levelRadius*(liv+sl*.2);
			var x = this.centerOffsetX+r_circ*Math.cos(Math.PI/2+(Math.PI)*(ne%2)+dif*(~~(ne/2))+(sl-1)*3.05);
			var y = this.centerOffsetX+r_circ*Math.sin(Math.PI/2+(Math.PI)*(ne%2)+dif*(~~(ne/2))+(sl-1)*3.05);
			if (sl==2 && ne>1){
				var per = 1;
				if (ne>3) per=-1;
				x = this.centerOffsetX+r_circ*Math.cos(Math.PI/2+(Math.PI)*(ne%2)+1.35*per+(sl-1)*3.05);
				y = this.centerOffsetX+r_circ*Math.sin(Math.PI/2+(Math.PI)*(ne%2)+1.35*per+(sl-1)*3.05);
			}
			return [x,y,this.colours[sl-1],liv,sl];
		},

		isLevelComplete: function(m) {
			var view = this;
			for (var liv = 1; liv<=m.length; liv++) {
				var complete = true;
				var lung_liv = view.levelsComplete[liv-1];
				var sm = m[liv-1];
				for (var j=1; j<=lung_liv.length; j++){
					var sme = sm[j-1];
					var lung_sliv = lung_liv[j-1];
					if (sme != lung_sliv && sme/1000 != lung_sliv) complete = false;
				}
				if (complete && view.levelsCompleted.indexOf(liv)==-1) {
					view.levelsCompleted.push(liv);
					var elements = view.el.querySelectorAll('.elettronec[livello="'+liv+'"], .orbitale[livello="'+liv+'"]');
					for (var i = 0; i < elements.length; i++) {
						elements[i].classList.add('livello_schiarito');
					}
				} else {
					if (!complete) {
						view.levelsCompleted[view.levelsCompleted.indexOf(liv)] = 0;
						var elements = view.el.querySelectorAll('.elettronec[livello="'+liv+'"], .orbitale[livello="'+liv+'"]');
						for (var i = 0; i < elements.length; i++) {
							elements[i].classList.remove('livello_schiarito');
						}
					}
				}
				if (complete && m.length<view.lastMatrices.length){
					view.levelsCompleted[view.levelsCompleted.indexOf(liv)] = 0;
					var elements = view.el.querySelectorAll('.elettronec[livello="'+m.length+'"], .orbitale[livello="'+m.length+'"]');
					for (var i = 0; i < elements.length; i++) {
						elements[i].classList.remove('livello_schiarito');
					}
				}
			}
		},

		isLevelToHide: function(m) {
			var last;
			var liv = cloneObject(this.levels);
			for (var ri = 0; ri<m.length; ri++) {
				var mri = m[ri];
				for (var rk = 0; rk<mri.length; rk++) {
					delete liv[(ri+1)+' '+(rk+1)];
				}
			}
			if (liv.length>0) last = this.levelsCompleted.pop();
			for (var co in liv) {
				$(liv[co]).hide();
			}
		},

		calculateLevelAndSublevel: function(str) {
			var arliv = (str.split('</sup>'));
			var box = (arliv[arliv.length-2]);
			var liv = parseInt(box[0]);
			var sliv = this.sublevelsSymbols.indexOf(box[1]) + 1;
			return [liv,sliv];
		},

		calculateSublevel: function(ne) {
			var view = this;
			var i=0;
			while (ne>0) {
				ne -= view.electronsNumberForLevel[i];
				i++;
			}
			return i;
		},

		completeSpinGraph: function(ar) {
			var view = this;
			var spins = this.spins;
			for (var i = 0; i < spins.length; i++) {
				spins[i].innerHTML = '';
			}
			for (var i=0; i<ar.length; i++){
				var	arliv = ar[i];
				for (var k=0; k<arliv.length; k++){
					var val = arliv[k];
					if (val%1000==0) val=val/1000;			
					var tab = view.tables[k];
					var artd  = tab.querySelectorAll('[class="' + (i+1) + '"] td');
					var livlength = artd.length;
					for (var z = 0; z<val; z++){
						if (z<livlength)
							artd[z%livlength].children[0].innerHTML = 'ç';
						else 
							artd[z%livlength].children[0].innerHTML = '§';
					}
				}
			}
		},

		animate: function(el,el2,x,y,d,o,callback) {
			el = $(el);
			el2 = $(el2);
			if (BrowserInfo.Webkit() && Modernizr.csstransforms3d) {
				transition(el, { translateX: x }, d, callback);
				transition(el2, { translateY: y }, d, callback);
			} else {
				transition(el, { left: x, top: y, opacity: o }, d, callback);
			}
		},

		electronIn: function(co,liv,sliv,ne,xi,yi) {
			var view = this;
			var x_target=co[0];
			var y_target=co[1];
			if (view.electrons[liv+'_'+sliv+'_'+ne]==undefined) {
				var nwel = document.createElement('div');
				var nweli = document.createElement( 'div' );
				nwel.setAttribute('class','elettronec');
				nwel.setAttribute('orbitale',liv+'_'+sliv);
				nwel.setAttribute('livello',liv);
				var slv = view.colours.indexOf(co[2]);
				nweli.setAttribute('class','elettrone elet'+(view.sublevelsSymbols[slv]).toUpperCase());
				nwel.appendChild(nweli);
				this.atom.appendChild(nwel);
		   		view.electrons[liv+'_'+sliv+'_'+ne] = nwel;
		    }
			var nel = view.electrons[liv+'_'+sliv+'_'+ne];
			nel.style['display'] = 'block';
			var boxar = [x_target,y_target];
			this.electronsObject[liv+'_'+sliv+'_'+ne]=boxar;
			var o_or_v = (Math.random(1)*2)|0;
			var xs=0;
			var ys=0;
			if (x_target<view.canvasWidth*.5) {
				xs = 0;
				ys = Math.random(100)*view.canvasWidth;
			} else {
				xs = view.canvasWidth;
				ys = Math.random(100)*view.canvasWidth;
			}
			if (xi==undefined) {
				view.animate(nel, nel.children[0],xs,ys,0,0);
			} else {
				var slv = view.colours.indexOf(co[2]);
				var cl = nel.children[0].classList;
				cl.remove('eletS');
				cl.remove('eletP');
				cl.remove('eletD');
				cl.remove('eletF');
				cl.add('elet'+(view.sublevelsSymbols[slv]).toUpperCase());
			}
			var molt = 1;
			if (xs>x_target) molt=-1;
			setTimeout(function(){
				nel.classList.add('eletV');
				view.animate(nel,nel.children[0],x_target,y_target,500,1);
			},5);
		},

		electronOut: function(ne){
			var view = this;
			var info = (this.electronsObject[ne]);
			var nel = view.electrons[ne];
			var cl = nel.classList;
			cl.remove('eletV');
			cl.remove('livello_schiarito');
			var o_or_v = (Math.random(1)*2)|0;
			var xs=0;
			var ys=0;
			if (info[0]<view.canvasWidth*.5) {
				if (o_or_v==0){
					if (info[1]<view.canvasWidth*.5) ys = 0; else ys = view.canvasWidth;
					xs = Math.random(100)*(info[0]);
				} else {
					xs = 0;
					ys = Math.random(100)*view.canvasWidth;
				}
			} else {
				if (o_or_v==0){
					if (info[1]<view.canvasWidth*.5) ys = 0; else ys = view.canvasWidth;
					xs = Math.random(100)*(view.canvasWidth-info[0])+info[0];
				} else {
					xs = view.canvasWidth;
					ys = Math.random(100)*view.canvasWidth;
				}
			}
			view.animate(nel,nel.children[0],xs,ys,500,0, function() {
				if (nel.parentNode) nel.parentNode.removeChild(nel);
			});
		},

		deSelect: function(model) {
			this.resetConfiguration();
			if (this.isActive && model) {
				model.set('active',false);
			}
		},
	});

}());