(function() {
	'use strict';

	PeriodicTable.elementView = Backbone.View.extend({
		horizontalLabel: null,
		verticalLabel: null,

		events: {
			"click": "triggerOpen",
			"mouseenter": "mouseenter",
			"mouseleave": "mouseleave",
		},

		initialize: function() {
			var view = this;
			var div = this.el;
			var data = this.model.toJSON();
			var number = data.number;
			div.classList.add('elemento');
			div.classList.add(data.group);
			var html = '';
			if (number==57) html += '<div id="b1"></div>';
			if (number==89) html += '<div id="b2"></div>';

			html +=
				'<span class="tl">'+number+'</span>' +
				'<span class="simbolo">'+data.symbol+'</span>' +
				'<span class="nome">'+data.name+'</span>';

			div.innerHTML = html;

			Backbone.on('periodictable:shown', this.onReady, this);
			Backbone.on('focusplus:open', this.onPlusOpen, this);
			Backbone.on('focusplus:close', this.onPlusClose, this);

			return this;
		},

		onReady: function() {
			var data = this.model.toJSON();
			this.el.innerHTML +=
				'<img class="elimg" src="images/'+data.image_url+'" width="100%" />' +
				'<span class="eldesc">'+data.image_description+'</span>' +
				'<div class="elscheda">'+data.more_text+'</div>' +
				'<span class="grp">'+data.group+'</span>' +
				'<span class="datihide tema">&nbsp;</span>';

			this.model.set('elementView', this);
			$.data(this.el, 'hasScheda', this.model.get('more_text').length>0);

			var td = this.el.parentNode;
			var tr = td.parentNode;
			var iv = document.getElementById('iv');

			var k = Array.prototype.slice.call(tr.children).indexOf(td);
			if (tr.id == 'w1' || tr.id == 'w2') k=3;

			this.verticalLabel = $(iv.children[k]);

			if (tr.id == 'w1') {
				var el57 = document.getElementById('el57');
				tr = el57.parentNode.parentNode;
			} else if (tr.id == 'w2') {
				var el87 = document.getElementById('el87');
				tr = el87.parentNode.parentNode;
			}
			this.horizontalLabel = $(tr.children[0]);
		},

		width: function () {
			return (this.el.getBoundingClientRect) ? this.el.getBoundingClientRect().width : this.$el.width();
		},

		left: function () {
			return (this.el.getBoundingClientRect) ? this.el.getBoundingClientRect().left : this.$el.offset().left;
		},

		getClass: function() {
			return this.$el.attr('class');
		},

		setClass: function(classesToAdd,classesToRemove) {
			if (classesToRemove && classesToRemove!='' && classesToRemove!=' ') this.$el.removeClass(classesToRemove);
			if (classesToAdd && classesToAdd!='' && classesToAdd!=' ') this.$el.addClass(classesToAdd);
		},

		toggleActive: function(model) {
			if (model.get('active') && model.get('number')==this.model.get("number")) {
				this.select();
			} else {
				this.model.attributes.active = false;
				this.deSelect(model);
			}
		},

		triggerOpen: function() {
			if (this.model.get('clickable')) this.model.set('active',true);
		},

		select: function() {
			this.$el.addClass('elementoact');
			this.verticalLabel.addClass('cellactivec');
			this.horizontalLabel.addClass('cellactivec');
		},

		deSelect: function(model) {
			if (this.$el.hasClass('elementoact')) {
				this.model.set('active', false);
				this.verticalLabel.removeClass('cellactivec cellactive');
				this.horizontalLabel.removeClass('cellactivec cellactive');
				this.$el.removeClass('elementoact');
			}
			this.$el.removeClass('elementover');
		},

		mouseenter: function(ev) {
			if (this.model.get('clickable')) {
				if (!Modernizr.touch) this.$el.addClass('elementover');
				this.getCooX();
				this.getCooY();
			}
		},

		mouseleave: function() {
			if (this.model.get('clickable')) {
				$('.cellactive').removeClass('cellactive');
				this.$el.removeClass('elementover');
			}
		},

		getCooX: function(elem) {
			this.verticalLabel.addClass('cellactive');
		},

		getCooY: function(elem){
			this.horizontalLabel.addClass('cellactive');
		},

		onPlusOpen: function() {
			this.model.set('clickable', false);
		},

		onPlusClose: function() {
			this.model.set('clickable', true);
		},
	});

}());
