(function() {
	'use strict';

	PeriodicTable.FocusView = Backbone.View.extend({

		events: {
			"click #close": "close",
			"click #next": "next",
			"click #prev": "prev",
			"click .plus": "togglePlus"
		},

		/*** attributes ***/
		isOpen: false,
		isOpening: false,
		isClosing: false,
		plusOpen: false,
		isScrolling: false,
		currentModel: null,

		/*** position ref elements ***/
		wrapper: null,
		halfTableHeader: null,
		panel: null,
		scroll: null,
		scroller: null,
		scrollbar: null,
		plusToggle: null,
		whiteSpace: null,
		ingData: null,
		ingTemp: null,
		ing1: null,
		ing2: null,
		ing3: null,
		ingc1: null,
		ingt1: null,
		ingt2: null,
		ingt3: null,
		ingt4: null,

		iscrollPanel: null,

		initialize: function() {
			var that = this;
			Backbone.on('credits:open', this.close, this);
			Backbone.on('temperature:change', this.changeTemperatureClass, this);

			this.wrapper = $('#wrap');
			this.halfTableHeader = $('#halfTableHeader');
			this.panel = $('#scheda');
			this.scroll = $('.scroll');
			this.scroller = $('.scroller');
			this.scrollbar = $('.scrollbar');
			this.plusToggle = $('.plus')[0];
			this.whiteSpace = $('.tc')[0];
			this.ingData = document.getElementById('dati');
			this.ing1 = document.getElementById('ing1');
			this.ing2 = document.getElementById('ing2');
			this.ing3 = document.getElementById('ing3');
			this.ingc1 = document.getElementById('ingc1');
			this.ingt1 = document.getElementById('ingt1');
			this.ingt2 = document.getElementById('ingt2');
			this.ingt3 = document.getElementById('ingt3');
			this.ingt4 = document.getElementById('ingt4');

			this.iscrollPanel = new IScroll($('.scrollpanel')[0], {
				scrollbars: true,
				mouseWheel: true,
				scrollX: false,
				scrollY: true,
				bounce: false,
				momentum: true,
				useTransition: false,
				HWCompositing: false,
				interactiveScrollbars: true,
			});

			$(document).bind('keydown', function (ev) {
				that.onKeyDown(ev);
			});
		},

		onKeyDown: function (ev) {
			if (this.isOpen) {
				var code = ev.keyCode,
					that = this;
				switch(code) {
					case 39: //next
						ev.preventDefault();
						that.next();
						break;
					case 37: //prev
						ev.preventDefault();
						that.prev();
						break;
					case 38: //up
						ev.preventDefault();
						that.prev();
						break;
					case 40: //down
						ev.preventDefault();
						that.next();
						break;
					case 27: //esc
						ev.preventDefault();
						that.close();
						break;
					case 107: //plus
						ev.preventDefault();
						if (!this.plusOpen) {
							this.openPlus();
						}
						break;
					case 109: //minus
						ev.preventDefault();
						if (this.plusOpen) {
							this.closePlus();
						}
						break;
				}
			}
		},

		open: function(mode,model) {
			var view = this;
			if (mode!="configuration") this.position(model);
			this[mode](model);
			this.currentModel = model;
		},

		slideDown: function(callback) {
			var view = this;
			this.isOpen=true;
			this.isOpening = true;

			var storeClickable = [];
			var models = periodicTableView.zElements.models;

			for (var i=0; i<models.length; i++) {
				var model = models[i];
				storeClickable.push(model.get('clickable'));
				model.set('clickable', false);
			}

			var fcallback = function() {
				view.isOpening = false;
				for (var i=0; i<models.length; i++) {
					var model = models[i];
					model.set('clickable', storeClickable[i]);
				}
				if (callback) callback();
			}
			this.$el.show();
			if (Modernizr.csstransforms3d) {
				transition(this.$el,{ translateY: 700 }, 500, fcallback);
			} else {
				transition(this.$el,{ top: 0 }, 500, fcallback);
			}
		},

		slideUp: function(callback) {
			var view = this;
			this.isOpen=false;

			var storeClickable = [];
			var models = periodicTableView.zElements.models;

			for (var i=0; i<models.length; i++) {
				var model = models[i];
				storeClickable.push(model.get('clickable'));
				model.set('clickable', false);
			}

			var fcallback = function() {
				view.isClosing = false;
				for (var i=0; i<models.length; i++) {
					var model = models[i];
					model.set('clickable', storeClickable[i]);
				}
				if (callback) callback();
			}

			if (Modernizr.csstransforms3d) {
				transition(this.$el,{ translateY: 0 }, 500, fcallback);
			} else {
				transition(this.$el,{ top: -700 }, 500, fcallback);
			}

		},

		close: function(ev, callback) {
			if (ev) {
				ev.stopPropagation();
				ev.preventDefault();
			}
			var view = this;
			this.isClosing=true;
			this.panel.hide();
			this.closePlus();
			$('body').removeClass('dettaglio');
			periodicTableView.audio.play("scheda");
			if (this.isOpen) {
				this.slideUp(callback);
			}
			Backbone.trigger("focus:close", periodicTableView.getActiveElement());
			return false;
		},

		position: function(model) {
			var view = this;
			this.$el.clearQueue();
			var ev = model.get('elementView');
			var offLeft = ev.left();
			var l = (offLeft - this.wrapper.offset().left) / zoom;
			var mov;
			if (Math.round(offLeft) <= Math.round(this.halfTableHeader.offset().left+2)) {
				mov = (l + ev.width() * 3);
			} else {
				mov = (l - ev.width() * 2 - this.$el.width());
			}
			if (Modernizr.csstransforms3d) {
				transition(this.panel,{ translateX: 0 }, 0);
			} else {
				transition(this.panel,{ left: 0 }, 0);
			}
			if (this.isOpen) {
				if (Modernizr.csstransforms3d) {
					transition(this.$el,{ translateX: mov }, 500)
				} else {
					transition(this.$el,{ left: mov }, 500)
				}
			} else {
				if (Modernizr.csstransforms3d) {
					transition(this.$el,{ translateX: mov }, 0, function() { view.slideDown() });
				} else {
					transition(this.$el,{ left: mov }, 0, function() { view.slideDown() });
				}
			}
		},

		next: function(ev) {
			if (ev) {
				ev.stopPropagation();
				ev.preventDefault();
			}
			if (this.plusOpen) this.closePlus();
			var id = periodicTableView.getActiveElement().get("number")+1;
			if (id>periodicTableView.zElements.models.length) return false;
			var next = periodicTableView.zElements.where({ number: id })[0];
			if (next.get('clickable')) {
				next.set({active: true});
				this.currentModel = next;
			}
			return false;
		},

		prev: function(ev) {
			if (ev) {
				ev.stopPropagation();
				ev.preventDefault();
			}
			if (this.plusOpen) this.closePlus();
			var id = periodicTableView.getActiveElement().get("number")-1;
			if (id <= 0) return false;
			var prev = periodicTableView.zElements.where({ number: id })[0];
			if (prev.get('clickable')) {
				prev.set({active: true});
				this.currentModel = prev;
			}
			return false;
		},

		togglePlus: function(ev) {
			if (ev) {
				ev.stopPropagation();
				ev.preventDefault();
			}
			if (this.plusOpen) {
				this.closePlus();
			} else {
				this.openPlus();
			}
		},

		openPlus: function() {
			if (this.currentModel && !this.currentModel.hasMoreText()) {
				return;
			}
			var view = this;
			this.iscrollPanel.scrollTo(0,0,0);
			this.plusOpen = true;
			var panel = this.panel[0];
			panel.classList.add('scheda_aperta');
			panel.style['display'] = 'block';
			if (Modernizr.csstransforms3d) {
				transition(this.$el,{ translateX: 30 }, 500);
				transition(this.panel,{ translateX: 345 }, 500);
			} else {
				transition(this.$el,{ left: 30 }, 500);
				transition(this.panel,{ left: 345 }, 500);
			}
			Backbone.trigger("focusplus:open");
			this.plusToggle.textContent = '-';
			this.iscrollPanel.refresh();
			periodicTableView.audio.play("scheda");
			return false;
		},

		closePlus: function() {
			var view = this;
			if (this.plusOpen) {
				this.plusOpen = false;
				var panel = this.panel[0];
				panel.style['display'] = 'none';
				view.panel.hide();
				if (Modernizr.csstransforms3d) {
					transition(view.panel,{ translateX: 0 }, 0);
				} else {
					transition(view.panel,{ left: 0 }, 0);
				}
				if (!this.isClosing) this.position(this.currentModel);
				Backbone.trigger("focusplus:close");
			}
			this.plusToggle.textContent = '+';
			periodicTableView.audio.play("scheda");
			return false;
		},

		simple: function(model) {
			var cl = model.get('group');
			var ev = model.get('elementView');
			var el = ev.el;
			var $el = ev.$el;

			var ing1 = this.ing1;
			ing1.setAttribute('class', 'tavola_simb '+cl);
			ing1.innerHTML = '';
			ing1.appendChild(el.querySelector('.tl').cloneNode(true));
			ing1.appendChild(el.querySelector('.simbolo').cloneNode(true));
			ing1.appendChild(el.querySelector('.nome').cloneNode(true));

			var ing2 = this.ing2;
			ing2.setAttribute('class', 'tavola_img');
			ing2.innerHTML = '';
			ing2.appendChild(el.querySelector('img').cloneNode());

			var desc = model.get('image_description');

			var ing3 = this.ing3;
			ing3.setAttribute('class', 'tavola_desc');
			ing3.innerHTML = '';

			if (desc.length>5) ing3.appendChild(el.querySelector('.eldesc').cloneNode(true));

			var scroll = this.scroll[0];
			scroll.innerHTML = '';
			var elScroll = el.querySelector('.elscheda').cloneNode(true);
			scroll.appendChild(elScroll);
			elScroll.innerHTML = '<br><br>' + elScroll.innerHTML + '<br><br>';

			var data = this.el.querySelectorAll('#tabelladati td');
			for (var k = 0; k<data.length; k++) {
				var td = data[k];
				var children = td.children;
				var title = children[0].getAttribute('rel');
				var d = model.get(title);
				if (title=="melting_temperature" || title=="boiling_temperature") {
					var unit = TPCONF['show_temperature_in_C'] ? '°C' : 'K';
					var text = td.firstChild.textContent;
					text = text.replace('(K)', '').replace('(°C)', '');
					text += '('+unit+')';
					td.firstChild.textContent = text;
					if (TPCONF['show_temperature_in_C'] && d!='/') {
						d -= 273;
					}
				}
				children[children.length-1].innerHTML = '<b>'+d+'<b>';
			}

			this.ingData.style['display'] = 'block';
			ing3.style['display'] = 'block';
			if (model.hasMoreText()) {
				this.plusToggle.style['display'] = 'table-cell';
				this.whiteSpace.style['display'] = 'none';
			} else {
				this.plusToggle.style['display'] = 'none';
				this.whiteSpace.style['display'] = 'table-cell';
			}
		},

		temperature: function(model) {
			var cl = model.get('currentStatus')+' '+model.get('group');
			var ev = model.get('elementView');
			var el = ev.el;
			var $el = ev.$el;
			this.ingData.style['display'] = 'none';

			var ing1 = this.ingt1;
			ing1.setAttribute('class', 'tavola_simb '+cl);
			ing1.innerHTML = '';
			ing1.appendChild(el.querySelector('.tl').cloneNode(true));
			ing1.appendChild(el.querySelector('.simbolo').cloneNode(true));
			ing1.appendChild(el.querySelector('.nome').cloneNode(true));

			var ing2 = this.ingt2;
			ing2.setAttribute('class', 'tavola_img');
			ing2.innerHTML = '';
			ing2.appendChild(el.querySelector('img').cloneNode());

			var ing3 = this.ingt3;
			ing3.setAttribute('class', 'ing_l');
			ing3.innerHTML = '<span class="titoli_temp">Temperatura di fusione:</span>';

			var unit = TPCONF['show_temperature_in_C'] ? '°C' : 'K';
			var melting = model.get('melting_temperature');
			var boiling = model.get('boiling_temperature');
			if (TPCONF['show_temperature_in_C']) {
				melting -= 273;
				boiling -= 273;
			}
			if (isNaN(melting)) melting = '/';
			if (isNaN(boiling)) boiling = '/';

			ing3.innerHTML +=
				'<span class="dati_temp">'+melting+' '+unit+'</span>' +
				'<span class="titoli_temp">Temperatura di ebollizione:</span>' +
				'<span class="dati_temp">'+boiling+' '+unit+'</span>' +
				'<span class="titoli_temp">Stato fisico in condizioni standard:</span>' +
				'<span class="dati_temp">'+model.get('standard_physical_state')+'</span>';

			var ing4 = this.ingt4;
			ing4.setAttribute('class', 'tavola_desc');
			ing4.innerHTML = '';
			ing4.appendChild(el.querySelector('.eldesc').cloneNode(true));

			this.plusToggle.style['display'] = 'none';
			this.whiteSpace.style['display'] = 'table-cell';
			this.changeTemperatureClass();
		},

		changeTemperatureClass: function() {
			if (periodicTableView.getActiveElement()) {
				var view = periodicTableView.getActiveElement().get('elementView');
				var color = view.$el.css('background-color');
				this.ingt1.style['backgroundColor'] = color;
			}
		},

		configuration: function(model) {
			var view = this;
			this.plusToggle.style['display'] = 'none';
			this.ingData.style['display'] = 'none';
			this.whiteSpace.style['display'] = 'table-cell';
			var id = model.get('number');
			var ev = model.get('elementView');
			var el = ev.el;
			var $el = ev.$el;
			var cl = model.get('group');

			var ing1 = this.ingc1;
			ing1.setAttribute('class', 'tavola_simb '+cl);
			ing1.innerHTML = '';
			ing1.appendChild(el.querySelector('.tl').cloneNode(true));
			ing1.appendChild(el.querySelector('.simbolo').cloneNode(true));
			ing1.appendChild(el.querySelector('.nome').cloneNode(true));

			if (this.isOpen) {
				if (Modernizr.csstransforms3d) {
					transition(this.$el,{ translateX: 100 }, 500);
				} else {
					transition(this.$el,{ left: 100 }, 500);
				}
				periodicTableView.configurationView.completeConfiguration(model);
			} else {
				if (Modernizr.csstransforms3d) {
					transition(this.$el,{ translateX: 100 }, 0, function() { view.slideDown(function() {
							periodicTableView.configurationView.completeConfiguration(model);
						});
					});
				} else {
					transition(this.$el,{ left: 100 }, 0, function() { view.slideDown(function() {
							periodicTableView.configurationView.completeConfiguration(model);
						});
					});
				}
			}
		}
	});

}());
