(function() {
	'use strict';

	PeriodicTable.HistoryView = Backbone.View.extend({

		model: new PeriodicTable.history,

		initialize: function() {
			this.listenTo(periodicTableView.zElements,'change:active',this.changeElement);
			Backbone.on('mode:change', this.changeMode, this);
			Backbone.on('temperature:change', this.changeTemperature, this);
			Backbone.on("focusplus:open", this.openPlus, this);
			Backbone.on("focusplus:close", this.closePlus, this);
			Backbone.on("credits:open", this.openCredits, this);
			Backbone.on("credits:close", this.closeCredits, this);
			Backbone.on("class:open", this.openClass, this);
			Backbone.on("class:close", this.closeClass, this);
			Backbone.on("theme:open", this.openTheme, this);
			Backbone.on("theme:close", this.closeTheme, this);
			Backbone.on('periodictable:ready', this.syncViews, this);
			this.listenTo(this.model, 'change', this.updateHash);
		},

		syncViews: function() {
			if (TPCONF.trace_history==true || TPCONF.trace_history==undefined) {
				var view = this;
				var hash = window.location.hash.slice(1).replace('element:', 'zelement:');
				var ar = hash.split('/');
				for (var i=0; i<ar.length; i++) {
					var reg = ar[i].split(':');
					view.model.attributes[reg[0]] = reg[1];
					var prop = reg[0].charAt(0).toUpperCase() + reg[0].slice(1);
					var cmd = 'set'+prop;
					if (periodicTableView[cmd]) periodicTableView[cmd](reg[1]);
				}
			}
		},

		changeMode: function(rel) {
			this.model.set({
				'temperature': null,
				'theme': null,
				'class': null,
				'mode': rel,
			});
		},

		changeElement: function(model) {
			var number = (model.get('active')) ? model.get('number') : null;
			this.model.set('zelement', number);
		},

		changeTemperature: function(number) {
			if (this.model.get('mode')=='temperature') this.model.set('temperature', number);
		},

		openPlus: function() {
			this.model.set('plus', true);
		},

		closePlus: function() {
			this.model.set('plus', null);
		},

		openCredits: function() {
			this.model.set('credits', true);
		},

		closeCredits: function() {
			this.model.set('credits', null);
		},

		openClass: function(cl) {
			this.model.set('class', cl);
		},

		closeClass: function() {
			this.model.set('class', null);
		},

		openTheme: function(cl) {
			this.model.set('theme', cl);
		},

		closeTheme: function() {
			this.model.set('theme', null);
		},

		updateHash: function() {
			if (TPCONF.trace_history==true || TPCONF.trace_history==undefined) {
				var attr = this.model.toJSON();
				var hash = '';
				for (var k in attr) {
					var val = attr[k];
					if (val!=null && !(k=='mode' && val=='simple')) hash += k+':'+val+'/';
				}
				hash = hash.slice(0, hash.length-1).replace('zelement:', 'element:');
				window.location.hash = hash;
			}
		},

	});

}());