var PeriodicTable = {};
PeriodicTable.domain = TPCONF.server_url || 'http://www.PeriodicTable.lcl/';
var periodicTable;

(function() {
	'use strict';

	periodicTable = Backbone.View.extend({
		el: '#wrap',
		/*** attributes ***/
		zElements: null,
		audio: {},
		zoom: 1,
		activeElement: null,
		$wrapper: null,
		$window: $(window),

		commonView: null,
		simpleView: null,
		temperatureView: null,
		classesView: null,
		thematicsView: null,
		configurationView: null,
		focusView: null,

		historyView: null,

		currentView: undefined,

		labels: null,
		allElementsData: null,

		events: {
			"click #labelbox .label": "switchMenu",
			"click #creditslink": "toggleCredits",
			"click #creditsbox a": "manageExternalLink",
		},

		cookieName: "ZanichelliPeriodicTable",

		initialize: function() {
			var view = this;
			var zElements = new PeriodicTable.Elements();
			this.zElements = zElements;

			var wp = this.el;
			this.simpleView = new PeriodicTable.SimpleView({ el: wp });
			this.temperatureView = new PeriodicTable.TemperatureView({ el: wp });
			this.classesView = new PeriodicTable.ClassesView({ el: wp });
			this.thematicsView = new PeriodicTable.ThematicsView({ el: wp });
			this.configurationView = new PeriodicTable.ConfigurationView({ el: wp });
			this.trainingView = new PeriodicTable.TrainingView({ el: wp });

			var fo = document.getElementById('approfondimento');
			this.focusView = new PeriodicTable.FocusView({ el: fo });

			this.audio.play = function(name) {
				var a = document.getElementById('s_'+name);
				if (a!=null) {
					if (a.currentTime!=0) a.currentTime=0;
					a.play();
				}
			};

			this.labels = wp.querySelectorAll('#labelbox .label');

			this.listenTo(this.zElements,'change:active',this.onElementTap);

			if (!(TPCONF.package && BrowserInfo.isiOS) && !(TPCONF.package && BrowserInfo.isAndroid)) {
				$(window).resize(function() { view.zoomToFit(); });
			}
			this.zoomToFit();
		},

		load: function() {
			var dfr = $.Deferred();
			var view = this;
			if (TPCONF.search_update) {
				zElements.fetch().done(function() {
					view.zElements.each(function(model) {
						var id = model.get('number');
						var zeView = new PeriodicTable.elementView({ el: document.getElementById('el'+id), model: model });
					});
					view.saveCollection(zElements);
					view.onReady();
					dfr.resolve();
				}).fail(function(e){
					return view.loadFromLocal();
				});
			} else {
				return view.loadFromLocal();
			}

			return dfr.promise();
		},

		loadFromLocal: function() {
			var dfr = $.Deferred();
			var view = this;
			var zElements = this.zElements;
			var obj = view.loadCollection();
			for (var k = 0; k < obj.length; k++) {
				var el = obj[k];
				for (var j in el) {
					if ((el[j] = unescape(el[j])) != null && isNumeric(el[j])) {
						el[j] = parseFloat(el[j].replace(/,/,'.'));
					} else if (el[j] == "true") {
						el[j] = true;
					} else if (el[j] == "false") {
						el[j] = false;
					}
				}
				var model = new PeriodicTable.element(el);
				var id = model.get('number');
				var zeView = new PeriodicTable.elementView({ el: document.getElementById('el'+id), model: model });
				zElements.add(model);
			}
			view.onReady();
			dfr.resolve();
			return dfr.promise();
		},

		saveCollection: function(c) {
			var obj = c.toJSON();
			for (var k in obj) {
				delete obj[k]['elementView'];
				for (var j in obj[k]) {
					obj[k][j] = escape(obj[k][j]);
				}
			}
			var value = JSON.stringify(obj)+"";
			if (Modernizr.localstorage) {
				try {
					localStorage.setItem(this.cookieName, value);
				} catch(e) {
					console.log('quota error');
				}
			} else {
				console.log('only remote');
			}
		},

		loadCollection: function() {
			return saved_json;
		},

		onReady: function() {
			this.allElementsData = this.$('.elemento').children();
			Backbone.trigger('periodictable:ready');
			this.$el.css({ 'visibility':'visible' }).removeClass('loading');
		},

		onElementTap: function(model) {
			if (this.activeElement) {
				this.activeElement.get('elementView').deSelect();
			}
			if (model.get('active')) {
				this.activeElement = model;
				model.get('elementView').select();
			} else {
				this.activeElement = null;
			}
		},

		getActiveElement: function() {
			return this.activeElement;
		},

		zoomToFit: function() {
			var aw = window.innerWidth;
			var ah = window.innerHeight;
			var ww = this.$el.width();
			var wh = this.$el.height();
			var perw = aw / ww;
			var perh = ah / wh;
			if (perw < perh) {
				zoom = perw;
			} else {
				zoom = perh;
			}
			var translate = ((aw - ww * zoom) * 0.5) / zoom;
			this.$el.css({'transform': 'scale(' + zoom + ') translate(' + translate + 'px,0px)'});
			this.zoom = zoom;
		},

		switchColours: function(t) {
			switch (t) {
				case 'default':
					$('.elemento').removeClass('solido liquido gassoso sconosciuto schiarita fondoscuro metalg semimetalg alogenig metalcg metaldg attinidig postmetalg gasg nonmetalg metalg lantanidig importance0 importance1 importance2 importance3 importance4 importance5 importance6 importance7 importance8 importance9 importance10 noimportance nonconsiderare showGroup');
					break;
			}
		},

		datesToShow: function(toShow, toHide) {
			var view = this;

			if (toShow && toShow.length>0) {
				var goods = view.allElementsData.filter(toShow[0]);
				for (var i=1; i<toShow.length; i++) {
					var g = view.allElementsData.filter(toShow[i]);
					goods = goods.add(g);
				}
				goods.not(':visible').show();
				view.allElementsData.filter(':visible').not(goods).hide();
			}
		},

		modalToShow: function(name) {
			this.$('#tabella_ing, #ce_ing, #temperatura_ing').hide();
			this.$('#'+name).show();
		},

		toggleMenu: function(ev) {
			$('#labelbox').toggleClass('open');
		},

		switchMenu: function(ev) {
			ev.stopPropagation();
			ev.preventDefault();
			if (this.focusView && this.focusView.isOpening) return false;
			var et = ev.currentTarget;
			var cl = et.classList;
			if (!cl.contains('active')) {
				var labels = this.labels;
				if (labels) {
					for (var i = 0; i < labels.length; i++) {
						labels[i].classList.remove('active');
					}
				}
				cl.add('active');
				var id = et.id;
				var rel = et.getAttribute('rel');
				this[rel]();
				Backbone.trigger('mode:change', rel);
				periodicTableView.audio.play("scheda");
				document.body.setAttribute('class', 'mode_'+rel);
			} else {
				document.body.classList.remove('showCredits');
				if (this.currentView && this.currentView.reset) this.currentView.reset();
			}
		},

		deActiveViews: function() {
			if (this.currentView) {
				this.currentView.setActive(false);
				this.currentView = null;
			}
		},

		simple: function() {
			this.deActiveViews();
			periodicTableView.simpleView.setActive(true);
			this.currentView = periodicTableView.simpleView;
		},

		temperature: function() {
			this.deActiveViews();
			periodicTableView.temperatureView.setActive(true);
			this.currentView = periodicTableView.temperatureView;
		},

		classes: function() {
			this.deActiveViews();
			periodicTableView.classesView.setActive(true);
			this.currentView = periodicTableView.classesView;
		},

		thematics: function() {
			this.deActiveViews();
			periodicTableView.thematicsView.setActive(true);
			this.currentView = periodicTableView.thematicsView;
		},

		training: function() {
			this.deActiveViews();
			periodicTableView.trainingView.setActive(true);
			this.currentView = periodicTableView.trainingView;
		},

		configuration: function() {
			this.deActiveViews();
			periodicTableView.configurationView.setActive(true);
			this.currentView = periodicTableView.configurationView;
		},

		toggleCredits: function() {
			var body = document.body;
			var cl = body.classList;
			if (!cl.contains('showCredits')) {
				Backbone.trigger("credits:open");
				cl.add('showCredits');
			} else {
				Backbone.trigger("credits:close");
				cl.remove('showCredits');
			}
		},

		onPlusOpen: function() {
		},

		onPlusClose: function() {
		},

		setMode: function(name) {
			$(this.el.querySelector('#labelbox .label[rel="'+name+'"]')).trigger('click');
		},

		setZelement: function(number) {
			var num = parseInt(number);
			var models = this.zElements.where({number : num});
			if (models.length>0) models[0].set('active', true);
		},

		setTemperature: function(number) {
			var num = parseInt(number);
			this.$('#temp_esattak').val(num).trigger('change');
		},

		setClass: function(rel) {
			this.$('.p_classi span[rel="'+rel+'"]').trigger('click');
		},

		setTheme: function(rel) {
			this.$('#tematiche_box li[rel="'+rel+'"]').trigger('click');
		},

		setPlus: function(rel) {
			this.$('#menu_approfondimento .plus').trigger('click');
		},

		setCredits: function() {
			this.$('#creditslink').trigger('click');
		},

		manageExternalLink: function(ev) {
			if (navigator.targetsystem) {
				ev.stopPropagation();
				ev.preventDefault();
				navigator.targetsystem.open(ev.currentTarget.href);
    			return false;
			}
		},
	});

}());
