(function() {
	'use strict';

	PeriodicTable.TemperatureView = Backbone.View.extend({

		events: {
			"click .cerchioc": "manageDisplayes",
			"click .cerchiok": "manageDisplayes",
			"mousedown #cerchi input": "focusInput",
			"touchstart #cerchi input": "focusInput",
			"change #cerchi input": "manageTextarea",
			"touchstart #dragger": "manageSlider",
			"touchmove": "moveSlider",
			"touchend": "stopSlider",
			"mousedown #dragger": "manageSlider",
			"mousemove": "moveSlider",
			"mouseup": "stopSlider"
		},

		/*** attributes ***/
		isActive: false,
		allElements: [],
		circles: null,
		displayes: {},
		using_c: true,
		currentVal: 273,
		tool_width: 500,
		init_delta: 0,
		draglimit: 0,
		temperature_max: 6000,
		isChanging: false,
		lastChange: 0,
		changeElementTemperatureTimeout: null,
		waitBeforeChange: 0,
		panel: null,
		ingt1: null,

		themes: {
			gas: 'orange',
			liquid: 'blue',
			solid: 'green',
		},

		initialize: function() {
			this.allElements = $('.elemento').get();
			this.panel = document.getElementById('approfondimento');
			this.ingt1 = document.getElementById('ingt1');
			this.circles = this.$('#cerchi');
			this.circles.children().first().next().addClass('cerchioact');
			this.displayes['cerchioc'] = $('.cerchioc')[0] || null;
			this.displayes['cerchiok'] = $('.cerchiok')[0] || null;
			//this.displayes['cerchioc_small'] = $('.cerchioc_small')[0];
			//this.displayes['cerchiok_small'] = $('.cerchiok_small')[0];
			this.inputC = $('#temp_esattac')[0];
			this.inputK = $('#temp_esattak')[0];
			this.redLine = $("#linea_rossa");
			this.tool_width = this.redLine[0].clientWidth;
			this.draggerLimit = $('#dragger_limit');
			this.slider_width = this.draggerLimit[0].clientWidth;
			this.dragger = $('#cerchi');
			this.themes = {
				gas: $('#theme-gas').css('background-color'),
				liquid: $('#theme-liquid').css('background-color'),
				solid: $('#theme-solid').css('background-color'),
			};

			if (BrowserInfo.isAndroid || BrowserInfo.isiOS) {
				this.waitBeforeChange = 500;
			};

			Backbone.on('focus:close', this.deSelect, this);
			Backbone.on('temperature:change', this.sincroAll, this);
			return this;
		},

		setActive: function(bool) {
			if (this.isActive!=bool) {
				this.isActive = bool;
				if (bool) this.active();
				else this.deactive();
				return this;
			}
		},

		active: function() {
			this.panel.classList.remove('app_ce');
			this.listenTo(periodicTableView.zElements,'change:active',this.onElementTap);
			this.sincroAll(true);
			periodicTableView.modalToShow('temperatura_ing');
			var activeEl = periodicTableView.getActiveElement();
			if (activeEl) this.onElementTap(activeEl);
		},

		deactive: function() {
			periodicTableView.switchColours('default');
			var elements = this.allElements;
			for (var k = 0; k < elements.length; k++) {
				elements[k].removeAttribute('style');
			}
			this.ingt1.removeAttribute('style');
			this.stopListening();
		},

		onElementTap: function(model) {
			if (this.isActive && model.get('active')) {
				document.body.classList.remove('showCredits');
				document.body.classList.add('mode2');
				periodicTableView.focusView.open('temperature',model);
				periodicTableView.audio.play("elemento");
			}
		},

		setTempClass: function(force) {
			clearTimeout(this.changeElementTemperatureTimeout);
			var view = this;			
			var time = ( new Date() ).valueOf();
			
			if ( (time-view.lastChange>view.waitBeforeChange) || !view.isChanging) {
				view.changeElementTemperature();
				view.lastChange = time;
			} else {
				this.changeElementTemperatureTimeout = setTimeout(function() {
					view.changeElementTemperature();
					view.lastChange = time;
				}, view.waitBeforeChange-(time-view.lastChange))
			}
		},

		changeElementTemperature: function() {
			var view = this;		
			var themes = this.themes;
			var temp = this.currentVal;
			var models = periodicTableView.zElements.models;
			for (var i=0; i<models.length; i++) {
				var model = models[i];
				var ev = model.get('elementView');
				var tf = model.get('melting_temperature');
				var te = model.get('boiling_temperature');
				var cu = model.get('currentStatus');
				var cl = cu;
				
				if (temp<tf) { cl='solid'; }
				else if (temp<te) {	cl='liquid'; }
				else if (temp>=te) { cl='gas'; }
				else { cl='solid'; }
				ev.el.style['backgroundColor'] = themes[cl];
				model.set('currentStatus',cl);
			}
		},

		sincroTemperatures: function(){
			if (this.inputC) this.inputC.value = this.currentVal-273;
			if (this.inputK) this.inputK.value = this.currentVal;
		},

		manageDisplayes: function(ev) {
			if (ev.target.tagName == "INPUT") return true;
			ev.stopPropagation();
			ev.preventDefault();
			periodicTableView.audio.play("scheda");
			var t = ev.currentTarget;
			if (t.getAttribute('rel')=="C") this.switchDisplayes(1);
			else this.switchDisplayes(2);
			return false;
		},

		switchDisplayes: function(n){
			if (n==1){
				this.displayes['cerchiok'].className += ' cerchioact';
				this.displayes['cerchioc'].className = this.displayes['cerchioc'].className.replace(/cerchioact/g,'');
				this.using_c=true;
			} else {
				this.displayes['cerchioc'].className += ' cerchioact';
				this.displayes['cerchiok'].className = this.displayes['cerchiok'].className.replace(/cerchioact/g,'');
				this.using_c=false;
			}
		},

		manageTextarea: function(ev) {
			var t = ev.currentTarget;
			var nn = parseInt(t.value);
			this.using_c = t.getAttribute('rel')=="C";
			if (isNaN(nn)) nn=0;
			if (this.using_c) {
				if(nn>5727) nn=5727;
				else if(nn<-273) nn=-273;
				this.currentVal=nn+273;
			} else {
				if(nn>this.temperature_max) nn=this.temperature_max;
				else if(nn<0) nn=0;
				this.currentVal=nn;
			}
			Backbone.trigger("temperature:change", this.currentVal);
		},

		sincroAll: function(force) {
			if (this.isActive) {
				this.setTempClass(force);
				this.sincroTemperatures();
				this.moveDrag();
			}
		},

		manageSlider: function(ev) {
			var view = this;
			if (ev.target.tagName == "INPUT") return true;
			ev.stopPropagation();
			ev.preventDefault();
			if (view.isChanging && (ev.type=="mousedown" || ev.type=="touchstart")) return false;
			var elem_drag = ev.currentTarget;
			var limit_usare = this.slider_width;
			var initPX = ev.pageX || ev.originalEvent.changedTouches[0].pageX;
			var displayes = this.circles;
			this.init_delta = initPX - displayes.offset().left;
			this.draglimit = view.draggerLimit.offset().left;
			view.isChanging = true;
		},

		moveSlider: function(ev) {
			var view = this;
			if (!view.isChanging) return true;
			ev.stopPropagation();
			ev.preventDefault();
			view.isChanging = true;
			var bX = ev.pageX || ev.originalEvent.changedTouches[0].pageX;
			var draglimit = this.draglimit;
			var limit_usare = this.slider_width;
			var dd = this.init_delta;
			var move = (bX - draglimit - dd)/zoom;
			if ((move>=0)&&(move<=limit_usare)) {
				var div = move*100/limit_usare;
				var num = view.fromPercentToValue(div)+273;
				view.currentVal = num;
				Backbone.trigger("temperature:change", num);
			} else {
				if (move<0){
					view.currentVal = 0;
					Backbone.trigger("temperature:change", 0)
				} else {
					view.currentVal = view.temperature_max;
					Backbone.trigger("temperature:change", view.temperature_max)
				}
			}
		},

		stopSlider: function(ev) {
			if (!this.isChanging) return true;
			this.isChanging = false;
			Backbone.trigger("temperature:change", this.currentVal);
		},

		moveDrag: function() {
			var n = this.slider_width * this.currentVal / this.temperature_max;
			var color = 'rgba(202,0,0,1)';
			if (n<100) color = 'rgba(66,172,254,1)';
			else if (n<280) color = 'rgba(255,168,0,1)';
			this.redLine[0].style['width'] = n+'px';
			this.redLine[0].style['backgroundColor'] = color;
			var use3d = this.isChanging && !(BrowserInfo.isWebkit && !BrowserInfo.isAndroid) && Modernizr.csstransforms3d;
			var transform = 'matrix(1, 0, 0, 1, '+n+', 0)';
			if (use3d) transform = 'matrix3d(1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, '+n+', 0, 0, 1)';
			var d = this.dragger[0];
			d.style[window['CSS3TransformationKey']] = transform;
		},

		fromPercentToValue: function(limit) {
			var num = (((limit*this.temperature_max/100)-273)|0);
			if (num>5727) num = 5727;
			if (num<-273) num = -273;
			return num;
		},

		fromValueToPercent: function(l,dl,c) {
			var limit = (dl.width()-c.width())|0;
			var num = (((l+273)*limit/this.temperature_max)|0);
			if (num>limit) num = limit;
			if (num<0) num = 0;
			return num;
		},

		deSelect: function(model) {
			if (model && this.isActive) {
				model.set('active',false);
			}
		},

		focusInput: function(ev) {
			ev.stopPropagation();
			var input = ev.currentTarget;
			input.focus();
		},
	});

}());