(function() {
	'use strict';

	PeriodicTable.ThematicsView = Backbone.View.extend({
		/*** attributes ***/
		isActive: false,
		activeLabel: null,
		activeTheme: 'atomic_mass',

		events: {
			"click #tematiche LI": "switchThemes"
		},

		title: null,
		labels: null,
		themeName: null,
		themeData: null,

		initialize: function() {
			var box = document.getElementById('tematiche_box');
			var li = box.querySelector('li').getAttribute('rel');
			box.style['left'] = (this.$('#labelbox [rel="thematics"]').offset().left - this.$('#tematiche').offset().left) + 'px';
			var labels =this.labels = this.$('#tematiche li');
			if (labels.length<6) labels.css('width', '100%');
			this.themeName = document.getElementById('tema_nome');
			this.themeData = document.getElementById('tema_dato');
			this.title = document.querySelector('#tema_ing h2');
		},

		setActive: function(bool) {
			if (bool!=this.isActive) {
				this.isActive = bool;
				if (bool) this.active();
				else this.deactive();
				return this;
			}
		},

		active: function() {
			this.listenTo(periodicTableView.zElements,'change:active',this.onElementTap);
			this.themeName.innerHTML = '';
			this.themeData.innerHTML = '';
			this.openTheme(this.activeTheme);
			periodicTableView.focusView.close();
		},

		reset: function() {
			Backbone.trigger("theme:close");
			this.themeName.innerHTML = '';
			this.themeData.innerHTML = '';
			periodicTableView.zElements.each(function(model) {
				model.set('active', false)
			});
		},

		deactive: function() {
			periodicTableView.switchColours('default');
			this.stopListening();
		},

		onElementTap: function(model) {
			if (this.isActive && model.get('active')) {
				periodicTableView.audio.play("elemento");
				this.activeElement = model;
				this.selectElement(model);
				return false;
			}
		},

		selectElement: function(model) {
			var name = model.get('name');
			var data = model.get(this.activeTheme);
			this.themeName.innerHTML = name;
			this.themeData.innerHTML = data;
		},

		openTheme: function(cl) {
			this.activeTheme = cl;
			this.activeLabel = null;
			var labels = this.labels;
			for (var k = 0; k < labels.length; k++) {
				var label = labels[k];
				var rel = label.getAttribute('rel');
				if (rel == cl) {
					label.classList.add('tematicaact');
					this.activeLabel = label;
				} else {
					label.classList.remove('tematicaact');
				}
			}
			this.selectTheme(cl);
			var activeEl = periodicTableView.getActiveElement();
			if (activeEl) this.selectElement(activeEl);
		},

		switchThemes: function(ev) {
			periodicTableView.audio.play("scheda");
			periodicTableView.switchColours('default');
			var cl = ev.currentTarget.getAttribute('rel');
			this.openTheme(cl);
			return false;
		},

		selectTheme: function(cl) {
			var id = cl;
			Backbone.trigger("theme:open", cl);
			var header = this.activeLabel.textContent;
			this.incrementalColours(cl);
			switch (cl) {
				case 'melting_temperature': 
					header+=' (K)';
					break;
				case 'boiling_temperature': 
					header+=' (K)';
					break;
				case 'atomic_radius':
					header+=' (pm)';
					break;
				case 'first_ionisation_energy':
					header+=' (kJ/mol)';
					break;
				case 'electronegativity':
					header+=' (secondo Pauling)';
					break;
				case 'electronic_affinity':
					header+=' (eV)';
					break;
			}

			this.title.innerHTML = header;
			var elements = periodicTableView.zElements.toArray();
			for (var k = 0; k < elements.length; k++) {
				var model = elements[k];
				var ev = model.get('elementView');
				var txt = model.get(cl) + '';
				txt = txt.replace("Noto fin dall’antichità",'Antichità').replace("Noto fin dall'antichità",'Antichità');
				if (txt.length>10) txt = txt.substr(0,6)+'...';
				txt = txt.replace(' g/mL','<span style="font-size: 8px">&thinsp;g/mL</span>');
				txt = txt.replace(' g/L','<span style="font-size: 8px">&thinsp;g/L</span>');
				var b = ev.el.querySelector('.tema');
				b.innerHTML = txt;
			};
		},

		incrementalColours: function(cl) {
			var elements = periodicTableView.zElements.toArray();
			for (var k = 0; k < elements.length; k++) {
				var el = elements[k].get('elementView').el;
				var list = el.classList;
				list.remove('importance0');
				list.remove('importance1');
				list.remove('importance2');
				list.remove('importance3');
				list.remove('importance4');
				list.remove('importance5');
				list.remove('importance6');
				list.remove('importance7');
				list.remove('importance8');
				list.remove('importance9');
				list.remove('importance10');
				list.remove('noimportance');
			}
			switch (cl) {
				case 'standard_physical_state':
					periodicTableView.zElements.each(function(model) {
						var state = model.get('standard_physical_state');
						var cl = '';
						switch (state) {
							case 'Solido': 
								cl = 'solido';
								break;
							case 'Liquido': 
								cl = 'liquido';
								break;
							case 'Gas': 
								cl = 'gassoso';
								break;
						}
						model.get('elementView').setClass(cl,'');
					});
					break;
				case 'electronic_configuration':
					break;
				case 'discovery_year':
					break;
				case 'oxidation_numbers':
					break;
				default:
					var ar = [];
					var arModel = [];
					var max = 0;
					var min = 9999999999999;
					for (var k = 0; k < elements.length; k++) {
						var model = elements[k];
						var val = parseFloat((model.get(cl)+'').replace('[','').replace('Noto fin dall’antichità','0').replace(',','.').replace(']',''));
						if (val>max) max = val;
						if (val<min) min = val;
						if (isNaN(val)) val = -1;
						ar[model.get('number')] = val;
						arModel[model.get('number')] = model;
					};
					ar = sortAssoc(ar);
					var dif = (max-min)/10;
					for (var sKey in ar){
						var m = arModel[sKey];
						var num = Math.floor(ar[sKey]/dif);
						var imp = '';
						if (num>10) num=10;
						if (num>=0) imp = 'importance'+num;
						else imp = 'noimportance';
						m.get('elementView').setClass(imp,' ');
					}
					break;
			}
		},
	});

}());